/**
	$Id: MultiProtocolRequestHandler.java 5823 2011-10-24 13:02:46Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.jobagent.handlers;

import java.net.InetAddress;
import java.net.Socket;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.clients.jobagent.Agent;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.jobagent.RequestHandler;

/**
	Request handler implementation that forwards all requests to a registered 
	protocol-specific handler. All unknown protocols are forwarded to a
	default request handler. Protocols and their handlers are 
	registered with {@link #registerProtocols(RequestHandler, String[])}.
	An incoming request should be of the form: protocol://additional-info.
	Requests for protocols that doesn't have a registered handler and requests
	that doesn't follow the above form are forwarded to the default handler.

	@author Nicklas
	@since 2.16
	@base.modified $Date: 2011-10-24 15:02:46 +0200 (Mon, 24 Oct 2011) $
*/
public class MultiProtocolRequestHandler
	implements RequestHandler
{

	/**
		Logger.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.jobagent.MultiProtocolRequestHandler");

	private final Agent agent;
	private final RequestHandler defaultHandler;
	private final Map<String, RequestHandler> handlers;
	private boolean isClosed;
	
	/**
		Create a new multi-protocol request handler.
		@param defaultHandler The default handler that should
			receive all requests that hasn't a registered specific
			handler
	 */
	public MultiProtocolRequestHandler(Agent agent, RequestHandler defaultHandler)
	{
		this.agent = agent;
		this.defaultHandler = defaultHandler;
		this.handlers = Collections.synchronizedMap(new HashMap<String, RequestHandler>());
		this.isClosed = false;
	}
	
	/*
		From the RequestHandler interface
		---------------------------------
	*/
	@Override
	public String handleCmd(Socket socket, String cmd)
	{
		if (isClosed()) return "FAILED Handler is closed";
		
		InetAddress remote = socket.getInetAddress();
		if (log.isInfoEnabled())
		{
			log.info("Incoming request from '" + remote + "': " + cmd);
		}
		
		// Get the protocol
		String protocol = cmd;
		if (cmd != null)
		{
			int index = cmd.indexOf("://");
			if (index > 0) protocol = cmd.substring(0, index);
		}
		
		// ...and it's handler
		RequestHandler handler = handlers.get(protocol);
		if (handler == null) handler = defaultHandler;
		
		String answer = null;
		if (!agent.isAllowedControl(remote, protocol))
		{
			if (log.isDebugEnabled()) 
			{
				log.debug("Permission denied from '" + remote + "': " + cmd);
			}
			answer = "FAILED Permission denied: cmd=" + cmd + "; host=" + remote;
		}
		else if (handler == null)
		{
			if (log.isDebugEnabled()) 
			{
				log.debug("No handler for protocol: " + protocol);
			}
			answer = "FAILED Unknown command: " + cmd;
		}
		else
		{
			if (log.isDebugEnabled())
			{
				log.debug("Sending request to: " + handler);
			}
			answer = handler.handleCmd(socket, cmd);
		}
		if (log.isInfoEnabled())
		{
			log.info("Returning '" + Values.getString(answer).replaceAll("\n", "\\\\n") + "' for command '" + cmd +"' to " + remote);
		}
		return answer;
	}
	// ---------------------------------
	
	/**
		Register a handler for one or more protocols. Ignored if this
		request handler has been closed.
		
		@param handler The handler
		@param protocols An array with protocol names
	*/
	public void registerProtocols(RequestHandler handler, String... protocols)
	{
		if (isClosed()) return;
		for (String protocol : protocols)
		{
			handlers.put(protocol, handler);
		}
	}
	
	/**
		Unregister a handler for one or more protocols. Ignored if this
		request handler has been closed.
		@param protocols An array with protocol names
		@since 2.16
	*/
	public void unregisterProtocols(String... protocols)
	{
		if (isClosed()) return;
		for (String protocol : protocols)
		{
			RequestHandler handler = handlers.remove(protocol);
			if (handler instanceof CustomRequestHandler)
			{
				((CustomRequestHandler)handler).close();
			}
		}
	}

	/**
		Close this handler. It can't be used again.
	*/
	public void close()
	{
		if (isClosed()) return;
		isClosed = true;
		for (RequestHandler handler : handlers.values())
		{
			if (handler instanceof CustomRequestHandler)
			{
				((CustomRequestHandler)handler).close();
			}
		}
		handlers.clear();
	}

	
	/**
		If the request handler is closed, it will igore all method calls.
	*/
	public boolean isClosed()
	{
		return isClosed;
	}
	
}
