/**
	$Id: PlainTextTemplate.java 5843 2011-11-01 14:46:08Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.List;
import java.util.regex.Pattern;

import net.sf.basedb.core.Item;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.formatter.Formatter;
import net.sf.basedb.util.formatter.PrefixSuffixFormatter;

/**
 	A plain text template that exports data in tab-delimited format.
 	 
	@author nicklas
	@version 2.2
	@base.modified $Date: 2011-11-01 15:46:08 +0100 (Tue, 01 Nov 2011) $
*/
public class PlainTextTemplate
	implements ExportTemplate
{

	private final Writer exportStream;
	private final List<ExportedProperty> properties;
	private int colNum;
	
	/**
		Create a new template.
		@param exportStream The stream to write to
		@param properties All exported properties
		@param itemType The type of items we are exporting
	*/
	public PlainTextTemplate(Writer exportStream, List<ExportedProperty> properties, Item itemType)
	{
		this.exportStream = exportStream;
		this.properties = properties;
	}
	
	/**
		@return text/plain
	*/
	public String getMimeType()
	{
		return "text/plain";
	}

	/**
		@return Always "csv"
		@since 3.0
	*/
	@Override
	public String getDefaultFileExtension()
	{
		return "csv";
	}
	
	/**
		Do nothing.
	*/
	public void beginDocument() 
		throws IOException
	{}

	/**
		Write the headers on a single line. 
	*/
	public void writeHeaders() 
		throws IOException
	{
		for (ExportedProperty ep : properties)
		{
			exportStream.write(escape(ep.title));
			exportStream.write("\t");
		}
		exportStream.write("\n");
	}

	/**
		Reset column number.
	*/
	public void beginItem(int id) 
		throws IOException
	{
		colNum = 0;
	}

	/**
		Join the annotation values into a single string with a comma.
	*/
	@SuppressWarnings("unchecked")
	public void writeAnnotations(ExportedProperty ep, List<?> values, String unit)
		throws IOException
	{
		if (colNum > 0) exportStream.write("\t");
		colNum++;
		Formatter f = ep.formatter;
		if (unit != null) f = new PrefixSuffixFormatter("", f, PrefixSuffixFormatter.NBSP + unit);
		exportStream.write(escape(Values.getString(values, ",", true, f)));
	}

	/**
		Write the data object.
	*/
	@SuppressWarnings("unchecked")
	public void writeProperty(ExportedProperty ep, Object data)
		throws IOException
	{
		if (colNum > 0) exportStream.write("\t");
		colNum++;
		exportStream.write(escape(ep.formatter.format(data)));
	}
	
	/**
	 	Writes a collection of values.
	 	@since 2.12
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void writeCollection(ExportedProperty ep, Collection<?> values)
			throws IOException
	{
		if (colNum > 0) exportStream.write("\t");
		colNum++;		
		exportStream.write(escape(Values.getString(values, ",", true, ep.formatter)));
	}

	/**
		Write a newline.
	*/
	public void endItem() 
		throws IOException
	{
		exportStream.write("\n");
	}

	/**
		Do nothing.
	*/
	public void endDocument() 
		throws IOException
	{}

	
	private static final Pattern UNSAFE = Pattern.compile("(\\r?\\n\\r?|\\t)");
	/**
		Replace newlines and tabs with a space. Null values are converted to empty strings.
	*/
	private String escape(String data)
	{
		if (data == null) 
		{
			return "";
		}
		else
		{
			return UNSAFE.matcher(data).replaceAll(" ");
		}
	}
}
