/*
	$Id: ArraySlide.java 5642 2011-05-26 07:37:32Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ArraySlideData;
import net.sf.basedb.core.data.PhysicalBioAssayData;

import java.util.Date;
import java.util.Set;
import java.util.HashSet;

/**
	This class represents a physical microarray slide.
	
 	@author Nicklas, Enell
 	@version 2.0
*/
public class ArraySlide 
	extends AnnotatedItem<ArraySlideData>
	implements Registered
{
	/**
		The type of item represented by this class.
		
		@see Item#ARRAYSLIDE
		@see #getType()
	*/
	public static final Item TYPE = Item.ARRAYSLIDE;

	/**
		The maximum length of the barcode that can be stored in the
		database. Check the length against this value before calling the
		{@link #setBarcode(String) setBarcode} method to avoid exceptions.
	*/
	public static final int MAX_BARCODE_LENGTH = ArraySlideData.MAX_BARCODE_LENGTH;
	
	/**
		Create a new <code>ArraySlide</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param arrayBatch The <code>ArrayBatch</code> this slide belongs to
		@return The new <code>ArraySlide</code> item
		@throws PermissionDeniedException If the logged in user doesn't
			have use permission for the array batch
		@throws BaseException If there is another error
	*/
	public static ArraySlide getNew(DbControl dc, ArrayBatch arrayBatch)
		throws PermissionDeniedException, BaseException
	{
		ArraySlide as = dc.newItem(ArraySlide.class);
		as.setName("New array slide");
		as.setArrayBatch(arrayBatch);
		as.getData().setEntryDate(new Date());
		return as;
	}

	/**
		Get a <code>ArraySlide</code> object when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>ArraySlide</code> item
		@throws ItemNotFoundException If an item with the specified ID is
			not found
		@throws PermissionDeniedException If the logged in user doesn't
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static ArraySlide getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		ArraySlide as = dc.loadItem(ArraySlide.class, id);
		if (as == null) throw new ItemNotFoundException("ArraySlide[id=" + id + "]");
		return as;
	}
	
	/**
		Checks if a slide with the given barcode already exists in the database. The
		check is done on ALL slides, even on those not normally accessible by the
		logged in user.
		
		@param dc The DbControl to use for database access
		@param barcode The barcode to check
		@return TRUE if the barcode exists, FALSE if not or if the argument is NULL
		@since 2.7
	*/
	public static boolean barcodeExists(DbControl dc, String barcode)
	{
		if (barcode == null) return false;
		barcode = barcode.trim();
		org.hibernate.Session session = dc.getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ARRAYSLIDES_WITH_BARCODE", "count(*)");
		/*
			SELECT count(*)
			FROM ArraySlideData ars
			WHERE ars.barcode = :barcode
		*/
		query.setString("barcode", barcode);
		return HibernateUtil.loadData(Long.class, query) > 0;
	}
	
	/**
		Get a {@link ItemQuery} object configured
		to retrieve array slides.
		
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<ArraySlide> getQuery() 
	{
		return new ItemQuery<ArraySlide>(ArraySlide.class);
	}

	ArraySlide(ArraySlideData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
 	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		Get the array batch.
	*/
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		try
		{
			annotatable.add(getArrayBatch());
		}
		catch (PermissionDeniedException ex)
		{}
		return annotatable;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	@Override
	public boolean isUsed() 
		throws BaseException
	{
		return getData().getPhysicalBioAssay() != null || super.isUsed();
	}
	/**
		Get the bioassay that was used on this plate.
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
		throws BaseException
	{
		Set<ItemProxy> using = super.getUsingItems();
		PhysicalBioAssay item = getPhysicalBioAssay();
		if (item != null) using.add(new ItemProxy(item.getId(), item.getType()));
		return using;
	}
	// -------------------------------------------

	/**
		Get the index of this slide in the batch.
	
		@return The index of the slide
	*/
	public int getBatchIndex()
	{
		return getData().getBatchIndex();
	}

	/**
		Set the index of this slide in the batch.
		
		@param batchIndex The index of the slide in the batch
		@throws PermissionDeniedException This exception is thrown if the
			logged in user doesn't have write
			permission.
	*/
	public void setBatchIndex(int batchIndex) 
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setBatchIndex(batchIndex);
	}
	
	/**
		Get the barcode of the slide.
		@return A string with the barcode sequence
	*/
	public String getBarcode()
	{
		return getData().getBarcode();
	}

	/**
		Set the barcode for the slide. The value
		can be null and must not be longer than the value specified by
		the {@link #MAX_BARCODE_LENGTH} constant.
		
		@param barcode The new barcode for the slide
		@throws PermissionDeniedException If the logged in user doesn't 
			have write permission
		@throws InvalidDataException This exception is thrown if the barcode
			is longer than {@link #MAX_BARCODE_LENGTH}
	*/
	public void setBarcode(String barcode) 
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setBarcode(StringUtil.setNullableString(barcode, "Barcode", MAX_BARCODE_LENGTH));
	}
	
	/**
		Check if the slide is flagged as destroyed or not. A
		destroyed slide may not be connected to a hybridization.

		@return TRUE if the slide is destroyed, FALSE otherwise
	*/
	public boolean isDestroyed()
	{
		return getData().isDestroyed();
	}

	/**
		Set if the slide should be flagged as destroyed. A
		destroyed slide may not be connected to a hybridization.
		
		@param destroyed true if the slide should be flagged as destroyed,
			false otherwise
		@throws PermissionDeniedException If the logged in user doesn't 
			have write permission
	*/
	public void setDestroyed(boolean destroyed) 
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setDestroyed(destroyed);
	}
	
	/**
		Get the {@link ArrayBatch} item to which this slide belongs.
		
		@return The <code>ArrayBatch</code> item
		@throws PermissionDeniedException If the logged in user doesn't have read 
			permission for the array batch
		@throws BaseException If there is another error
	*/
	public ArrayBatch getArrayBatch()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(ArrayBatch.class, getData().getArrayBatch());
	}
	
	/**
		Set the {@link ArrayBatch} this arrayslide belongs to.
		
		@param arrayBatch the arrayBatch to be set. Cannot be null.
		@throws InvalidDataException If the arrayBatch is null
		@throws PermissionDeniedException If the logged in user doesn't have 
			use permission for the batch
	*/
	private void setArrayBatch(ArrayBatch arrayBatch) 
		throws InvalidDataException, PermissionDeniedException
	{
		if (arrayBatch == null) throw new InvalidUseOfNullException("arrayBatch");
		arrayBatch.checkPermission(Permission.USE);
		getData().setArrayBatch(arrayBatch.getData());
	}
	
	/**
		Get the {@link PhysicalBioAssay} item which was used on this slide. To set, 
		use the {@link PhysicalBioAssay#setArraySlide(ArraySlide)} method.

		@return The <code>PhysicalBioAssay</code> item, or null
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have read permission to the item
		@throws BaseException If there is another error
		@since 3.0
	*/
	public PhysicalBioAssay getPhysicalBioAssay()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(PhysicalBioAssay.class, getData().getPhysicalBioAssay());
	}
	
	/**
		Checks if this array slide is used by a different bioassay
		than the given one.
		@param pba The bioassay to check, or null to check if the slide is used
			by any bioassay
		@return TRUE if the slide is used by a different bioassay, or
			FALSE if it is used by the given bioassay, or not used at all
		@since 2.8.3
	*/
	public boolean isUsedByOther(PhysicalBioAssay pba)
	{
		PhysicalBioAssayData usedBy = getData().getPhysicalBioAssay();
		if (usedBy == null) return false;
		if (pba == null) return true;
		return !usedBy.equals(pba.getData());
	}
	
}
