/*
	$Id: BioMaterialEvent.java 5988 2012-03-02 14:01:45Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.query.Hql;

import net.sf.basedb.core.data.BioMaterialData;
import net.sf.basedb.core.data.BioMaterialEventData;
import net.sf.basedb.core.data.BioMaterialEventSourceData;
import net.sf.basedb.core.data.HardwareData;
import net.sf.basedb.core.data.MeasuredBioMaterialData;
import net.sf.basedb.core.data.ProtocolData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.data.SharedData;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;

/**
	This class represents an event in the life of a {@link MeasuredBioMaterial}. 
	There are three main types of events: creation events,
	hybridization events and other events. A creation event is an event that
	produces a measured biomaterial from other biomaterials. A hybridization event 
	is an event that produces a hybridization from labeled extracts. Other events 
	are used to leave a note about something that affected an already existing biomaterial.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2012-03-02 15:01:45 +0100 (Fri, 02 Mar 2012) $
*/
public class BioMaterialEvent
	extends ChildItem<BioMaterialEventData>
	implements Registered
{

	/**
		The type of item represented by this class.
		@see Item#BIOMATERIALEVENT
		@see #getType()
	*/
	public static final Item TYPE = Item.BIOMATERIALEVENT;

	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#OTHER} type.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioMaterial The affected biomaterial
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
	*/
	public static BioMaterialEvent getNew(DbControl dc, MeasuredBioMaterial bioMaterial)
		throws BaseException
	{
		return getNew(dc, bioMaterial, null);
	}
	
	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#OTHER} type
		that is linked with a master {@link BioPlateEventParticipant}. If a master
		bioplate event is used it must be a new event that has not yet been saved to the
		database.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioMaterial The affected biomaterial
		@param master The master bioplate event or null
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
		@since 2.17
	*/
	public static BioMaterialEvent getNew(DbControl dc, MeasuredBioMaterial bioMaterial, BioPlateEventParticipant master)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setBioMaterial(bioMaterial);
		bme.getData().setEventType(Type.OTHER.getValue());
		if (master != null)
		{
			if (master.isInDatabase())
			{
				throw new PermissionDeniedException("Must use a new bioplate event: "+master);
			}
			master.getEvent().synchronizeProperties(bme.getData(), master.getData());
		}
		else
		{
			bme.getData().setEntryDate(new Date());
			org.hibernate.Session session = dc.getHibernateSession();
			SessionControl sc = dc.getSessionControl();
			bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		}
		return bme;
	}

	
	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#CREATION}
		type (creation event).
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioMaterial The biomaterial that was created.
		@param master The master bioplate event or null
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
		@since 2.17
	*/
	static BioMaterialEvent getNewCreationEvent(DbControl dc, MeasuredBioMaterial bioMaterial, BioPlateEventParticipant master)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setBioMaterial(bioMaterial);
		bme.getData().setEventType(Type.CREATION.getValue());
		if (master != null)
		{
			if (master.isInDatabase())
			{
				throw new PermissionDeniedException("Must use a new bioplate event: "+master);
			}
			master.getEvent().synchronizeProperties(bme.getData(), master.getData());
		}
		else
		{
			bme.getData().setEntryDate(new Date());
			org.hibernate.Session session = dc.getHibernateSession();
			SessionControl sc = dc.getSessionControl();
			bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		}
		return bme;
	}
	
	/**
		Create a new <code>BioMaterialEvent</code> of the {@link Type#BIOASSAY}
		type.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param physicalBioAssay The hybridization that was created.
		@param master The master bioplate event or null
		@return The new <code>BioMaterialEvent</code> item
		@throws BaseException If there is an error
		@since 2.17
	*/
	static BioMaterialEvent getNewBioAssayEvent(DbControl dc, PhysicalBioAssay physicalBioAssay, BioPlateEventParticipant master)
		throws BaseException
	{
		BioMaterialEvent bme = dc.newItem(BioMaterialEvent.class);
		bme.setPhysicalBioAssay(physicalBioAssay);
		bme.getData().setEventType(Type.BIOASSAY.getValue());
		if (master != null)
		{
			if (master.isInDatabase())
			{
				throw new PermissionDeniedException("Must use a new bioplate event: "+master);
			}
			master.getEvent().synchronizeProperties(bme.getData(), master.getData());
		}
		else
		{
			bme.getData().setEntryDate(new Date());
			org.hibernate.Session session = dc.getHibernateSession();
			SessionControl sc = dc.getSessionControl();
			bme.getData().setUser(HibernateUtil.loadData(session, UserData.class, sc.getLoggedInUserId()));
		}
		return bme;
	}


	/**
		Get a <code>BioMaterialEvent</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>BioMaterialEvent</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static BioMaterialEvent getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		BioMaterialEvent bme = dc.loadItem(BioMaterialEvent.class, id);
		if (bme == null) throw new ItemNotFoundException("BioMaterialEvent[id="+id+"]");
		return bme;
	}

	/**
		Get a query that returns events for a biomaterial. All events where the
		given biomaterial is either a source or a product are returned. 
	
		@param bioMaterial The biomaterial to retreive events for, null is not allowed
		@return An {@link ItemQuery} object
		@throws InvalidDataException If the <code>bioMaterial</code> is null
		@see MeasuredBioMaterial#getEvents()
	*/
	public static ItemQuery<BioMaterialEvent> getQuery(MeasuredBioMaterial bioMaterial)
		throws InvalidDataException
	{
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		// Create query, no runtime filter since read permission to a bioMaterial grants 
		// read permission to it's events (event other creation events)
		ItemQuery<BioMaterialEvent> query = 
			new ItemQuery<BioMaterialEvent>(BioMaterialEvent.class, null);
		query.setDistinct(true);  // Otherwise, the join to sources can result in one copy for each source
		query.joinPermanent(Hql.leftJoin("sources", "src"));
		query.restrictPermanent(
			Restrictions.or(
				Restrictions.eq(
					Hql.property("bioMaterial"), 
					Hql.entity(bioMaterial)
				),
				Restrictions.eq(
					Hql.index("src", null),
					Hql.entity(bioMaterial)
				)
			)
		);
		return query;
	}
	

	BioMaterialEvent(BioMaterialEventData bioMaterialEventData)
	{
		super(bioMaterialEventData);
	}
	
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Return used guantity to biomaterials on delete action.
	*/
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			setUsedQuantity(null);
		}
	}

	/**
		READ permission is granted and DELETE permissions is denied
		to evereyone for creation events. WRITE permission is denied
		if this event is associated with a plate event.
		Other permissions are always checked on the parent biomaterial 
		or hybridization.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		switch (getEventType())
		{
			case CREATION:
			{
				granted |= Permission.grant(Permission.READ);
				denied |= Permission.deny(Permission.DELETE);
				break;
			}
			case BIOASSAY:
			{
				granted |= Permission.grant(Permission.READ);
				denied |= Permission.deny(Permission.DELETE);
				break;
			}
			case OTHER:
			{
				break;
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+getEventType());
			}
		}
		// Deny write permission if there is a master bioplate event
		if (getData().getBioPlateEventParticipant() != null)
		{
			denied |= Permission.deny(Permission.WRITE);
		}
		super.initPermissions(granted, denied);
	}
	/**
		Set protocol and hardware from project default settings.
		@since 2.9
	*/
	@Override
	void setProjectDefaults(Project activeProject)
		throws BaseException
	{
		super.setProjectDefaults(activeProject);
		if (getEventType() == Type.OTHER || !hasPermission(Permission.WRITE)) return;
		
	
		DbControl dc = getDbControl();
		Subtypable createdItem = null;
		String defaultProtocolId = null;
		if (getEventType() == Type.BIOASSAY)
		{
			createdItem = getPhysicalBioAssay();
		}
		else
		{
			MeasuredBioMaterial bioMaterial = getBioMaterial();
			createdItem = bioMaterial;
			Item parentType = createdItem.getType();
			if (parentType == Item.SAMPLE)
			{
				defaultProtocolId = Protocol.SAMPLING;
			}
			else if (parentType == Item.EXTRACT)
			{
				defaultProtocolId = Protocol.EXTRACTION;
			}
		}
		
		if (!protocolHasBeenSet)
		{
			ItemSubtype protocolSubtype = 
				ItemSubtype.getRelatedSubtype(dc, createdItem, Item.PROTOCOL, SystemItems.getId(defaultProtocolId));
			if (protocolSubtype != null)
			{
				ProtocolData protocol = 
					(ProtocolData)activeProject.findDefaultData(Item.PROTOCOL, protocolSubtype.getData(), false);
				if (protocol != null)
				{
					getData().setProtocol(protocol);
					protocolHasBeenSet = true;
				}
			}
		}
		if (!hardwareHasBeenSet)
		{
			HardwareData hardware = 
				(HardwareData)activeProject.findDefaultRelatedData(dc, createdItem, Item.HARDWARE, false);
			if (hardware != null) 
			{
				getData().setHardware(hardware);
				hardwareHasBeenSet = true;
			}
		}
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	SharedData getSharedParent()
	{
		switch (getEventType())
		{
			case CREATION:
			{
				return getData().getBioMaterial();
			}
			case BIOASSAY:
			{
				return getData().getPhysicalBioAssay();
			}
			case OTHER:
			{
				return getData().getBioMaterial();
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+getEventType());
			}
		}
	}
	// -------------------------------------------

	/**
		Get the type of event.
		@see Type
	*/
	public Type getEventType()
	{
		return Type.fromValue(getData().getEventType());
	}
	
	/**
		Get the {@link MeasuredBioMaterial} that was affected or created by this event.
		@return A <code>MeasuredBioMaterial</code> item or null if this is 
			hybridization event
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the biomaterial
		@throws BaseException If there is another error
	*/
	public MeasuredBioMaterial getBioMaterial()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(MeasuredBioMaterial.class, getData().getBioMaterial());
	}
	/**
		Set the biomaterial that was affected or created by this event.
	*/
	private void setBioMaterial(MeasuredBioMaterial bioMaterial)
		throws PermissionDeniedException, InvalidDataException
	{
		// No write permission check since this is only called at object creation
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		bioMaterial.checkPermission(Permission.USE);
		getData().setBioMaterial((MeasuredBioMaterialData)bioMaterial.getData());
	}
	
	/**
		Get the {@link PhysicalBioAssay} that was created by this event.
		@return A <code>PhysicalBioAssay</code> item or null if this isn't a
			bioassay event
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the bioassay
		@throws BaseException If there is another error
	*/
	public PhysicalBioAssay getPhysicalBioAssay()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(PhysicalBioAssay.class, getData().getPhysicalBioAssay());
	}

	/**
		Set the hybidization that was created by this event.
	*/
	private void setPhysicalBioAssay(PhysicalBioAssay physicalBioAssay)
		throws PermissionDeniedException, InvalidDataException
	{
		// No write permission check since this is only called at object creation
		if (physicalBioAssay == null) throw new InvalidUseOfNullException("physicalBioAssay");
		physicalBioAssay.checkPermission(Permission.USE);
		getData().setPhysicalBioAssay(physicalBioAssay.getData());
	}

	/**
		Check if this event is related with a bioplate event.
		@since 2.17
	*/
	public boolean isPlateEvent()
	{
		return getData().getBioPlateEventParticipant() != null;
	}
	
	/**
		Get the bioplate event participant that is the master of this event.
		If this event has a master event, many properties such as event date,
		protocol, etc. must be set on the master event.
		
		@return A bioplate event participant or null
		@since 2.17
	*/
	public BioPlateEventParticipant getBioPlateEventParticipant()
	{
		return getDbControl().getItem(BioPlateEventParticipant.class, getData().getBioPlateEventParticipant());
	}


	/**
		Get the {@link User} that is responsible for this event.
		@return A <code>User</code> item 
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the user
		@throws BaseException If there is another error
	*/
	public User getUser()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(User.class, getData().getUser());
	}

	/**
		Get the {@link Protocol} describing the procedure used in this event.
		@return A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			read permission to the protocol
		@throws BaseException If there is another error
	*/
	public Protocol getProtocol()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Protocol.class, getData().getProtocol());
	}

	private boolean protocolHasBeenSet = false;
	/**
		Set the {@link Protocol} that describes the procedure used in this event.
		@param protocol A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event or use permission for the protocol
	*/
	public void setProtocol(Protocol protocol)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (protocol != null) protocol.checkPermission(Permission.USE);
		protocolHasBeenSet = true;
		getData().setProtocol(protocol == null ? null : protocol.getData());
	}
	
	/**
		Get the {@link Hardware} that was used in this event.
	 	@return A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have 
	 		read permission to the hardware
	 	@throws BaseException If there is another error
	*/
	public Hardware getHardware()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(Hardware.class, getData().getHardware());
	}
	
	private boolean hardwareHasBeenSet = false;
	/**
	 	Get the {@link Hardware} that was used in this event.
	 	@param hardware A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the hardware
	*/
	public void setHardware(Hardware hardware)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (hardware != null) hardware.checkPermission(Permission.USE);
		hardwareHasBeenSet = true;
		getData().setHardware(hardware == null ? null : hardware.getData());
	}

	/**
		Get the quantity that was used from the biomaterial by this event.
		If this is a creation event this value is the negative value of
		the original quantity of the created biomaterial.
	*/
	public Float getUsedQuantity()
	{
		return getData().getUsedQuantity();
	}
	/**
		Set the used quantity.
	 	@param usedQuantity The quantity to set. 
	 	@throws PermissionDeniedException If logged in user doesn't have write 
	 		permission on current item. 
	*/
	public void setUsedQuantity(Float usedQuantity)
		throws PermissionDeniedException
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		MeasuredBioMaterialData data = getData().getBioMaterial();
		Float used = getData().getUsedQuantity();
		if (used != null) 
		{
			MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, -used);
		}
		if (usedQuantity != null) MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), data, usedQuantity);
		getData().setUsedQuantity(usedQuantity);
	}

	/**
		Get the quantity that was used by this event for the specified biomaterial.
		@param bioMaterial A biomaterial object
	*/
	public Float getUsedQuantity(MeasuredBioMaterial bioMaterial)
	{
		if (bioMaterial.getData().equals(getData().getBioMaterial()))
		{
			return getUsedQuantity();
		}
		else
		{
			BioMaterialEventSourceData src = getData().getSources().get(bioMaterial.getData());
			return src == null ? null : src.getUsedQuantity();
		}
	}
	
	/**
		Get information about a biomaterial used as a source to this event.
		@param bioMaterial The biomaterial
		@return A BioMaterialEventSource object or null if the given biomaterial
			is not a source to the event
		@see #addSource(BioMaterial)
		@since 3.0
	*/
	public BioMaterialEventSource getEventSource(BioMaterial bioMaterial)
	{
		BioMaterialEventSourceData src = getData().getSources().get(bioMaterial.getData());
		return src == null ? null : new BioMaterialEventSource(getDbControl(), src); //(this);
	}
	
	/**
		Check if the given biomaterial has been used as a source in this
		event.
		@param bioMaterial The biomaterial to check
	*/
	public boolean isSource(BioMaterial bioMaterial)
	{
		return getData().getSources().containsKey(bioMaterial.getData());
	}
	
	/**
		Set a single parent item for this event. Any sources
		that are currently associated with it are removed and
		the used quantity is returned.
		
		@param parent The parent biomaterial, or null to remove all parents
		@since 3.0
	*/
	public BioMaterialEventSource setSource(BioMaterial parent)
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		if (parent != null) 
		{
			parent.checkPermission(Permission.USE);
			checkAllowedSource(parent, false);
		}
		
		clearSourcesExcept(parent);
		BioMaterialEventSource eventSource = null;
		if (parent != null) 
		{
			eventSource = addCheckedSource(parent);
		}
		return eventSource;
	}

	/**
		Add a biomaterial as a source to this event. If the given 
		biomaterial already is a source, the existing information is 
		returned. If this event already has sources the new source
		must be of the same type as the existing sources and the
		produced biomaterial.
		
		@param bioMaterial The biomaterial (null is not allowed)
		@return A BioMaterialEventSource object
		@since 3.0
	*/
	public BioMaterialEventSource addSource(BioMaterial bioMaterial)
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		bioMaterial.checkPermission(Permission.USE);
		
		checkAllowedSource(bioMaterial, true);
		return addCheckedSource(bioMaterial);
	}
	
	private BioMaterialEventSource addCheckedSource(BioMaterial bioMaterial)
	{
		BioMaterialData bmData = (BioMaterialData)bioMaterial.getData();
		
		Map<BioMaterialData, BioMaterialEventSourceData> sources = getData().getSources();
		
		BioMaterialEventSourceData src = sources.get(bmData);
		if (src == null)
		{
			src = new BioMaterialEventSourceData();
			src.setBioMaterial(bmData);
			src.setEvent(this.getData());
			sources.put(bmData, src);
		}

		if (getData().getBioMaterial() != null)
		{
			// If the product is a biomaterial we may need to set parent + parent type
			MeasuredBioMaterialData product = getData().getBioMaterial();
			if (sources.size() == 1)
			{
				// This is the first parent
				product.setParentType(bioMaterial.getType().getValue());
				product.setParent(bmData);
			}
			else
			{
				// Clear if we get more than one parent
				product.setParent(null);
			}
		}

		return new BioMaterialEventSource(getDbControl(), src);
	}

	/**
		Remove a biomaterial used as a source. Sources can only be specified if
		the produced biomaterial is a pooled biomaterial and this is a creation event.
		@param bioMaterial The biomaterial used as a source
		@throws PermissionDeniedException If the logged in user doesn't have 
			write permission for this event of use permission for the source
		@throws InvalidDataException If this is not a creation event or 
			if the produced biomaterial is not pooled or if the source
			biomaterial is of another type
		@throws BaseException If there is another error
		@since 3.0
	*/
	public void removeSource(BioMaterial bioMaterial)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		if (bioMaterial == null) throw new InvalidUseOfNullException("bioMaterial");
		BioMaterialData data = (BioMaterialData)bioMaterial.getData();
		Map<BioMaterialData, BioMaterialEventSourceData> sources = getData().getSources();
		BioMaterialEventSourceData src = sources.remove(data);
		
		if (sources.size() == 0)
		{
			if (getData().getBioMaterial() != null)
			{
				getData().getBioMaterial().setParentType(null);
			}
		}
		
		if (src == null) return;
		
		if (src.getId() != 0) getDbControl().getHibernateSession().delete(src);
		if (data instanceof MeasuredBioMaterialData)
		{
			Float used = src.getUsedQuantity();
			if (used != null)
			{
				MeasuredBioMaterial.updateRemainingQuantity(getDbControl(), (MeasuredBioMaterialData)data, -used);
			}
		}
	}

	/**
		Check if the biomaterial is allowed as a source.
	*/
	private void checkAllowedSource(BioMaterial source, boolean forAdd)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		Type eventType = getEventType();
		Item sourceType = source.getType();
		switch (eventType)
		{
			case OTHER:
			{
				throw new InvalidDataException("OTHER events can't have sources");
			}
			case BIOASSAY:
			{
				if (sourceType != Item.EXTRACT)
				{
					throw new InvalidDataException("The source for a physical bioassay must be an extract");
				}
				break;
			}
			case CREATION:
			{
				MeasuredBioMaterial product = getBioMaterial();
				Item productType = product.getType();
				Item parentType = product.getParentType();
				
				if (forAdd && parentType != null)
				{
					if (parentType != sourceType)
					{
						throw new InvalidDataException("Can't add '" + source.getName() + "' (" + sourceType + 
								") as parent since there is already a " + parentType + " parent");
					}
				}
				
				if (productType != sourceType)
				{
					if (productType == Item.EXTRACT && sourceType != Item.SAMPLE)
					{
						throw new InvalidDataException("The parent for an extract must be an extract or sample: " + source);
					}
					if (productType == Item.SAMPLE && sourceType != Item.BIOSOURCE)
					{
						throw new InvalidDataException("The parent for a sample must be a biosource or sample: " + source);
					}
				}
				
				if (source.equals(product))
				{
					throw new InvalidDataException("The parent biomaterial cannot be the same as the produced biomaterial: " + source);
				}
				break;
			}
			default:
			{
				throw new AssertionError("Invalid event type: "+eventType);
			}
		}
	}

	/**
		Clear the sources collection and return the used quantities to
		the source biomaterials.
		@since 3.0
	*/
	public void clearSources()
	{
		clearSourcesExcept(null);
	}

	/**
		Clear the sources collection and return the used quantities to
		the source biomaterials, except for the given biomaterial.
		@param A biomaterial that should already be a parent biomaterial
			or null to clear all sources
		@since 3.0.4
	*/
	private void clearSourcesExcept(BioMaterial parent)
	{
		DbControl dc = getDbControl();
		Map<BioMaterialData, BioMaterialEventSourceData> sources = getData().getSources();
		int parentId = parent == null ? 0 : parent.getId();
		
		Iterator<Map.Entry<BioMaterialData, BioMaterialEventSourceData>> it = sources.entrySet().iterator();
		while (it.hasNext())
		{
			Map.Entry<BioMaterialData, BioMaterialEventSourceData> entry = it.next();
			
			BioMaterialEventSourceData src = entry.getValue();
			BioMaterialData bioMaterial = entry.getKey();
			
			if (bioMaterial.getId() != parentId)
			{
				Float used = src.getUsedQuantity();
				if (used != null && bioMaterial instanceof MeasuredBioMaterialData) 
				{
					MeasuredBioMaterial.updateRemainingQuantity(dc, (MeasuredBioMaterialData)bioMaterial, -used);
				}
				dc.getHibernateSession().delete(src);
				it.remove();
			}
		}
		if (getData().getBioMaterial() != null)
		{
			getData().getBioMaterial().setParentType(parent == null ? null : parent.getType().getValue());
			getData().getBioMaterial().setParent(parent == null ? null : (BioMaterialData)parent.getData());
		}
	}

	
	/**
		Get a query that return all biomaterials used as sources
		for this event. To filter or sort on the used quantity or position columns
		use 'srcevt' as alias. For example to only return sources in position 2
		and to order the result by used quantity:
		<pre class="code">
query = event.getSources();
query.restrict(Expressions.gteq(
  Hql.property("srcevt", "position"), Expressions.integer(2))
);
query.order(Orders.asc(Hql.property("srcevt", "usedQuantity")));
</pre>
		<p>
		NOTE! The filtering and ordering on the "srcevt" columns is only
		available in BASE 2.6 and later.
		
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<? extends BioMaterial> getSources()
	{
		ItemQuery<? extends BioMaterial> query = null;
		MeasuredBioMaterialData bioMaterialData = getData().getBioMaterial();
		Item parentType = null;
		if (getEventType() == Type.BIOASSAY)
		{
			parentType = Item.EXTRACT;
		}
		else
		{
			Integer tmp = getData().getBioMaterial().getParentType();
			parentType = tmp != null ? Item.fromValue(tmp) : Item.fromDataObject(getData().getBioMaterial());
		}
		switch (parentType)
		{
			case EXTRACT:
			{
				query = Extract.getQuery();
				break;
			}
			case SAMPLE:
			{
				query = Sample.getQuery();
				break;
			}
			case BIOSOURCE:
			{
				query = BioSource.getQuery();
				break;
			}
			default:
			{
				throw new AssertionError("Invalid biomaterial type: "+parentType);
			}
		}
		
		query.joinPermanent(Hql.innerJoin("childCreationEvents", "srcevt"));
		query.joinPermanent(Hql.innerJoin("srcevt", "event", TYPE.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(TYPE.getAlias()),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Create a special query that return BioMaterialEventSource objects.
		The objects are not {@link BasicItem}:s but can be used to get information
		about and modify the source biomaterials used in the event. Note
		that the query may return objects that link to biomaterial that the
		logged in used doesn't have read access to.
		@return A special query
		@since 3.0
	*/
	public SpecialQuery<BioMaterialEventSource> getEventSources()
	{
		SpecialQuery<BioMaterialEventSource> query = 
			new SpecialQuery<BioMaterialEventSource>(null, "BioMaterialEventSourceData", "es", 
					new BioMaterialEventSource.BioMaterialEventSourceTransformer());
		query.restrictPermanent(Restrictions.eq(Hql.property("es", "event"), Hql.entity(this)));
		return query;
	}
	
	/**
		Utility method for finding all sources on a given bioassay
		position.
		@param position The position number (between 1 and {@link PhysicalBioAssay#getSize()}.
		@return A list (empty if no sources are found)
	*/
	public List<BioMaterialEventSource> getEventSources(DbControl dc, int position)
	{
		SpecialQuery<BioMaterialEventSource> posQuery = getEventSources();
		posQuery.restrict(Restrictions.eq(Hql.property("position"), Expressions.integer(position)));
		return posQuery.list(dc);
	}
	
	/**
		The maximum length of the comment about this event. Check the length
		against this value before calling the {@link #setComment(String)}
		method to avoid exceptions.
	*/
	public static final int MAX_COMMENT_LENGTH = BioMaterialEventData.MAX_COMMENT_LENGTH;
	/**
		Get the comment about this event.
		@return The comment
	*/
	public String getComment()
	{
		return getData().getComment();
	}
	/**
		Set the comment about this event. The value may be null, but mustn't
		be longer than the value specified by the {@link #MAX_COMMENT_LENGTH} constant.
		@param comment The new comment
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event
		@throws InvalidDataException If the comment is too long
	*/
	public void setComment(String comment)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		getData().setComment(StringUtil.setNullableString(comment, "comment", MAX_COMMENT_LENGTH));
	}
	
	/**
		Get the date this event was added to the database.
		@return A <code>Date</code> object
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}

	/**
		Get the date this event was done in the lab.
		@return A <code>Date</code> object, or null if not known
	*/
	public Date getEventDate()
	{
		return DateUtil.copy(getData().getEventDate());
	}
	/**
		Set the date this event was done in the lab.
		@param eventDate The date, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event
	*/
	public void setEventDate(Date eventDate)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setEventDate(DateUtil.setNullableDate(eventDate, "eventDate"));
	}

	
	/**
		The type of the event.
	*/
	public enum Type
	{
		/**
			An event of this type created a biomaterial.
		*/
		CREATION(1, "Creation"),

		/**
			An event of this type created a physical bioassay.
		*/
		BIOASSAY(2, "Bioassay creation"),
		
		/**
			An event of this type affected a single biomaterial in some way.
		*/
		OTHER(3, "Other");

		/**
			Maps the integer that is stored in the database.
		*/
		private static final Map<Integer, Type> valueMapping = new HashMap<Integer, Type>();
		
		static
		{
			for(Type type : Type.values())
			{
				Type t = valueMapping.put(type.getValue(), type);
				assert t == null : "Another BioMaterialEvent.Type with the value "+type.getValue()+" already exists";
			}
		}
		
		/**
			Get the <code>BioMaterialEvent.Type</code> object when you know the integer code.
			
			@param value The integer value.
			@return The type object or null if the value is unknown
		*/
		public static Type fromValue(int value)
		{
			Type type = valueMapping.get(value);
			assert type != null : "type == null for value "+value;
			return type;
		}

		private final int value;
		private final String displayValue;
		private Type(int value, String displayValue)
		{
			this.value = value;
			this.displayValue = displayValue;
		}
		
		public String toString()
		{
			return displayValue;
		}
		
		/**
			Get the integer value that is used when storing a <code>BioMaterialEvent.Type</code> 
			to the database.
			@return The intger value of this type 
		*/
		public int getValue()
		{
			return value;
		}
	}
}

