/**
	$Id: BioMaterialEventSource.java 5662 2011-06-20 11:55:13Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collections;
import java.util.Set;

import net.sf.basedb.core.data.BioMaterialEventSourceData;
import net.sf.basedb.core.data.MeasuredBioMaterialData;
import net.sf.basedb.util.transform.Transformer;

/**
	Class for holding information about a parent biomaterial
	that was used in an event that created a child biomaterial or
	a physical bioassay. Each event may only include the same parent
	biomaterial once. Permission-wise the information belongs
	to the {@link BioMaterialEvent} and the user has the same permissions
	as for the event.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-06-20 13:55:13 +0200 (Mon, 20 Jun 2011) $
 */
public class BioMaterialEventSource
	implements AccessControlled
{

	private final DbControl dc;
	private final BioMaterialEventSourceData data;
	
	BioMaterialEventSource(DbControl dc, BioMaterialEventSourceData data)
	{
		this.dc = dc;
		this.data = data;
	}
	
	/*
		From the AccessControlled interface
		-----------------------------------
	*/
	@Override
	public boolean hasPermission(Permission permission)
	{
		try
		{
			return getEvent().hasPermission(permission);
		}
		catch (PermissionDeniedException ex)
		{}
		return false;
	}

	@Override
	public void checkPermission(Permission permission)
		throws PermissionDeniedException
	{
		getEvent().checkPermission(permission);
	}

	@Override
	public Set<Permission> getPermissions()
	{
		try
		{
			return getEvent().getPermissions();
		}
		catch (PermissionDeniedException ex)
		{}
		return Collections.emptySet();
	}

	// -----------------------------------
	/**
		Get the event that this information belongs to.
	*/
	public BioMaterialEvent getEvent()
	{
		return dc.getItem(BioMaterialEvent.class, data.getEvent());
	}

	/**
		Get the biomaterial that was used as a source in this event.
	*/
	public BioMaterial getBioMaterial()
	{
		return dc.getItem(BioMaterial.class, data.getBioMaterial());
	}

	/**
		Get the quantity that was used from the source biomaterial by the event.
		@return A float, or null if not known
	*/
	public Float getUsedQuantity()
	{
		return data.getUsedQuantity();
	}

	/**
		Set the quantity that was used from the source biomaterial by the event.
		This method call is ignored if the source biomaterial is not a 
		{@link MeasuredBioMaterial}.
		@param usedQuantity The used quantity or null if not known
	*/
	public void setUsedQuantity(Float usedQuantity)
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		if (data.getBioMaterial() instanceof MeasuredBioMaterialData)
		{
			MeasuredBioMaterialData mbData = (MeasuredBioMaterialData)data.getBioMaterial();
			Float delta = 0.0f;
			if (data.getUsedQuantity() != null)
			{
				delta -= data.getUsedQuantity();
			}
			if (usedQuantity != null)
			{
				delta += usedQuantity;
			}
			MeasuredBioMaterial.updateRemainingQuantity(dc, mbData, delta);
			data.setUsedQuantity(usedQuantity);
		}
	}
	
	/**
		Get the position that the source biomaterial was used on in the event.
	*/
	public int getPosition()
	{
		return data.getPosition();
	}
	
	public void setPosition(int position)
	{
		checkPermission(Permission.RESTRICTED_WRITE);
		data.setPosition(position);
	}
	
	
	public static class BioMaterialEventSourceTransformer
		implements Transformer<BioMaterialEventSource, BioMaterialEventSourceData>
	{

		public BioMaterialEventSourceTransformer()
		{}
		
		
		@Override
		public BioMaterialEventSource transform(DbControl dc, BioMaterialEventSourceData from)
		{
			return new BioMaterialEventSource(dc, from);
		}
		
	}


	
}
