/**
	$Id: DerivedBioAssay.java 5749 2011-09-19 12:31:39Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.data.ArrayBatchData;
import net.sf.basedb.core.data.ArraySlideData;
import net.sf.basedb.core.data.DerivedBioAssayData;
import net.sf.basedb.core.data.HardwareData;
import net.sf.basedb.core.data.PhysicalBioAssayData;
import net.sf.basedb.core.data.ProtocolData;
import net.sf.basedb.core.data.SoftwareData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-09-19 14:31:39 +0200 (Mon, 19 Sep 2011) $
*/
public class DerivedBioAssay
	extends AnnotatedItem<DerivedBioAssayData>
	implements Registered, FileStoreEnabled, Subtypable
{

	/**
		The type of item represented by this class.
		@see Item#DERIVEDBIOASSAY
		@see #getType()
	*/
	public static final Item TYPE = Item.DERIVEDBIOASSAY;
	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing a
		scanned hybridization.
	*/
	public static final String SCAN = "net.sf.basedb.core.DerivedBioAssayType.SCAN";

	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing
		original sequenced data.
	*/
	public static final String SEQUENCES = "net.sf.basedb.core.DerivedBioAssayType.SEQUENCES";
	
	/**
		The id for the <code>DerivedBioAssayType</code> item representing
		sequenced data that has been aligned to a reference database.
	*/
	public static final String ARRANGEMENT = "net.sf.basedb.core.DerivedBioAssayType.ARRANGEMENT";

	
	/**
		Get a <code>DerivedBioAssay</code> item when you know the id.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>DerivedBioAssay</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static DerivedBioAssay getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		DerivedBioAssay dbas = dc.loadItem(DerivedBioAssay.class, id);
		if (dbas == null) throw new ItemNotFoundException("DerivedBioAssay[id="+id+"]");
		return dbas;
	}
	
	/**
		Create a new (root) derived bioassay from a physical bioassay.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param bioAssay The parent physical bioassay
		@param job The job that created the new bioassay
		@return A new DerivedBioAssay item
	*/
	public static DerivedBioAssay getNew(DbControl dc, PhysicalBioAssay bioAssay, Job job)
	{
		if (bioAssay == null) throw new InvalidUseOfNullException("bioAssay");
		bioAssay.checkPermission(Permission.USE);
		
		DerivedBioAssay dbas = dc.newItem(DerivedBioAssay.class);
		dbas.setName("New derived bioassay");
		dbas.getData().setPhysicalBioAssay(bioAssay.getData());
		dbas.getData().setEntryDate(new Date());
		dbas.getData().setJob(job == null ? null : job.getData());
		return dbas;
	}
	
	/**
		Create a new child derived bioassay from a parent derived bioassay.
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param parent The parent bioassay 
		@param job The job that created the new bioassay
		@return A new DerivedBioAssay item
	*/
	public static DerivedBioAssay getNew(DbControl dc, DerivedBioAssay parent, Job job)
	{
		if (parent == null) throw new InvalidUseOfNullException("parent");
		parent.checkPermission(Permission.USE);
		
		DerivedBioAssay dbas = dc.newItem(DerivedBioAssay.class);
		dbas.setName("New derived bioassay");
		dbas.getData().setParent(parent.getData());
		dbas.getData().setPhysicalBioAssay(parent.getData().getPhysicalBioAssay());
		dbas.getData().setExtract(parent.getData().getExtract());
		dbas.getData().setEntryDate(new Date());
		dbas.getData().setJob(job == null ? null : job.getData());
		return dbas;
	}
	
	/**
		Get a query that returns derived bioassay.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<DerivedBioAssay> getQuery()
	{
		return new ItemQuery<DerivedBioAssay>(DerivedBioAssay.class);
	}
	
	DerivedBioAssay(DerivedBioAssayData data)
	{
		super(data);
	}
	
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
	 	Get all:
	 	<ul>
	 	<li>Child derived bioassays
	 	<li>Child raw bioassays
	 	</ul>
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Child derived bioassays
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_CHILD_DERIVEDBIOASSAYS_FOR_DERIVEDBIOASSAY", "dba.id");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.parent = :bioAssay
			*/
		query.setEntity("bioAssay", this.getData());
		addUsingItems(using, Item.DERIVEDBIOASSAY, query);

		// Child raw bioassays
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", "rba.id");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.parentBioAssay = :bioAssay
			*/
		query.setEntity("bioAssay", this.getData());
		addUsingItems(using, Item.RAWBIOASSAY, query);
		
		return using;
	}
	
	/**
		
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		boolean used = false;
		
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", "count(*)");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.parentBioAssay = :bioAssay
			*/
			query.setEntity("bioAssay", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}

		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_CHILD_DERIVEDBIOASSAYS_FOR_DERIVEDBIOASSAY", "count(*)");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.parent = :bioAssay
			*/
			query.setEntity("bioAssay", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		
		return used || super.isUsed();
	}

	/**
		Set protocol, hardware and software from project default settings.
	*/
	@Override
	void setProjectDefaults(Project activeProject)
		throws BaseException
	{
		super.setProjectDefaults(activeProject);
		if (!hasPermission(Permission.WRITE)) return;
			
		DbControl dc = getDbControl();
		if (!protocolHasBeenSet)
		{
			ProtocolData protocol = 
				(ProtocolData)activeProject.findDefaultRelatedData(dc, this, Item.PROTOCOL, false);
			if (protocol != null)
			{
				getData().setProtocol(protocol);
				protocolHasBeenSet = true;
			}
		}
		if (!hardwareHasBeenSet)
		{
			HardwareData hardware = 
				(HardwareData)activeProject.findDefaultRelatedData(dc, this, Item.HARDWARE, false);
			if (hardware != null) 
			{
				getData().setHardware(hardware);
				hardwareHasBeenSet = true;
			}
		}
		if (!softwareHasBeenSet)
		{
			SoftwareData software = 
				(SoftwareData)activeProject.findDefaultRelatedData(dc, this, Item.SOFTWARE, false);
			if (software != null) 
			{
				getData().setSoftware(software);
				softwareHasBeenSet = true;
			}
		}
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		Get the extract and parent bioassay set or physical bioassay.
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
		throws BaseException
	{
		Set<Annotatable> annotatable = new HashSet<Annotatable>();
		try
		{
			if (isRoot())
			{
				annotatable.add(getPhysicalBioAssay());
			}
			else
			{
				annotatable.add(getParent());
			}
			if (getData().getExtract() != null) annotatable.add(getExtract());
		}
		catch (PermissionDeniedException ex)
		{}
		return annotatable;
	}
	@Override
	public Protocol getProtocol()
	{
		return getDbControl().getItem(Protocol.class, getData().getProtocol());
	}
	// -------------------------------------------
	/*
		From the Subtypable interface
		-----------------------------
	*/
	@Override
	@SubtypableRelatedItems({Item.PHYSICALBIOASSAY, Item.DERIVEDBIOASSAY, Item.EXTRACT, Item.SOFTWARE, Item.HARDWARE, Item.PROTOCOL})
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	@Override
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		if (subtype != null)
		{
			subtype.setOnItem(this);
		}
		else
		{
			getData().setItemSubtype(null);
		}
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	// -------------------------------------------

	/*
		From the FileStoreEnabled interface
		-------------------------------------------
	*/
	@Override
	public FileSet getFileSet()
	{
		DbControl dc = getDbControl();
		FileSet fs = dc.getItem(FileSet.class, getData().getFileSet(), this);
		if (fs == null)
		{
			fs = FileSet.getNew(dc, this);
			getData().setFileSet(fs.getData());
			dc.saveItemIf(this, fs, false);
		}
		return fs;
	}
	@Override
	public boolean hasFileSet()
	{
		return getData().getFileSet() != null;
	}
	@Override
	public Platform getPlatform()
	{
		return null;
	}
	@Override
	public PlatformVariant getVariant()
	{
		return null;
	}
	@Override
	public Collection<FileSet> getParentFileSets()
	{
		DerivedBioAssay parent = getParent();
		Collection<FileSet> parents = null;
		if (parent != null && parent.hasFileSet())
		{
			parents = Collections.singleton(parent.getFileSet());
		}
		return parents;
	}
	// -------------------------------------------

	/**
		Get the physical bioassay that is the root bioassay for this derived
		bioassay set. This property is always inherited down to child events 
		so this method never return null.
	*/
	public PhysicalBioAssay getPhysicalBioAssay()
	{
		return getDbControl().getItem(PhysicalBioAssay.class, getData().getPhysicalBioAssay());
	}
	
	/**
		Get the parent derived bioassay set.		
		@return The parent bioassay set or null if this item is a root bioassay set
	*/
	public DerivedBioAssay getParent()
	{
		return getDbControl().getItem(DerivedBioAssay.class, getData().getParent());
	}

	/**
		Check if this bioassay set is derived from a physical bioassay or from another
		bioassay set.
		@return TRUE if this is a root bioassay set derived form a physical bioassay
	*/
	public boolean isRoot()
	{
		return getData().getParent() == null;
	}
	
	/**
		Get the extract that is the parent of the data in this
		derived bioassay. The extract should be an extract that is
		present on the physical bioassay that this bioassay is derived from.
		But this is not enforced by the core. If no extract is specified this
		bioassay applies to all extracts that are part of the physical bioassay.
		@return An extract or null
	*/
	public Extract getExtract()
	{
		return getDbControl().getItem(Extract.class, getData().getExtract());
	}
	
	/**
		Set the extract that is the parent of data in this bioassay. The extract 
		should be an extract that is present on the physical bioassay that this 
		bioassay is derived from. But this is not enforced by the core.
		@param extract The extract or null
	*/
	public void setExtract(Extract extract)
	{
		checkPermission(Permission.WRITE);
		if (extract != null) extract.checkPermission(Permission.USE);
		getData().setExtract(extract == null ? null : extract.getData());
	}
	
	/**
		Get all child derived bioassays.
		@return A query returning all child bioassays
	*/
	public ItemQuery<DerivedBioAssay> getChildren()
	{
		ItemQuery<DerivedBioAssay> query = DerivedBioAssay.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("parent"),
				Hql.entity(this)
			)
		);
		return query;
	}

	/**
		Get the job that created this bioassay.
		@return A job or null if not created by a job
	*/
	public Job getJob()
	{
		return getDbControl().getItem(Job.class, getData().getJob());
	}
	
	private boolean protocolHasBeenSet = false;
	/**
		Set the {@link Protocol} that describes the procedure used when 
		creating this bioassay set.
		@param protocol A <code>Protocol</code> item, or null if not known
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission on this event or use permission for the protocol
	*/
	public void setProtocol(Protocol protocol)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (protocol != null) protocol.checkPermission(Permission.USE);
		protocolHasBeenSet = true;
		getData().setProtocol(protocol == null ? null : protocol.getData());
	}
	
	/**
		Get the {@link Hardware} that was used  when 
		creating this bioassay set.
	 	@return A <code>Hardware</code> item, or null if not known
	*/
	public Hardware getHardware()
	{
		return getDbControl().getItem(Hardware.class, getData().getHardware());
	}
	
	private boolean hardwareHasBeenSet = false;
	/**
	 	Set the {@link Hardware} that was used when 
		creating this bioassay set.
	 	@param hardware A <code>Hardware</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the hardware
	*/
	public void setHardware(Hardware hardware)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (hardware != null) hardware.checkPermission(Permission.USE);
		hardwareHasBeenSet = true;
		getData().setHardware(hardware == null ? null : hardware.getData());
	}
	
	/**
		Get the {@link Software} that was used when 
		creating this bioassay set.
	 	@return A <code>Software</code> item, or null if not known
	*/
	public Software getSoftware()
	{
		return getDbControl().getItem(Software.class, getData().getSoftware());
	}
	
	private boolean softwareHasBeenSet = false;
	/**
	 	Set the {@link Software} that was used when 
		creating this bioassay set.
	 	@param software A <code>Software</code> item, or null if not known
	 	@throws PermissionDeniedException If the logged in user doesn't have
	 		write permission on this event or use permission for the software
	*/
	public void setSoftware(Software software)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (software != null) software.checkPermission(Permission.USE);
		hardwareHasBeenSet = true;
		getData().setSoftware(software == null ? null : software.getData());
	}

	
	public ItemQuery<RawBioAssay> getRawBioAssays()
	{
		ItemQuery<RawBioAssay> query = RawBioAssay.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("parentBioAssay"),
				Hql.entity(this)
			)
		);
		return query;
	}
	
	/**
		Get the array design connected to this derived bioassay by 
		following the path from the physical bioassay to the array slide to 
		the array batch  to the array design.
		@return The array design, or null if no array design is connected
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission for the array design
		@throws BaseException If there is another error.
	*/
	public ArrayDesign getArrayDesign()
		throws PermissionDeniedException, BaseException
	{
		PhysicalBioAssayData pba = getData().getPhysicalBioAssay();
		if (pba == null) return null;
		ArraySlideData slide = pba.getArraySlide();
		if (slide == null) return null;
		ArrayBatchData batch = slide.getArrayBatch();
		if (batch == null) return null;
		return getDbControl().getItem(ArrayDesign.class, batch.getArrayDesign());
	}

	/**
		Counts all raw bioassays that have been created from this bioassay. The count
		includes raw bioassays that the logged in user doesn't have access to.
		@return The number of raw bioassays
	 	@throws BaseException If the raw bioassays couldn't be counted. 
	*/
	public long countRawBioAssays()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_DERIVEDBIOASSAY", "count(*)");
		/*
			SELECT {1}
			FROM RawBioAssayData rba
			WHERE rba.parentBioAssay = :bioAssay
		*/
		query.setEntity("bioAssay", this.getData());
		return HibernateUtil.loadData(Long.class, query);
	}

}
