/*
	$Id: DynamicReporterQuery.java 5711 2011-08-30 13:49:58Z nicklas $
	
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.query.AutoJoiner;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.ReporterListExpression;
import net.sf.basedb.core.query.Select;

/**
	Represents a query for reporter data in a "dynamic" way. The main difference 
	from a client applications point of view, is that it is possible to 
	specify which columns that should be selected. In a techinical point of
	view the difference is that we are using SQL instead of HQl.
	
	@author Nicklas
	@version 2.4
	@see Reporter#getDynamicQuery()
	@base.modified $Date: 2011-08-30 15:49:58 +0200 (Tue, 30 Aug 2011) $
*/
public class DynamicReporterQuery
	extends AbstractSqlQuery
{

	private final RealTable rootTable;
	private Set<Object> joinedItems;

	/**
		Create a new dynamic query for reporter data.		
	*/
	DynamicReporterQuery()
	{
		super(RealTable.REPORTERS.getQualifiedTableName());
		this.rootTable = RealTable.REPORTERS;
		joinedItems = new HashSet<Object>();
		setAutoJoinType(JoinType.LEFT);
	}
	
	/*
		From the Query interface
		-------------------------------------------
	*/
	/**
		The alias of the item that is returned by this query.
		@see Item#getAlias()
	*/
	public String getRootAlias()
	{
		return HibernateUtil.quote(rootTable.getAlias());
	}
	public boolean isReadonly()
	{
		return false;
	}
	// -------------------------------------------

	/*
		From the AbstractSqlQuery class
		-------------------------------------------
	*/
	/**
		No default selection.
	*/
	Select[] getDefaultSelects()
	{
		return null;
	}
	// -------------------------------------------

	RealTable getRootTable()
	{
		return rootTable;
	}
	
	/**
	  	Makes a join with a <code>ReporterList</code>
	  	@param reporterList To do the join on.
	  	@param joinType The join type to use. 
			@see net.sf.basedb.core.query.JoinType
			@since 3.0
	*/
	public void joinReporterList(ReporterList reporterList, JoinType joinType)
	{
		if (joinType == null) throw new InvalidUseOfNullException("joinType");
		if (reporterList == null) throw new InvalidUseOfNullException("reporterList");
		if (!joinedItems.contains(reporterList))
		{
			join(new ReporterListJoin(reporterList, joinType, RealTable.REPORTERS, "id"));
			joinedItems.add(reporterList);
		}
	}
	
	/**
		Specify the join type of automatic joins. The default join type is
		{@link JoinType#LEFT}. This setting doesn't affect joins that has been made
		directly to the query.
		@param joinType The type of the automatic joins
		@since 3.0
	*/
	public void setAutoJoinType(JoinType joinType)
	{
		addAutoJoiner(getReporterListJoiner(joinType));
	}

	/**
		Get a joiner instance for reporter lists for the specified join type.
		@param joinType The type of join
		@since 3.0
	*/
	public static ReporterListJoiner getReporterListJoiner(JoinType joinType)
	{
		return new ReporterListJoiner(joinType);
	}
	
	/**
		Auto joiner for reporter lists.
		@since 3.0
	*/
	private static class ReporterListJoiner
		implements AutoJoiner<DynamicReporterQuery, ReporterListExpression>
	{
		private final JoinType joinType;
	
		private ReporterListJoiner(JoinType joinType)
		{
			this.joinType = joinType;
		}
		
		/*
			From the AutoJoiner interface
			-------------------------------------------
		*/
		@Override
		public Class<DynamicReporterQuery> getQueryClass()
		{
			return DynamicReporterQuery.class;
		}
		@Override
		public Class<ReporterListExpression> getElementClass()
		{
			return ReporterListExpression.class;
		}
		@Override
		public void joinIfNeeded(DynamicReporterQuery query, ReporterListExpression element)
		{
			JoinType jt = element.getAutoJoinType();
			query.joinReporterList(element.getReporterList(), jt == null ? joinType : jt);
		}
		// -------------------------------------------
	}

}
