/*
	$Id: DynamicResultIterator.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.SqlResultIterator;
import net.sf.basedb.core.query.SqlResult;

import java.util.NoSuchElementException;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
	An iterator view of the result of an {@link net.sf.basedb.core.AbstractSqlQuery}.
	
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.AbstractSqlQuery#iterate(DbControl)
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class DynamicResultIterator
	implements SqlResultIterator
{

	/**
		Logger.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.query.DynamicResultIterator");
	
	/**
		The results of the query.
	*/
	private final ResultSet results;

	/**
		The total number of items.
	*/
	private final long totalCount;

	private final int limit;
	
	private int returnedRows;
	
	private final SqlResult sqlResult;
	
	private boolean hasNext;
	private boolean checkNext;
	private boolean isClosed;

	/**
		Create a new result iterator.
		@param results The <code>ResultSet</code> object containing the
			results
		@param totalCount The total number of rows
		@param limit The maximum numbre of rows that was specified for the
			query
	*/
	DynamicResultIterator(ResultSet results, long totalCount, int limit)
	{
		assert results != null : "results == null";
		this.results = results;
		this.totalCount = totalCount;
		this.limit = limit;
		this.returnedRows = 0;
		this.sqlResult = new SqlResultImpl();
		this.checkNext = true;
	}

	/**
		From the QueryResult interface
		---------------------------------
	*/
	public long getTotalCount()
	{
		return totalCount;
	}
	// ---------------------------------
	/**
		From the ResultIterator interface
		---------------------------------
	*/
	public void close()
	{
		if (!isClosed()) 
		{
			isClosed = true;
			hasNext = false;
			checkNext = false;
			try
			{
				results.getStatement().close();
				results.close();
			}
			catch (SQLException ex)
			{
				log.warn("Exception when closing result set", ex);
			}
		}
	}
	public boolean isClosed()
	{
		return isClosed;
	}
	// ---------------------------------
	/*
		From the Iterator interface
		---------------------------
	*/
	public boolean hasNext()
	{
		if (checkNext && !isClosed())
		{
			try
			{
				hasNext = results.next();
				returnedRows++;
			}
			catch (SQLException ex)
			{
				throw new DatabaseException(ex);
			}
			if (!hasNext || (returnedRows > limit && limit > 0))
			{
				close();
				hasNext = false;
			}
			checkNext = false;
		}
		return hasNext;
	}
	/**
		Get the results in the next row. The actual {@link SqlResult} object
		is always the same object. It just changes the underlying pointer to
		the next row in the result set. If you need to store data from
		a previous row you must copy the values to another place.
		@return A <code>SqlResult</code> object
		@throws NoSuchElementException If there are no more results
	*/
	public SqlResult next()
	{
		if (!hasNext()) throw new NoSuchElementException();
		checkNext = true;
		return sqlResult;
	}
	/**
		Not supported.
		@throws UnsupportedOperationException Always
	*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
	// ---------------------------------

	/*
		From the SqlResultIterator interface
		---------------------------------
	*/
	public int getIndex(String name)
		throws SQLException
	{
		return results.findColumn(name);
	}
	// ---------------------------------

	/**
		Implements the SqlResult interface.
	*/
	private class SqlResultImpl 
		implements SqlResult
	{
		/*
			From the SqlResult interface
			---------------------------
		*/
		public short getShort(int index)
			throws SQLException
		{
			return results.getShort(index);
		}
		public int getInt(int index)
			throws SQLException
		{
			return results.getInt(index);
		}
		public float getFloat(int index)
			throws SQLException
		{
			return results.getFloat(index);
		}
		public String getString(int index)
			throws SQLException
		{
			return results.getString(index);
		}
		public Object getObject(int index)
			throws SQLException
		{
			return results.getObject(index);
		}
		// ---------------------------------
	}

}
