/*
	$Id: Item.java 5730 2011-09-12 10:42:14Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.*;

import java.util.Comparator;
import java.util.Set;
import java.util.EnumSet;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;

/**
	This class defines constants for various items in BASE.

	@author Samuel
	@version 2.0
	@base.modified $Date: 2011-09-12 12:42:14 +0200 (Mon, 12 Sep 2011) $
*/
public enum Item
{
	/**
		Not an item as susch. Used for setting system permissions
		to role keys.
		@see Install
		@see SessionControl#hasSystemPermission(Permission)
	*/
	SYSTEM(1, "System", null, null, null, EnumSet.of(Permission.ACT_AS_ANOTHER_USER, Permission.SHARE_TO_EVERYONE), 
		0),
	
	/**
		The item is a {@link User}.
	*/
	USER(21, "User", "usr", User.class, UserData.class, DefinedPermissions.basic,
		1410),
	/**
		The item is a {@link Group}.
	*/
	GROUP(22, "Group", "grp", Group.class, GroupData.class, DefinedPermissions.basic,
		1420),
	/**
		The item is a {@link Role}.
	*/
	ROLE(23, "Role", "rle", Role.class, RoleData.class, DefinedPermissions.basic,
		1430),
	/**
		The item is a {@link Project}.
	*/
	PROJECT(24, "Project", "prj", Project.class, ProjectData.class, DefinedPermissions.ownable,
		1400),
	/**
		The item is an {@link ItemKey}.
	*/
	ITEMKEY(25, "Item key", "ik", ItemKey.class, ItemKeyData.class, null,
		1310),
	/**
		The item is a {@link ProjectKey}.
	*/
	PROJECTKEY(26, "Project key", "pk", ProjectKey.class, ProjectKeyData.class, null,
		1320),
	/**
		The item is a {@link RoleKey}.
	*/
	ROLEKEY(27, "Role key", "rk", RoleKey.class, RoleKeyData.class, null,
		1330),
	/**
		The item is a {@link PermissionTemplate}.
	*/
	PERMISSIONTEMPLATE(28, "Permission template", "pt", PermissionTemplate.class, PermissionTemplateData.class, 
		DefinedPermissions.shareable, 1300),
	

	/**
		The item is an {@link Client}.
	*/
	CLIENT(41, "Client", "cli", Client.class, ClientData.class, DefinedPermissions.shareable,
		1260),
	/**
		The item is a {@link Help}.
	*/
	HELP(42, "Help", "hlp", Help.class, HelpData.class, null,
		1270),
	/**
		The item is an {@link Session}.
	*/
	SESSION(43, "Session", "ses", Session.class, SessionData.class, DefinedPermissions.read,
		1200),
	/**
		The item is an {@link Setting}.
	*/
	SETTING(44, "Setting", "set", Setting.class, SettingData.class, DefinedPermissions.basic,
		1210),
	/**
		The item is an {@link GlobalDefaultSetting}.
	*/
	GLOBALDEFAULTSETTING(45, "Global default setting", "gds", GlobalDefaultSetting.class, GlobalDefaultSettingData.class, DefinedPermissions.basic,
		1220),
	/**
		The item is an {@link UserDefaultSetting}.
	*/
	USERDEFAULTSETTING(46, "User default setting", "uds", UserDefaultSetting.class, UserDefaultSettingData.class, DefinedPermissions.basic,
		1230),
	/**
		The item is an {@link ClientDefaultSetting}.
	*/
	CLIENTDEFAULTSETTING(47, "Client default setting", "cds", ClientDefaultSetting.class, ClientDefaultSettingData.class, DefinedPermissions.basic,
		1240),
	/**
		The item is an {@link UserClientSetting}.
	*/
	USERCLIENTSETTING(48, "User client setting", "ucs", UserClientSetting.class, UserClientSettingData.class, DefinedPermissions.basic,
		1250),

	/**
		The item is a {@link Quota}.
	*/
	QUOTA(61, "Quota", "qta", Quota.class, QuotaData.class, DefinedPermissions.basic,
		1600),
	/**
		The item is a {@link QuotaType}.
	*/
	QUOTATYPE(62, "Quota type", "qtp", QuotaType.class, QuotaTypeData.class, DefinedPermissions.write,
		1610),
	/**
		The item is a {@link DiskUsage}.
	*/
	DISKUSAGE(63, "Disk usage", "du", DiskUsage.class, DiskUsageData.class, DefinedPermissions.read, 0),


	/**
		The item is a {@link File}.
	*/
	FILE(81, "File", "fle", File.class, FileData.class, DefinedPermissions.shareable,
		1100),
		
	/*
		IMPORTANT! Do not use the following codes since they were used in BASE 2:
		82 = FILETYPE
	*/

	/**
		The item is a {@link Directory}.
	*/
	DIRECTORY(83, "Directory", "dir", Directory.class, DirectoryData.class, DefinedPermissions.shareable,
		1120),
	/**
		The item is a {@link MimeType}.
	*/
	MIMETYPE(84, "Mime type", "mtp", MimeType.class, MimeTypeData.class, DefinedPermissions.basic,
		1130),
	/**
		The item is a {@link FileServer}.
	*/
	FILESERVER(85, "File server", "fsv", FileServer.class, FileServerData.class, DefinedPermissions.shareable,
		1140),

	/**
		The item is a {@link Protocol}.
	*/
	PROTOCOL(101, "Protocol", "prl", Protocol.class, ProtocolData.class, DefinedPermissions.shareable,
		740),
	/*
		IMPORTANT! Do not use the following codes since they were used in BASE 2:
		102 = PROTOCOLTYPE
		122 = HARDWARETYPE
		124 = SOFTWARETYPE
	*/

	/**
		The item is a {@link Hardware}.
	*/
	HARDWARE(121, "Hardware", "hw", Hardware.class, HardwareData.class, DefinedPermissions.shareable,
		700),
	/**
		The item is a {@link Software}.
	*/
	SOFTWARE(123, "Software", "sw", Software.class, SoftwareData.class, DefinedPermissions.shareable,
		720),

	/**
		The item is a {@link News}.
	*/
	NEWS(141, "News", "nws", News.class, NewsData.class, DefinedPermissions.basic,
		1500),

	/**
		The item is a {@link Message}.
	*/
	MESSAGE(142, "Message", "msg", Message.class, MessageData.class, DefinedPermissions.basic,
		1000),
	
	/**
		The item is an {@link AnyToAny} link.
	*/
	ANYTOANY(143, "Any-to-any link", "ata", AnyToAny.class, AnyToAnyData.class, null, 
		10),
		
	/**
		The item is a {@link ChangeHistory} entry.
	*/
	CHANGEHISTORY(144, "Change history", "chh", ChangeHistory.class, 
		ChangeHistoryDetailData.class, DefinedPermissions.read, 9999),

	/**
		The item is a {@link ItemSubtype}.
	*/
	ITEMSUBTYPE(145, "Item subtype", "ist", ItemSubtype.class, ItemSubtypeData.class, DefinedPermissions.basic, 
			1510),
	/**
		The item is a {@link ItemSubtypeFileType}
	*/
	ITEMSUBTYPEFILETYPE(146, "Item subtype file type", "sft", ItemSubtypeFileType.class, ItemSubtypeFileTypeData.class, null,
		960),

	/**
		The item is a {@link Reporter}.
	*/
	REPORTER(161, "Reporter", "rpt", null, ReporterData.class, DefinedPermissions.basic,
		810),
	/**
		The item is a {@link ReporterType}.
	*/
	REPORTERTYPE(162, "Reporter type", "rtp", ReporterType.class, ReporterTypeData.class, DefinedPermissions.basic,
		830),
	/**
		The item is a {@link ReporterType}.
	*/
	REPORTERLIST(163, "Reporter list", "rpl", ReporterList.class, ReporterListData.class, DefinedPermissions.shareable,
		800),
	/**
		The item is a {@link ReporterScore}. This item is not a BasicItem.
	*/
	REPORTERSCORE(164, "Reporter score", "rps", null, null, null,
		820),

	/**
		The item is an {@link AnnotationSet}.
	*/
	ANNOTATIONSET(181, "Annotation set", "ans", AnnotationSet.class, AnnotationSetData.class, null,
		900),

	/**
		The item is an {@link Annotation}.
	*/
	ANNOTATION(182, "Annotation", "ann", Annotation.class, AnnotationData.class, null,
		910),
	
	/**
		The item is an {@link AnnotationType}.
	*/
	ANNOTATIONTYPE(183, "Annotation type", "atp", AnnotationType.class, AnnotationTypeData.class, DefinedPermissions.shareable,
		920),
	
	/**
		The item is an {@link AnnotationTypeCategory}.
	*/
	ANNOTATIONTYPECATEGORY(184, "Annotation type category", "atc", AnnotationTypeCategory.class, AnnotationTypeCategoryData.class, DefinedPermissions.shareable,
		930),

	/**
		The item is a {@link Quantity}.
	*/
	QUANTITY(185, "Quantity", "qty", Quantity.class, QuantityData.class, DefinedPermissions.basic, 
		950),
		
	/**
		The item is a {@link Unit}.
	*/
	UNIT(186, "Unit", "unt", Unit.class, UnitData.class, DefinedPermissions.basic,
		940),

	/**
		The item is a {@link BioSource}.
	*/
	BIOSOURCE(201, "Biosource", "bs", BioSource.class, BioSourceData.class, DefinedPermissions.shareable,
		430),
	/**
		The item is a {@link Sample}.
	*/
	SAMPLE(202, "Sample", "smp", Sample.class, SampleData.class, DefinedPermissions.shareable,
		420),
	/**
		The item is an {@link Extract}.
	*/
	EXTRACT(203, "Extract", "xtr", Extract.class, ExtractData.class, DefinedPermissions.shareable,
		410),

	/*
		IMPORTANT! Do not use the following codes since they were used in BASE 2:
		204 = LABELEDEXTRACT
	*/
	/**
		The item is a {@link BioMaterialEvent}.
	*/
	BIOMATERIALEVENT(205, "Biomaterial event", "bme", BioMaterialEvent.class, BioMaterialEventData.class, null,
		450),
	/**
		The item is a {@link Tag}.
	*/
	TAG(206, "Tag", "tag", Tag.class, TagData.class, DefinedPermissions.shareable,
		440),

	/**
	 	The item is a {@link BioPlate}
	 */
	BIOPLATE(207, "Bioplate", "bpl", BioPlate.class, BioPlateData.class, DefinedPermissions.shareable,
			460),

	/**
	 	The item is a {@link BioWell}
	*/
	BIOWELL(208, "Biowell", "bwl", BioWell.class, BioWellData.class, null,
			470),
			
	/**
	 	The item is a {@link BioMaterialList}
	*/
	BIOMATERIALLIST(209, "Biomaterial list", "bml", BioMaterialList.class, BioMaterialListData.class, DefinedPermissions.shareable,
			480),

	/**
	 	The item is a {@link BioPlateType}.
	 	@since 2.16
	 */
	BIOPLATETYPE(210, "Bioplate type", "bpt", BioPlateType.class, BioPlateTypeData.class, 
			DefinedPermissions.basic, 490),

	/**
 		The item is a {@link BioPlateEvent}
 		@since 2.17
	*/
	BIOPLATEEVENT(211, "Bioplate event", "bpe", BioPlateEvent.class, BioPlateEventData.class, 
		DefinedPermissions.shareable, 465),

	/**
 		The item is a {@link BioPlateEventParticipant}
 		@since 2.17
	*/
	BIOPLATEEVENTPARTICIPANT(212, "Bioplate event participant", "bpp", BioPlateEventParticipant.class, BioPlateEventParticipantData.class, 
		null, 466),
	
	/**
 		The item is a {@link BioPlateEventType}
 		@since 2.17
	*/
	BIOPLATEEVENTTYPE(213, "Bioplate event type", "bet", BioPlateEventType.class, BioPlateEventTypeData.class, 
		DefinedPermissions.basic, 467),

		
	/**
		The item is a {@link PlateGeometry}.
	*/
	PLATEGEOMETRY(221, "Plate geometry", "pgm", PlateGeometry.class, PlateGeometryData.class, DefinedPermissions.basic,
		630),
	/**
		The item is a {@link Plate}.
	*/
	PLATE(222, "Plate", "pla", Plate.class, PlateData.class, DefinedPermissions.shareable,
		600),
	/**
		The item is a {@link PlateType}.
	*/
	PLATETYPE(223, "Plate type", "ptt", PlateType.class, PlateTypeData.class, DefinedPermissions.shareable,
		610),
	/**
		The item is a {@link PlateEvent}.
	*/
	PLATEEVENT(224, "Plate event", "ple", PlateEvent.class, PlateEventData.class, null,
		640),
	/**
		The item is a {@link PlateEventType}.
	*/
	PLATEEVENTTYPE(225, "Plate event type", "pet", PlateEventType.class, PlateEventTypeData.class, null,
		660),
	/**
		The item is a {@link Well}.
	*/
	WELL(226, "Well", "wll", Well.class, WellData.class, null,
		650),
	/**
		The item is a {@link PlateMapping}.
	*/
	PLATEMAPPING(227, "Plate mapping", "plm", PlateMapping.class, PlateMappingData.class, DefinedPermissions.shareable,
		620),


	/**
		The item is an {@link ArrayDesign}.
	*/
	ARRAYDESIGN(241, "Array design", "ard", ArrayDesign.class, ArrayDesignData.class, DefinedPermissions.shareable,
		520),
	/**
		The item is an {@link ArrayBatch}.
	*/
	ARRAYBATCH(242, "Array batch", "arb", ArrayBatch.class, ArrayBatchData.class, DefinedPermissions.shareable,
		510),
	/**
		The item is an {@link ArraySlide}.
	*/
	ARRAYSLIDE(243, "Array slide", "ars", ArraySlide.class, ArraySlideData.class, DefinedPermissions.shareable,
		500),
	/**
		The item is an {@link ArrayDesignBlock}.
	*/
	ARRAYDESIGNBLOCK(244, "Array design block", "adb", ArrayDesignBlock.class, ArrayDesignBlockData.class, null,
		530),
	/**
		The item is a {@link Feature}.
	*/
	FEATURE(245, "Feature", "ftr", null, FeatureData.class, null,
		540),
	/**
		The item is a {@link ArrayDesignPlate} connecter.
	*/
	ARRAYDESIGNPLATE(247, "Array design Plate", "adp", ArrayDesignPlate.class, ArrayDesignPlateData.class, null,
		550),

	/*
		IMPORTANT! Do not use the following codes since they were used in BASE 2:
		261 = HYBRIDIZATION
		262 = SCAN
		263 = IMAGE
	*/
	/**
		The item is a {@link PhysicalBioAssay}.
	*/
	PHYSICALBIOASSAY(267, "Physical bioassay", "pba", PhysicalBioAssay.class, PhysicalBioAssayData.class, DefinedPermissions.shareable,
		350),

	/**
		The item is a {@link DerivedBioAssay}.
	*/
	DERIVEDBIOASSAY(268, "Derived bioassay", "dba", DerivedBioAssay.class, DerivedBioAssayData.class, DefinedPermissions.shareable,
		330),
		
	/**
		The item is a {@link RawBioAssay}.
	*/
	RAWBIOASSAY(264, "Raw bioassay", "rba", RawBioAssay.class, RawBioAssayData.class, DefinedPermissions.shareable,
		300),
	/**
		The item is a {@link SpotImages}.
	*/
	SPOTIMAGES(265, "Spot images", "spi", SpotImages.class, SpotImagesData.class, null,
		310),
	/**
		The item is a {@link RawData} spot.
	*/
	RAWDATA(266, "Raw data", "rdt", null, RawData.class, null,
		320),

	/**
		The item is a {@link PluginType}
	*/
	PLUGINTYPE(281, "Plugin type", "plt", PluginType.class, PluginTypeData.class, DefinedPermissions.basic,
		1060),
	/**
		The item is a {@link PluginDefinition}
	*/
	PLUGINDEFINITION(282, "Plugin definition", "pld", PluginDefinition.class, PluginDefinitionData.class, DefinedPermissions.shareable,
		1050),
	/**
		The item is a {@link PluginConfiguration}
	*/
	PLUGINCONFIGURATION(283, "Plugin configuration", "plc", PluginConfiguration.class, PluginConfigurationData.class, DefinedPermissions.shareable,
		1040),
	/**
		The item is a {@link Job}
	*/
	JOB(284, "Job", "job", Job.class, JobData.class, DefinedPermissions.ownable,
		1010),
	/**
	 	The item is a {@link JobAgent}
	*/
	JOBAGENT(285, "Job agent", "jag", JobAgent.class, JobAgentData.class, DefinedPermissions.shareable,
		1020),
	/**
		The item is a {@link JobAgentSettings}
	*/
	JOBAGENTSETTINGS(286, "Job agent settings", "jas", JobAgentSettings.class, JobAgentSettingsData.class, null,
		1030),
	
	/**
		The item is an {@link Experiment}
	*/
	EXPERIMENT(301, "Experiment", "xpr", Experiment.class, ExperimentData.class, DefinedPermissions.shareable, 
		100),
	/**
		The item is a {@link BioAssaySet}
	*/
	BIOASSAYSET(302, "Bioassayset", "bas", BioAssaySet.class, BioAssaySetData.class, null,
		120),
	/**
		The item is a {@link BioAssay}
	*/
	BIOASSAY(303, "Bioassay", "bay", BioAssay.class, BioAssayData.class, null,
		130),
	/**
		The item is a {@link Transformation}
	*/
	TRANSFORMATION(304, "Transformation", "trf", Transformation.class, TransformationData.class, null, 
		110),
	/**
		The item is an {@link ExtraValueType}
	*/
	EXTRAVALUETYPE(305, "Extra value type", "evt", ExtraValueType.class, ExtraValueTypeData.class, DefinedPermissions.basic,
		150),
	/**
		The item is an {@link ExtraValue}
	*/
	EXTRAVALUE(306, "Extra value", "evl", ExtraValue.class, ExtraValueData.class, null,
		140),
	/**
		The item is a {@link Formula}
	*/
	FORMULA(307, "Formula", "fml", Formula.class, FormulaData.class, DefinedPermissions.shareable,
		160),
	/**
		Representing spot data, not an actual item.
	*/
	SPOTDATA(308, "Spot data", "spt", null, null, null, 0),

	/**
		The item is a {@link VirtualDb}
	*/
	VIRTUALDB(321, "Virtual database", "vdb", VirtualDb.class, VirtualDbData.class, null,
		200),
	/**
		The item is a {@link DataCube}
	*/
	DATACUBE(322, "Data cube", "dcb", DataCube.class, DataCubeData.class, null,
		210),
	/**
		The item is a {@link DataCubeLayer}
	*/
	DATACUBELAYER(323, "Data cube layer", "dcl", DataCubeLayer.class, DataCubeLayerData.class, null,
		220),
	/**
		The item is a {@link DataCubeColumn}
	*/
	DATACUBECOLUMN(324, "Data cube column", "dcc", DataCubeColumn.class, DataCubeColumnData.class, null,
		230),
	/**
		The item is a {@link DataCubeFilter}
	*/
	DATACUBEFILTER(325, "Data cube filter", "dcf", DataCubeFilter.class, DataCubeFilterData.class, null,
		240),
	/**
		The item is a {@link DataCubeExtraValue}
	*/
	DATACUBEEXTRAVALUE(326, "Data cube extra value", "dev", DataCubeExtraValue.class, DataCubeExtraValueData.class, null,
		250),
		
	/**
		The item is a {@link Platform}
	*/
	PLATFORM(350, "Platform", "plf", Platform.class, PlatformData.class, DefinedPermissions.basic,
		970),
		
	/**
		The item is a {@link PlatformVariant}
	*/
	PLATFORMVARIANT(351, "Platform variant", "plv", PlatformVariant.class, PlatformVariantData.class, null,
		965),
		
	/**
		The item is a {@link PlatformFileType}
	*/
	PLATFORMFILETYPE(352, "Platform file type", "pft", PlatformFileType.class, PlatformFileTypeData.class, null,
		960),
		
	/**
		The item is a {@link FileSet}
	*/
	FILESET(353, "File set", "fs", FileSet.class, FileSetData.class, null,
		950),

	/**
		The item is a {@link FileSetMember}
	*/
	FILESETMEMBER(354, "File set member", "fsm", FileSetMember.class, FileSetMemberData.class, null,
		955),
		
	/**
		The item is a {@link DataFileType}
	*/
	DATAFILETYPE(355, "Data file type", "dft", DataFileType.class, DataFileTypeData.class, DefinedPermissions.basic,
		980)
		
		;

	static int MAX_VALUE = 0;
	
	private static final Map<Integer, Item> valueMapping = new HashMap<Integer, Item>();
	private static final Map<String, Item> aliasMapping = new HashMap<String, Item>();
	private static final Map<Class<? extends BasicItem>, Item> itemClassMapping = new HashMap<Class<? extends BasicItem>, Item>();
	private static final Map<Class<? extends BasicData>, Item> dataClassMapping = new HashMap<Class<? extends BasicData>, Item>();
	
	static
	{
		for(Item item : Item.values())
		{
			int value = item.getValue();
			if (MAX_VALUE < value) MAX_VALUE = value;
			Item i = valueMapping.put(value, item);
			assert i == null : "Another item with the value "+value+" already exists: " + i;
			i = aliasMapping.put(item.getAlias(), item);
			assert i == null : "Another item with the alias "+item.getAlias()+" already exists: " + i;
			if (item.getItemClass() != null) 
			{
				i = itemClassMapping.put(item.getItemClass(), item);
				assert i == null : "Another item for class "+item.getItemClass()+" already exists: " + i;
			}
			if (item.getDataClass() != null) 
			{
				i = dataClassMapping.put(item.getDataClass(), item);
				assert i == null : "Another item for class "+item.getDataClass()+" already exists: " + i;
			}
		}
	}

	private final int value;
	private final String displayValue;
	private final String alias;
	private final Class<? extends BasicItem> itemClass;
	private final Class<? extends BasicData> dataClass;
	private final Set<Permission> definedPermissions;
	private final transient Method getById;
	private final transient Constructor<? extends BasicItem> constructor;
	private final float deletionOrder;
	
	
	private Item(int value, String displayValue, String alias, 
		Class<? extends BasicItem> itemClass, Class<? extends BasicData> dataClass, 
		Set<Permission> definedPermissions, float deletionOrder)
	{
		this.value = value;
		this.displayValue = displayValue;
		this.alias = alias;
		this.itemClass = itemClass;
		this.dataClass = dataClass;
		this.definedPermissions = definedPermissions;
		this.deletionOrder = deletionOrder;
		Method tempGetById = null;
		Constructor<? extends BasicItem> tempConstructor = null;
		if (itemClass != null)
		{
			try
			{
				tempGetById = itemClass.getDeclaredMethod("getById", DbControl.class, int.class);
			}
			catch (NoSuchMethodException ex)
			{}
			try
			{
				tempConstructor = itemClass.getDeclaredConstructor(dataClass);
			}
			catch (NoSuchMethodException ex)
			{}
		}
		getById = tempGetById;
		constructor = tempConstructor;
	}

	public String toString()
	{
		return displayValue;
	}
	
	/**
		Get the integer value that is used when storing an item to the database.
	*/
	public int getValue()
	{
		return value;
	}
	
	/**
		Get the alias for this item, which is used in queries,
		@see net.sf.basedb.core.query.Query#getRootAlias()
	*/
	public String getAlias()
	{
		return alias;
	}
	
	/**
		Get the class object that is used to handle items of this type.
	*/
	public Class<? extends BasicItem> getItemClass()
	{
		return itemClass;
	}
	
	/**
		Get the data object that is used to store information in the
		database for items of this type.
	*/
	public Class<? extends BasicData> getDataClass()
	{
		return dataClass;
	}
	
	/**
		Get the permissions which are meaningful to assign to an item
		if this type. This is not used internally by the core, but is
		can be used by client applications to dynamically display
		a proper input form while assigning permissions.
		@return A <code>Set</code> containing the permissions
	*/
	public Set<Permission> getDefinedPermissions()
	{
		return definedPermissions;
	}
	
	/**
		Get a value indicating the deletion order. Items with a low value
		should be deleted before items with a high value, to minimize the risk
		that the {@link BasicItem#isUsed()} method returns true.
		@return The deletion order value
		@since 2.4
	*/
	public float getDeletionOrder()
	{
		return deletionOrder;
	}
	
	/**
		Get the item of the type represented by this enumeration constant with
		the specified id. Ie. <code>Item.SAMPLE.getById()</code> is the same
		as <code>Sample.getById</code>.

		@param dc The <code>DbControl</code> object used for database
			access and permission checking
		@param id The id of the item
		@return An item of the type represented by this enumeration constant
		@throws ItemNotFoundException If an item with the specified id isn't found
		@throws PermissionDeniedException If the logged in user hasn't read
			permission for the item
		@throws BaseException If there is another error
	*/
	public BasicItem getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		if (getById == null)
		{
			throw new ItemNotFoundException(itemClass+"[id="+id+"]");
		}
		else
		{
			try
			{
				return (BasicItem)getById.invoke(null, dc, id);
			}
			catch (Throwable ex)
			{
				Throwable cause = ex.getCause();
				if (cause instanceof BaseException)
				{
					throw (BaseException)cause;
				}
				else if (cause != null)
				{
					throw new BaseException(cause);
				}
				else
				{
					throw new BaseException(ex);
				}
			}
		}
	}

	/**
		Get the default constructor for new objects of this type. The
		default constructor is a constructor that takes a 
		<code>BasicData</code> object as the only parameter. For
		example <code>Sample(SampleData data)</code>.
		@return The constructor, or null if no such constructor exists
		@see DbControl#getItem(Class, BasicData, Object[])
	*/
	Constructor<? extends BasicItem> getConstructor()
	{
		return constructor;
	}

	/**
		Get the <code>Item</code> object when you know the integer code.
	 	@param value Integer code  
	 	@return an Item enumeration.
	*/
	public static Item fromValue(int value)
	{
		Item item = valueMapping.get(value);
		assert item != null : "item == null for value "+value;
		return item;
	}
	
	/**
		Get the <code>Item</code> object when you know the item class.
	 	@param itemClass Item class for which <code>Item</code> object should be returned 
	 	@return an Item enumeration.
	*/
	public static Item fromItemClass(Class<? extends BasicItem> itemClass)
	{
		Item item = itemClassMapping.get(itemClass);
		assert item != null : "item == null for itemClass "+itemClass;
		return item;
	}

	/**
		Get the <code>Item</code> object when you know the data class.
	 	@param dataClass Data class for which the Item should be returned. 
	 	@return an Item enumeration.
	*/
	public static Item fromDataClass(Class<? extends BasicData> dataClass)
	{
		Item item = dataClassMapping.get(dataClass);
		assert item != null : "item == null for dataClass "+dataClass;
		return item;
	}
	
	/**
		Get the <code>Item</code> object when you know either the item or
		data class.
	 	@param anyClass The class to get corresponding <code>Item</code> for. 
		@return The Item object, or null if no matching item is found
	*/
	@SuppressWarnings({"unchecked"})
	public static Item fromClass(Class<?> anyClass)
	{
		Item item = null;
		if (BasicItem.class.isAssignableFrom(anyClass))
		{
			item = fromItemClass((Class<BasicItem>)anyClass);
		}
		else if (BasicData.class.isAssignableFrom(anyClass))
		{
			item = fromDataClass((Class<BasicData>)anyClass);
		}
		return item;
	}
	
	/**
		Get the <code>Item</code> object when you have a data layer object.
		This method takes Hibernate proxies into account.
	 	@param data The data layer object for which corresponding 
	 		<code>Item</code> should be returned. 
	 	@return The Item object, or null if no matching item is found.
	*/
	@SuppressWarnings({"unchecked"})
	public static Item fromDataObject(BasicData data)
	{
		Class<? extends BasicData> dataClass = data.getClass();
		if (data instanceof org.hibernate.proxy.HibernateProxy)
		{
			dataClass = (Class<? extends BasicData>)org.hibernate.proxy.HibernateProxyHelper.getClassWithoutInitializingProxy(data);
		}
		return fromDataClass(dataClass);
	}
	
	public static final Comparator<Identifiable> DELETION_ORDER =
		new Comparator<Identifiable>()
		{
			public int compare(Identifiable i1, Identifiable i2)
			{
				Item e1 = i1.getType();
				Item e2 = i2.getType();
				if (e1 == e2)
				{
					// Delete item with higher ID first
					return i2.getId() - i1.getId();
				}
				else
				{
					// Delete items with lower deletionOrder first
					return (int)(e1.getDeletionOrder() - e2.getDeletionOrder());
				}
			}
		};
	
	/**
		Helper class to make it easier to set up the defined permissions.
		@see #getDefinedPermissions()
	*/
	private static class DefinedPermissions
	{
		private static final Set<Permission> read = Collections.unmodifiableSet(EnumSet.of(Permission.READ));
		private static final Set<Permission> write = Collections.unmodifiableSet(EnumSet.of(Permission.READ, Permission.USE, Permission.WRITE));
		private static final Set<Permission> basic = Collections.unmodifiableSet(EnumSet.of(Permission.CREATE, Permission.READ, Permission.USE, Permission.WRITE, Permission.DELETE));
		private static final Set<Permission> ownable = Collections.unmodifiableSet(EnumSet.of(Permission.CREATE, Permission.READ, Permission.USE, Permission.WRITE, Permission.DELETE, Permission.SET_OWNER));
		private static final Set<Permission> shareable = Collections.unmodifiableSet(EnumSet.of(Permission.CREATE, Permission.READ, Permission.USE, Permission.WRITE, Permission.DELETE, Permission.SET_OWNER, Permission.SET_PERMISSION));
	}

}
