/*
	$Id $

	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.util.filter.Filter;

/**
	A class with utility methods for plug-ins.

	@author Nicklas
	@version 2.4
	@base.modified $Date: 2011-03-17 13:47:15 +0100 (Thu, 17 Mar 2011) $
*/
public class ContextUtil 
{

	/**
		Call the {@link InteractivePlugin#isInContext(GuiContext, Object)} method
		for all plug-ins returned by the queries.
		
		@param dc The DbControl to use for database access
		@param pluginQuery A query returning plugin definition items
		@param configQuery A query returning plugin configuration items. The query must
			have defined a parameter <code>plugin</code> which restricts the query
			to only return configurations for a particular plugin definition
		@param context The current context
		@param currentItem The current item, or null if used from a list context
		@return A list of {@link ContextResult} objects
	*/
	public static List<ContextResult> checkContext(DbControl dc, ItemQuery<PluginDefinition> pluginQuery, 
		ItemQuery<PluginConfiguration> configQuery, GuiContext context, Object currentItem)
	{
		List<ContextResult> results = new LinkedList<ContextResult>();
		SessionControl sc = dc.getSessionControl();
		List<PluginDefinition> plugins = pluginQuery.list(dc);
		for (PluginDefinition plugin : plugins)
		{
			if (!plugin.requiresConfiguration())
			{
					results.add(new ContextResult(sc, plugin, null, context, currentItem));
			}
			if (configQuery != null && plugin.supportsConfigurations())
			{
				configQuery.setParameter("plugin", plugin.getId(), Type.INT);
				List<PluginConfiguration> configs = configQuery.list(dc);
				for (PluginConfiguration config : configs)
				{
					results.add(new ContextResult(sc, plugin, config, context, currentItem));
				}
			}
		}
		return results;
	}
	
	/**
		Count all plug-in / configuration combinations that can be used.
		@param results A collection of {@link ContextResult} objects
		@return The number of objects in the collection where
			{@link ContextResult#isInContext()} returns true
	*/
	public static int countUsuablePlugins(Collection<ContextResult> results)
	{
		int total = 0;
		if (results != null)
		{
			for (ContextResult result : results)
			{
				if (result.isInContext()) ++total;
			}
		}
		return total;
	}
	
	/**
		Get all messages from plug-in / configurations combinations that can't be used.
		
		@param results A collection of {@link ContextResult} objects
		@param includeMessage If warning messages should be included or not
		@param includeError If error messages should be include or not
		@return A set containing the error / warning message
	*/
	public static Set<String> getContextMessages(Collection<ContextResult> results, boolean includeMessage, boolean includeError)
	{
		Set<String> messages = new HashSet<String>();
		if (results != null)
		{
			for (ContextResult result : results)
			{
				if (!result.isInContext() && (includeMessage || (includeError && result.isError()))) 
				{
					messages.add(result.getName() + ": " + result.getContextMessage());
				}
			}
		}
		return messages;
	}
	
	/**
		Class for holding the result of a call to the 
		{@link InteractivePlugin#isInContext(GuiContext, Object)}
		method.
	*/
	public static class ContextResult
	{
		private final PluginDefinition plugin;
		private final PluginConfiguration config;
		private String contextMessage;
		private Throwable throwable;
		
		private ContextResult(SessionControl sc, PluginDefinition plugin, PluginConfiguration config,
			GuiContext context, Object currentItem)
		{
			this.plugin = plugin;
			this.config = config;
			InteractivePlugin ip = null;
			try
			{
				ip = plugin.newInstance(InteractivePlugin.class, sc, config, null);
				contextMessage = ip.isInContext(context, currentItem);
			}
			catch (Throwable t)
			{
				throwable = t;
				contextMessage = t.getMessage() == null ? t.getClass().getName() : t.getMessage();
			}
			finally
			{
				if (ip != null) ip.done();
			}
		}
		
		/**
			Create a new warning-level result object.
			@param plugin The plugin definition
			@param config The plugin configuration or null
			@param contextMessage The warning message, or null if there is no warning or error
		*/
		public ContextResult(PluginDefinition plugin, PluginConfiguration config, String contextMessage)
		{
			this.plugin = plugin;
			this.config = config;
			this.contextMessage = contextMessage;
		}

		/**
			Create a new error-level result object.
			@param plugin The plugin definition
			@param config The plugin configuration or null
			@param t The exception that caused the error
		*/
		public ContextResult(PluginDefinition plugin, PluginConfiguration config, Throwable t)
		{
			this.plugin = plugin;
			this.config = config;
			this.throwable = t;
			this.contextMessage = t.getMessage() == null ? t.getClass().getName() : t.getMessage();
		}
		
		/**
			Get the plugin definition that was checked.
		*/
		public PluginDefinition getPluginDefinition()
		{
			return plugin;
		}
		
		/**
			Get the plugin configuration that was checked, or null.
		*/
		public PluginConfiguration getPluginConfiguration()
		{
			return config;
		}
		
		/**
			Get the warning level message, or the message from
			{@link Throwable#getMessage()} if this is an error-level object.
		*/
		public String getContextMessage()
		{
			return contextMessage;
		}
		
		/**
			Get the exception that caused the error, or null if this is a
			warning-level object.
		*/
		public Throwable getError()
		{
			return throwable;
		}

		/**
			Get the name of the plug-in configuration or, if the configuration is null, 
			of the plug-in definition
		*/
		public String getName()
		{
			return config != null ? config.getName() : plugin.getName();
		}
		
		/**
			If the context check was successful and generate no warnings or error.
		*/
		public boolean isInContext()
		{
			return contextMessage == null && throwable == null;
		}
		
		/**
			If this is an error-level object.
		*/
		public boolean isError()
		{
			return throwable != null;
		}
		
	}
	
	/**
		Filter implementation that works on collections
		of {@link ContextResult}:s and return only those
		that return true from {@link ContextResult#isInContext()}.
	
		@author Nicklas
		@version 2.9
		@base.modified $Date: 2011-03-17 13:47:15 +0100 (Thu, 17 Mar 2011) $
	*/
	public static class IsInContextFilter
		implements Filter<ContextResult>
	{

		public IsInContextFilter()
		{}
		
		/*
			From the Filter interface
			-------------------------
		*/
		@Override
		public boolean evaluate(ContextResult result)
		{
			return result != null && result.isInContext();
		}
		// -----------------------------
	}
	
}
