/**
	$Id: LoggingErrorHandlerFactory.java 5488 2010-11-15 12:32:35Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import net.sf.basedb.util.Values;

import org.apache.log4j.Level;

/**
	Exception handler factory implementation that logs error messages
	using the default logging mechanism. This factory is it's own
	error handler and the implementation is thread-safe (once the level
	has been set).
	
	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-15 13:32:35 +0100 (Mon, 15 Nov 2010) $
*/
public class LoggingErrorHandlerFactory
	implements ErrorHandlerFactory<Action>, ErrorHandler<Action>
{
	
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.LoggingErrorHandler");

	private Level level;
	private boolean rememberLast;
	
	/**
		Create a new error-level loggin error handler factory.
	*/
	public LoggingErrorHandlerFactory()
	{
		this.level = Level.ERROR;
		this.rememberLast = false;
	}

	/**
		Create a new custom-level logging error handler factory.
	*/
	public LoggingErrorHandlerFactory(String level)
	{
		setLevel(level);
	}

	/*
		From the ErrorHandlerFactory interface
		--------------------------------------
	*/
	@Override
	public ErrorHandler<Action> getErrorHandler(InvokationContext<? extends Action> context)
	{
		return this;
	}
	// --------------------------------------

	/*
		From the ErrorHandler interface
		--------------------------------------
	*/
	@Override
	@SuppressWarnings("unchecked")
	public void handleError(InvokationContext<? extends Action> context, Action action, 
		String message, Throwable t)
	{
		if (rememberLast) ((InvokationContext)context).setError(action, t);
		log.log(level, message, t);
	}
	// --------------------------------------
	
	/**
		Set the level of the error handler.
	*/
	public void setLevel(String level)
	{
		this.level = Level.toLevel(level, Level.ERROR);
	}
	
	/**
		Set a flag indicating if errors should be remembered
		by the registry. The last error for an extension or 
		extension point can be retreived by calling 
		{@link Registry#getLastExtensionError(String)} and/or
		{@link Registry#getLastExtensionPointError(String)}.
		@param remember A string that evaluates to a boolean
	*/
	public void setRememberLast(String remember)
	{
		this.rememberLast = Values.getBoolean(remember);
	}
	
}
