/**
	$Id$

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.manager.processor;


import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.util.extensions.manager.ExtensionsFile.WriteableExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFile;
import net.sf.basedb.util.extensions.manager.ExtensionsFileProcessor;
import net.sf.basedb.util.extensions.manager.ExtensionsManager;
import net.sf.basedb.util.extensions.manager.ProcessResults;
import net.sf.basedb.util.extensions.xml.PluginInfo;

/**
	File processor implementation that disables plug-ins.
	
	@author Nicklas
	@since 3.0
	@base.modified $Date$
*/
public class DisablePluginsProcessor
	implements ExtensionsFileProcessor
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.extensions.manager.DisablePluginsProcessor");

	private final DbControl dc;
	private final ProcessResults results;
	private int numFiles;
	private int numError;
	private int numTotalPlugins;

	/**
		Create a new processor. 
		
		@param dc DbControl to use for accessing the database
	*/
	public DisablePluginsProcessor(DbControl dc, ProcessResults results)
	{
		this.dc = dc;
		this.results = results;
	}

	/*
		From the ExtensionsFileProcessor interface
		------------------------------------------
	*/
	@Override
	public void begin(ExtensionsManager manager, int numFiles) 
	{
		this.numFiles = 0;
		this.numError = 0;
		this.numTotalPlugins = 0;
	}

	@Override
	public void processFile(ExtensionsManager manager, WriteableExtensionsFile wFile) 
	{
		ExtensionsFile xtFile = wFile.getExtensionsFile();
		
		log.info("Disabling plug-ins in file: " + xtFile);
		try
		{
			wFile.open();
			int num = 0;
			for (PluginInfo info : xtFile.getObjectsOfClass(PluginInfo.class))
			{
				if (info.getInternalId() > 0)
				{
					PluginDefinition plugin = PluginDefinition.getById(dc, info.getInternalId());
					plugin.setDisabled(true);
					num++;
				}
			}
			
			numTotalPlugins += num;
			numFiles++;
			if (results != null)
			{
				if (num > 0) results.addMessage(xtFile, num + " plug-in(s) disabled.");
			}
			log.info("Disabled  " + num + " plug-ins from file: " + xtFile);
		}
		catch (Throwable ex)
		{
			wFile.setError(true);
			numError++;
			if (results != null)
			{
				results.addErrorMessage(xtFile, "Could not disable plug-ins: " + ex.getMessage());
			}
			log.error("Could not unregister extensions in file: " + xtFile, ex);
		}
		finally
		{
			wFile.close();
		}
	}

	@Override
	public void done(ExtensionsManager manager) 
	{}

	@Override
	public void done(ExtensionsManager manager, Throwable t)
	{}
	// ------------------------------------------
	
	public ProcessResults getProcessResults()
	{
		return results;
	}

	/**
		Get the number of files that was successfully processed.
	*/
	public int getNumFiles()
	{
		return numFiles;
	}
	
	/**
		Get the number of files that was had an error.
	*/
	public int getNumError()
	{
		return numError;
	}

	/**
		Get the total number of plug-in installed or updated
		by the processor.
	*/
	public int getNumPlugins()
	{
		return numTotalPlugins;
	}
	
}
