/**
	$Id: ValueConverter.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions.xml;

import java.lang.reflect.Method;

/**
	Interface for classes that wants to inspect/convert 
	parameters from the XML file before they are passed 
	to factory setter methods.
	
	<p>
	Instances of this class are registered with an XML loader
	with the {@link XmlLoader#addValueConverter(ValueConverter)} method. 
	Before a factory setter method is called, all registered
	converters will get the chance to {@link #convert(String, Method)} 
	the parameter value from the XML file. The converters are
	called in the order they are registered.
	
	<p>
	A typical implementation of a converter is to check if the
	setter method has been annotated with some specific annotation.
	If, so the value is converted, otherwise it is returned without
	modification.
	
	<pre class="code">
// Add a prefix to all strings.
&#64;Override
public String convert(String in, Method method) 
{
   if (!method.isAnnotationPresent(AddPrefix.class)) return in;
   return in == null ? PREFIX : PREFIX + in;
}
</pre>
	
	<p>
	Note! Since each registered converter is invoked everytime a
	factory setter method is about to be called, we recommend that
	the {@link #convert(String, Method)} method is kept as quick
	as possible.
	
	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public interface ValueConverter
{
	/**
		Convert a value. 
		
		@param in The value to convert
		@param method The factory setter method
		@return The converted or original value
	*/
	public String convert(String in, Method method);

}
