/**
	$Id: BasicItemNodeLoader.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.node.NodeFactory;
import net.sf.basedb.util.overview.node.NodeNameGenerator;
import net.sf.basedb.util.overview.validator.NodeValidator;
import net.sf.basedb.util.overview.validator.NodeValidatorFactory;

/**
	Abstract class that is useful for all node loaders that works
	with {@link BasicItem}:s.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
*/
public abstract class BasicItemNodeLoader<I extends BasicItem>
	extends AbstractNodeLoader<I>
{
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.overview.loader.BasicItemNodeLoader");
	private static final boolean debug = log.isDebugEnabled();

	public static final boolean ALLOW_ROOT_NODE = true;
	public static final boolean DENY_ROOT_NODE = false;
	
	protected final Object factoryKey;
	protected final boolean allowAsRootNode;
	protected final NodeNameGenerator<I> nameGenerator;
	
	/**
		Create a new node loader.
		
		@param factoryKey The key to use when looking up a {@link NodeValidator}
			with {@link NodeValidatorFactory#createNodeValidator(Object)}
		@param allowAsRootNode If FALSE, the {@link 
			#createRootNode(DbControl, OverviewContext, BasicItem)} method
			throws an execption
		@param nameGenerator A name generator implementation that know how to
			create name and title for nodes
	*/
	protected BasicItemNodeLoader(Object factoryKey, boolean allowAsRootNode, NodeNameGenerator<I> nameGenerator)
	{
		this.factoryKey = factoryKey;
		this.allowAsRootNode = allowAsRootNode;
		this.nameGenerator = nameGenerator;
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createRootNode(DbControl dc, OverviewContext context, I item)
	{
		if (debug) log.debug("Create root node: " + item);
		if (!allowAsRootNode) throw new UnsupportedOperationException();
		NodeFactory<I> nf = getNodeFactory(dc, context);
		Node node = nf.createNode(item, item, null, ChildNodeDirection.ALL);
		if (debug) log.debug("Created: " + node);
		return node;
	}
	// -------------------------------
	
	/**
		Create a new item node using the supplied node factory. If <code>denied==true</code>
		the node will be created with {@link NodeFactory#createDeniedNode(Node)}. If
		<code>item==null</code> the node will be created with 
		{@link NodeFactory#createMissingNode(Node)}. Otherwise the node is created with
		{@link NodeFactory#createNode(BasicItem, Object, Node, ChildNodeDirection)}
		
		@param factory The node factory to use
		@param item The item that is contained by the node (can be null)
		@param cacheKey Optional key for cache lookup. If not null and an
			entry is found in the cache, the entire sub-tree is duplicated
			to avoid re-loading the same items again
		@param denied TRUE if the logged in user was denied access to the item
		@param parentNode The parent node
		@param direction The direction for loading children
	*/
	protected Node createItemNode(NodeFactory<I> factory, I item, Object cacheKey, boolean denied, 
		Node parentNode, ChildNodeDirection direction)
	{
		Node node = null;
		if (denied)
		{
			if (debug) log.debug("Create denied node: parent=" + parentNode);
			node = factory.createDeniedNode(parentNode);
		}
		else if (item == null)
		{
			if (debug) log.debug("Create missing node: parent=" + parentNode);
			node = factory.createMissingNode(parentNode);
		}
		else
		{
			if (debug) log.debug("Create node: item=" + item + "; parent=" + parentNode);
			node = factory.createNode(item, cacheKey, parentNode, direction);
		}
		if (debug) log.debug("Node created: " + node);
		return node;
	}
	
	protected void postValidateFolder(NodeFactory<I> nf, Node folderNode, Node parentNode, boolean createMissingNode)
	{
		if (createMissingNode && folderNode == null) nf.createMissingNode(parentNode);
		nf.postValidateFolder(folderNode, parentNode);
	}
	
	
	/**
		Get the node loader factory used by the current context. The factory
		must be a factory that create BasicItem node loaders.
		@param context The current context
		@see OverviewContext#getNodeLoaderFactory()
	*/
	@SuppressWarnings("unchecked")
	protected NodeLoaderFactory<BasicItem, Object> getNodeLoaderFactory(OverviewContext context)
	{
		return context.getNodeLoaderFactory();
	}
	
	/**
		Get a node loader for a specific item type using the current context's 
		node loader factory.
		@param context The current context
		@param key The item type that the factory should create a node loader for
		@return A node loader object
		@see OverviewContext#getNodeLoaderFactory()
	*/
	protected NodeLoader<? extends BasicItem> getNodeLoader(OverviewContext context, Object key)
	{
		return getNodeLoaderFactory(context).createNodeLoader(key);
	}
	
	/**
		Get the node validator factory used by the current context. The factory
		must be a factory that create BasicItem validators.
		@param context The current context
		@see OverviewContext#getNodeValidatorFactory()
	*/
	@SuppressWarnings("unchecked")
	protected NodeValidatorFactory<BasicItem, Object> getNodeValidatorFactory(OverviewContext context)
	{
		return context.getNodeValidatorFactory();
	}
	
	/**
		Get the default node validator for nodes of this type. The node validator is
		looked up using the factory key provided in the constructor.
		This method is a shortcut for
		<code>context.getNodeValidatorFactory().createNodeValidator(factoryKey)</code>
		@param context The current overview context
		@return A node validator, or null if no validator exists
	*/
	@SuppressWarnings("unchecked")
	protected NodeValidator<I> getNodeValidator(OverviewContext context)
	{
		return (NodeValidator<I>)getNodeValidatorFactory(context).createNodeValidator(factoryKey);
	}
	
	/**
		Get a node factory that can be used to create item nodes. The node factory
		will use the default node validator as returned by {@link #getNodeValidator(OverviewContext)}
		and the name generator as provided in the constructor.
		
		@param dc A DbControl that can be used for database access
		@param context The current overview context
		@return A node factory
		@see #getNodeFactory(DbControl, OverviewContext, NodeValidator, NodeNameGenerator)
	*/
	protected NodeFactory<I> getNodeFactory(DbControl dc, OverviewContext context)
	{
		NodeValidator<I> nv = getNodeValidator(context);
		return new NodeFactory<I>(dc, context, nv, nameGenerator);
	}
	
	/**
		Get a node factory that can be used to create item nodes using
		a non-default name generator and validator.
		
		@param dc A DbControl that can be used for database access
		@param context The current overview context
		@param validator A node validator, or null if no validation is required
		@param nameGenerator A name generator
		@return A node factory
		@see #getNodeFactory(DbControl, OverviewContext)
	*/
	protected NodeFactory<I> getNodeFactory(DbControl dc, OverviewContext context, 
		NodeValidator<I> validator, NodeNameGenerator<I> nameGenerator)
	{
		return new NodeFactory<I>(dc, context, validator, nameGenerator);
	}
	
}
