/**
	$Id: SampleLoader.java 5807 2011-10-14 13:08:23Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import java.util.Iterator;

import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultIterator;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Sample;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;

/**
	Node loader implementation for samples. The forward-loading
	direction goes from {@link BioSource}:s -&gt; {@link Sample}:s -&gt; 
	{@link Extract}:s. The reverse-loading direction is the opposite 
	direction. In both cases we also need to consider pooled samples.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2011-10-14 15:08:23 +0200 (Fri, 14 Oct 2011) $
*/
public class SampleLoader
	extends BasicItemNodeLoader<Sample>
{

	public SampleLoader()
	{
		super(Item.SAMPLE, ALLOW_ROOT_NODE, 
				new NameableNameGenerator<Sample>("sample", "Sample"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	/**
		Create forward-loading sample nodes from a given parent biosource
		or sample. In both cases the returned node is a folder-type node
		that contains the child samples.
		@return The folder node for the samples
	*/
	@Override
	public Node createForwardNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.BIOSOURCE)
		{
			returnNode = createForwardNode((BioSource)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentType == Item.SAMPLE)
		{
			returnNode = createForwardNode((Sample)parentNode.getItem(dc), dc, context, parentNode);
		}
		return returnNode;
	}
	
	/**
		Create reverse-loading sample nodes from an extract node
		or from a pooled sample. In the first case, the returned node is an
		item-type node with the parent sample. In the second case, the returned
		node is a folder-type node with the pooled samples. 
		@return A folder-type or item-type node
	*/
	@Override
	public Node createReverseNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.EXTRACT)
		{
			returnNode = createSingleParentReverseNode((Extract)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentType == Item.SAMPLE)
		{
			Sample s = (Sample)parentNode.getItem(dc);
			if (s.hasSingleParent())
			{
				returnNode = createSingleParentReverseNode(s, dc, context, parentNode);
			}
			else
			{
				returnNode = createPooledReverseNode(s, dc, context, parentNode);
			}
		}
		return returnNode;
	}
	// --------------------------------
	/*
		From the AbstractNodeLoader class
		----------------------------------
	*/
	/**
		Loads property nodes of a sample. 
		<ul>
		<li>Annotations: {@link AnnotationLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Protocol: {@link ProtocolLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		</ul>
	*/
	@Override
	protected void loadPropertyChildNodes(DbControl dc, OverviewContext context, Node sampleNode)
	{
		getNodeLoader(context, Item.ANNOTATION).createPropertyNode(dc, context, sampleNode);
		getNodeLoader(context, Item.PROTOCOL).createPropertyNode(dc, context, sampleNode);
		getNodeLoader(context, Item.ANYTOANY).createPropertyNode(dc, context, sampleNode);
	}

	/**
		Loads the pooled samples and extracts that has been created from
		this sample.
		@see ExtractLoader#createForwardNode(DbControl, OverviewContext, Node)
		@see SampleLoader#createForwardNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadForwardChildNodes(DbControl dc, OverviewContext context, Node sampleNode)
	{
		getNodeLoader(context, Item.EXTRACT).createForwardNode(dc, context, sampleNode);
		getNodeLoader(context, Item.SAMPLE).createForwardNode(dc, context, sampleNode);
	}
	
	/**
		If the sample is a pooled sample, loads the parent samples that
		it was pooled from. Otherwise load the parent biosource.
		@see SampleLoader#createReverseNode(DbControl, OverviewContext, Node)
		@see BioSourceLoader#createReverseNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadReverseChildNodes(DbControl dc, OverviewContext context, Node sampleNode)
	{
		Sample sample = (Sample)sampleNode.getItem(dc);
		if (sample.getParentType() == Item.SAMPLE)
		{
			getNodeLoader(context, Item.SAMPLE).createReverseNode(dc, context, sampleNode);
		}
		else
		{
			getNodeLoader(context, Item.BIOSOURCE).createReverseNode(dc, context, sampleNode);
		}
	}
	// ---------------------------------------
	
	/**
		Create reverse-loading samples nodes of a sample with other sample(s) as parent items. 
		Eg. the child nodes are the source sample(s) that was used to create the given product.
	*/
	@SuppressWarnings("unchecked")
	private Node createPooledReverseNode(Sample product, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Sample> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Sample> query = (ItemQuery<Sample>)context.initQuery(product.getCreationEvent().getSources(), "name");
		Iterator<Sample> it = query.iterate(dc);
		int numSamples = 0;
		while (it.hasNext())
		{
			numSamples++;
			Sample source = it.next();
			// We need to handle circular references inline because
			// we can never require that a validator implementation handles this
			Node circularNode = OverviewUtil.getCircularRef(parentNode, source);
			if (circularNode != null)
			{
				context.createFailure(Validator.CIRCULAR_REFERENCE, parentNode, 
						"Circular reference to '" + source.getName() + "'", 
						new Fix("Modify parent samples of '" + product.getName() + "'", product),
						new Fix("Modify parent samples of '" + source.getName() + "'", source)
					);
			}
			else
			{
				if (folderNode == null)
				{
					folderNode = new Node("pooled", "Parent samples", parentNode, ChildNodeDirection.REVERSE);
				}
				createItemNode(nf, source, source, false, folderNode, ChildNodeDirection.REVERSE);
			}
		}
		postValidateFolder(nf, folderNode, parentNode, numSamples == 0);
		return folderNode;
	}

	/**
		Create a reverse-loading sample node from an extract or sample. Eg. the child node
		is the sample that is the parent of the (non-pooled) extract/sample.
	*/
	private Node createSingleParentReverseNode(MeasuredBioMaterial bioMaterial, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Sample> nf = getNodeFactory(dc, context);
		Sample sample = null;
		boolean denied = false;
		try
		{
			sample = (Sample)bioMaterial.getParent();
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node sampleNode = createItemNode(nf, sample, sample, denied, 
				parentNode, ChildNodeDirection.REVERSE);
		return sampleNode;
	}

	/**
		Create forward-loading sample nodes for the samples that has a given
		biosource as their parent.
	*/
	private Node createForwardNode(BioSource bioSource, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Sample> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Sample> query = context.initQuery(bioSource.getSamples(), "name");
		ItemResultIterator<Sample> it = query.iterate(dc);
		while (it.hasNext())
		{
			Sample sample = it.next();
			if (folderNode == null)
			{
				folderNode = new Node("samples", "Child samples", parentNode, ChildNodeDirection.FORWARD);
			}
			createItemNode(nf, sample, sample, false, folderNode, ChildNodeDirection.FORWARD);
		}
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}
	
	/**
		Create forward-loading sample nodes for the (pooled) samples that has a given
		sample as their parent.
	*/
	private Node createForwardNode(Sample parentSample, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<Sample> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<Sample> query = context.initQuery(parentSample.getChildSamples(), "name");
		ItemResultIterator<Sample> it = query.iterate(dc);
		while (it.hasNext())
		{
			Sample childSample = it.next();
			// We need to handle circular references inline because
			// we can never require that a validator implementation handles this
			Node circularNode = OverviewUtil.getCircularRef(parentNode, childSample);
			if (circularNode != null)
			{
				context.createFailure(Validator.CIRCULAR_REFERENCE, parentNode, 
					"Circular reference to '" + childSample.getName() + "'", 
					new Fix("Modify parent samples of '" + childSample.getName() + "'", childSample),
					new Fix("Modify parent samples of '" + parentSample.getName() + "'", parentSample)
					);
			}
			else
			{
				if (folderNode == null)
				{
					folderNode = new Node("samples", "Child samples", parentNode, ChildNodeDirection.FORWARD);
				}
				createItemNode(nf, childSample, childSample, false, folderNode, ChildNodeDirection.FORWARD);
			}
		}		
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}
	
}
