/**
	$Id: HardwareValidator.java 5814 2011-10-18 11:46:59Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.Subtypable;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.Node;

/**
	Validator implementation for hardware. Validation rules:
	<ul>
	<li>Missing item: {@link Validator#MISSING_HARDWARE}
	<li>Access denied: {@link Validator#DENIED_HARDWARE}
	<li>Incorrect hardware type: {@link Validator#INCORRECT_HARDWARETYPE} 
	<li>Non-default hardware: {@link Validator#NONDEFAULT_HARDWARE}
	</ul>

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2011-10-18 13:46:59 +0200 (Tue, 18 Oct 2011) $
*/
public class HardwareValidator
	extends NameableNodeValidator<Hardware>
{
	
	public HardwareValidator()
	{
		super(null, Validator.DENIED_HARDWARE);
	}

	
	/* 
		From BasicValidator class
		-------------------------
	*/
	/**
		If the parent item has a subtype that is related to a HARDWARE subtype, report
		the missing item as a {@link Validator#MISSING_HARDWARE} failure. Otherwise,
		ignore the missing hardware.
		@return Always false
	*/
	@Override
	public boolean preMissingItem(DbControl dc, OverviewContext context, Node parentNode)
	{
		BasicItem parentItem = parentNode.getItem(dc);
		ItemSubtype expectedSubtype = getExpectedHardwareSubtype(dc, parentItem);
		if (expectedSubtype != null)
		{
			context.createFailure(Validator.MISSING_HARDWARE, parentNode, 
					"Missing hardware: " + expectedSubtype.getName(), getMissingItemFix(dc, parentNode));
		}
		return false;
	}
	
	/**
		Checks if the hardware is of the correct hardware type
		and, if a project is active, the hardware is included as a
		default hardware for the project.
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{
		super.postValidate(dc, context, node, parentNode);
		Hardware hardware = (Hardware)node.getItem(dc);
		
		BasicItem parentItem = parentNode.getItem(dc);
		ItemSubtype expectedSubtype = getExpectedHardwareSubtype(dc, parentItem);
		try
		{
			ItemSubtype subtype = hardware.getItemSubtype();
			if (expectedSubtype != null && !expectedSubtype.equals(subtype))
			{
				ItemSubtype mySubtype = ((Subtypable)parentItem).getItemSubtype();
				Fix subtypeFix = subtype == null ? null : 
					new Fix("Change related hardware subtype of '" + mySubtype.getName() + 
						"' to '" + subtype.getName() + "'", mySubtype);

				context.createFailure(Validator.INCORRECT_HARDWARETYPE, parentNode, 
					"Expected a hardware with subtype: " + expectedSubtype.getName(),
					new Fix("Change hardware of '" + parentNode.getTitle() + "'", parentItem),
					new Fix("Change hardware type of '" + hardware.getName() + "' to '" + expectedSubtype.getName() + "'", hardware),
					subtypeFix
				);
			}
		}
		catch (PermissionDeniedException ex)
		{}

		// Check if using project default hardware
		Project project = context.getProject();
		if (project != null)
		{
			if (!project.isDefaultItem(hardware) && project.hasDefaultForItem(hardware))
			{
				context.createFailure(Validator.NONDEFAULT_HARDWARE, parentNode, null,
					new Fix("Change hardware of '" + parentNode.getTitle() + "'", parentItem),
					new Fix("Add '" + hardware.getName() + "' as default hardware to project", project)
				);
			}
		}
	}
	
	/**
		@return Suggested fix is to add a hardware to the parent item
	*/
	@Override
	protected Fix getMissingItemFix(DbControl dc, Node parentNode)
	{
		BasicItem parentItem = parentNode.getItem();		
		return new Fix("Add hardware to '" + parentNode.getTitle() + "'", parentItem);
	}
	// --------------------------------
	
	
	/**
		Get the subtype of the associated hardware that we expect for the given
		parent item.
	*/
	protected ItemSubtype getExpectedHardwareSubtype(DbControl dc, BasicItem parentItem)
	{
		ItemSubtype expectedSubtype = null;
		int defaultSubtypeForItemType = SystemItems.getId(Hardware.getDefaultSystemId(parentItem.getType()));
		try
		{
			if (parentItem instanceof Subtypable)
			{
				expectedSubtype = ItemSubtype.getRelatedSubtype(dc, 
						(Subtypable)parentItem, Item.HARDWARE, defaultSubtypeForItemType);
			}
			else if (defaultSubtypeForItemType != 0)
			{
				expectedSubtype = ItemSubtype.getById(dc, defaultSubtypeForItemType);
			}
		}
		catch (PermissionDeniedException ex)
		{}
		return expectedSubtype;
	}

	
}
