/**
	$Id: UnitUtil.java 4711 2008-12-18 08:22:27Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.units;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Unit;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.Values;

/**
	A utility class for working with {@link Unit}:s.

	@author Nicklas
	@version 2.9
	@base.modified $Date: 2008-12-18 09:22:27 +0100 (Thu, 18 Dec 2008) $
*/
public class UnitUtil
{
	/**
		Pattern for checking if a string is a numeric value with an
		optional unit. Used by {@link #splitValueWithUnit(String)}
	*/
	public static final Pattern VALUE_WITH_UNIT = 
		Pattern.compile("\\s*([+-]?\\d+(?:\\.\\d+)?(?:[eE][+-]?\\d+)?)\\.?\\s*(.*)");
	
	/**
		Find the unit with a given symbol for a given quantity. Use
		this method only when you need to find few units. Each invokation
		results in a new database query. If you need to lookup mulitple units,
		consider using a {@link UnitCache} instead.

		@param dc The DbControl to use for database access
		@param quantityId The system ID of the quantity
		@param symbol The unit symbol
		@return A unit object, or null if no unit is found
	*/
	public static Unit getUnit(DbControl dc, String quantityId, String symbol)
	{
		ItemQuery<Unit> query = Unit.getQuery();
		query.join(Hql.innerJoin("symbols", "smb"));
		query.restrict(Restrictions.eq(Hql.property("quantity.systemId"), Expressions.string(quantityId)));
		query.restrict(Restrictions.eq(Hql.property("smb", "symbol"), Expressions.string(symbol)));
		List<Unit> list = query.list(dc);
		return list.isEmpty() ? null : list.get(0);
	}
	
	/**
		Splits a value with an optional unit. The input string 
		should start with a numerical value and optionally end
		with a unit. If the value is not numeric, null is returned,
		otherwise an array with two elements is returned. The 0th
		element is the numeric part, and the 1st is the unit 
		part. The unit part is null if the value doesn't have a unit.
		White-space has been stripped from all values.
		@param value The value to split
		@return An array of length 2, or null
	*/
	public static String[] splitValueWithUnit(String value)
	{
		if (value == null) return null;
		String[] result = null;
		Matcher m = VALUE_WITH_UNIT.matcher(value);
		if (m.matches())
		{
			result = new String[2];
			result[0] = m.group(1);
			result[1] = Values.getStringOrNull(m.group(2));
		}
		return result;
	}
}
