/**
	$Id: HttpConnectionManager.java 5582 2011-03-15 12:55:59Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.uri.http;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;

import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.util.HttpUtil;
import net.sf.basedb.util.ssl.SSLUtil;
import net.sf.basedb.util.uri.CloseResourceInputStream;
import net.sf.basedb.util.uri.ConnectionParameters;
import net.sf.basedb.util.uri.ConnectionManager;
import net.sf.basedb.util.uri.UriMetadata;

/**
	Connection manager implementation that support HTTP and HTTPS URI:s. 
	It supports Basic and Digest authentication schemes. When HTTPS
	is used it provides support for validation of both server- and 
	client-side certificates.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-03-15 13:55:59 +0100 (Tue, 15 Mar 2011) $
*/
public class HttpConnectionManager
	implements ConnectionManager
{

	private final URI uri;
	private final ConnectionParameters parameters;
	private UriMetadata metadata;
	
	public HttpConnectionManager(URI uri, ConnectionParameters parameters)
	{
		if (uri == null) throw new NullPointerException("uri");
		this.uri = uri;
		this.parameters = parameters;
	}
	
	/*
		From the UriHandler interface
		-----------------------------
	*/
	@Override
	public URI getURI()
	{
		return uri;
	}

	@Override
	public InputStream getInputStream() 
		throws IOException
	{
		URI uri = getURI();
		HttpClient client = null;
		InputStream stream = null;
		try
		{
			HttpGet get = new HttpGet(uri);
			client = createHttpClient(uri, parameters);
			HttpResponse response = client.execute(get);
			if (metadata == null) metadata = createMetadata(uri, response);
			HttpEntity entity = response.getEntity();
			if (entity != null)
			{
				// Wrap the stream so that stream.close() will also shutdown the client
				stream = new CloseResourceInputStream(entity.getContent(), new CloseableHttpClient(client));
			}
		}
		finally
		{
			// If no stream has been created we shutdown the client immediately
			if (stream == null && client != null) HttpUtil.shutdown(client);
		}
		return stream;
	}

	@Override
	public UriMetadata getMetadata() 
		throws IOException
	{
		if (metadata == null) 
		{
			URI uri = getURI();
			HttpClient client = null;
			try
			{
				HttpHead head = new HttpHead(uri);
				client = createHttpClient(uri, parameters);
				HttpResponse response = client.execute(head);

				int responseCode = response.getStatusLine().getStatusCode();
				if (responseCode >= 200 && responseCode < 300)
				{
					metadata = createMetadata(uri, response);
				}
				else if (responseCode == 404)
				{
					throw new InvalidDataException("404 URL not found: " + uri);
				}
				else if (responseCode == 401)
				{
					throw new InvalidDataException("401 Authorization required: " + uri);
				}
				else
				{
					throw new IOException(responseCode + " " + response.getStatusLine().getReasonPhrase());
				}
			}
			finally
			{
				if (client != null) HttpUtil.shutdown(client);
			}
		}
		return metadata;
	}
	// --------------------------------------
	
	/**
		Get a HttpClient object that has been configured to access the 
		given URI.
	*/
	public HttpClient createHttpClient(URI uri, ConnectionParameters parameters)
		throws IOException
	{
		DefaultHttpClient httpClient = new DefaultHttpClient();
	
		// Do we need to provide username/password?
		if (parameters != null && parameters.getUsername() != null)
		{
			Credentials credentials = new UsernamePasswordCredentials(parameters.getUsername(), parameters.getPassword());
			httpClient.getCredentialsProvider().setCredentials(AuthScope.ANY, credentials);
		}
	
		if (uri.getScheme().equals("https"))
		{
			// Adds support for https urls
			SSLSocketFactory sslFactory = null;
			if (parameters == null)
			{
				sslFactory = SSLUtil.getSSLSocketFactory();
			}
			else
			{
				sslFactory = SSLUtil.getSSLSocketFactory(parameters.getServerCertificate(), 
						parameters.getClientCertificate(), parameters.getClientCertificatePassword());
			}
			Scheme https = new Scheme("https", sslFactory, 443);
			httpClient.getConnectionManager().getSchemeRegistry().register(https);
		}
	   return httpClient;
	}
	
	/**
		Read metadata from the given http response and put the information
		in a UriMetadata object. This method uses whatever information that
		is in the response headers:
		<ul>
		<li>Content-Length: The length of the resource
		<li>Last-Modified: The last modified date
		<li>Content-Type: The MIME type and character set
		</ul>
	*/
	public UriMetadata createMetadata(URI uri, HttpResponse response)
	{
		UriMetadata meta = new UriMetadata(uri);
		meta.setLength(HttpUtil.getLong(response, "Content-Length", null));
		meta.setLastModified(HttpUtil.getLastModified(response));
		String mimeType = HttpUtil.getContentType(response);
		String charset = null;
		if (mimeType != null)
		{
			Pattern p = Pattern.compile("(\\w+/\\w+)\\s*;\\s*charset=(.*)");
			Matcher m = p.matcher(mimeType);
			if (m.matches())
			{
				mimeType = m.group(1);
				charset = m.group(2);
			}
		}
		meta.setMimeType(mimeType);
		meta.setCharacterSet(charset);
		return meta;
	}
	
	/**
		Closable implementation that closes the given HttpClient
		when the close() method is called.
	*/
	public static class CloseableHttpClient
		implements Closeable
	{

		private final HttpClient client;
		public CloseableHttpClient(HttpClient client)
		{
			this.client = client;
		}
		
		@Override
		public void close() 
			throws IOException
		{
			HttpUtil.shutdown(client);
		}
	}
	
}
