/*
	$Id: TestItemSubtype.java 5696 2011-08-12 12:51:53Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.Set;

public class TestItemSubtype
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing item subtypes");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(Item.EXTRACT, "Test labeled extract");
		int id2 = test_create(Item.EXTRACT, "Test library");
		test_load(id);
		test_list(-1);

		// Extra tests:
		int extractId = TestExtract.test_create(0, "Test extract", 0, 0, true);
		test_set_on_item(id, extractId);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestExtract.test_delete(extractId);
		test_delete(id);
		test_delete(id2);
		write("++Testing item subtypes "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(Item itemType, String name, int... relatedSubtypes)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.ITEMSUBTYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			ItemSubtype subtype = ItemSubtype.getNew(dc, itemType);
			subtype.setName(name);
			subtype.setDescription("Added at "+new Date());
			if (relatedSubtypes != null)
			{
				for (int i = 0; i < relatedSubtypes.length; ++i)
				{
					subtype.setRelatedSubtype(ItemSubtype.getById(dc, relatedSubtypes[i]));
				}
			}
			dc.saveItem(subtype);
			dc.commit();
			id = subtype.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(subtype, false);
			write_item(0, subtype);
			write("--Create item subtype OK");
		}
		catch (Throwable ex)
		{
			write("--Create item subtype FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemSubtype subtype  = ItemSubtype.getById(dc, id);
			write_item(0, subtype);
			write("--Load item subtype OK");
		}
		catch (Throwable ex)
		{
			write("--Load item subtype FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<ItemSubtype> l = ItemSubtype.getQuery(null).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List item subtypes OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List item subtypes FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemSubtype subtype = ItemSubtype.getById(dc, id);
			dc.deleteItem(subtype);
			Set<ItemProxy> using = subtype.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(subtype + " is used by " + using);
			}
			dc.commit();
			write("--Delete item subtype OK");
		}
		catch (Throwable ex)
		{
			write("--Delete item subtype FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tItem type\tDescription");
			write("-- \t-- \t--------- \t---------\t-----------");
		}
	}
	static void write_item(int i, ItemSubtype subtype)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+subtype.getId()+"\t"+subtype.getName()+"\t" + subtype.getMainItemType() +
					"\t"+subtype.getDescription());
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}


	static void test_set_on_item(int subtypeId, int itemId)
	{
		if (subtypeId == 0 || itemId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemSubtype subtype  = ItemSubtype.getById(dc, subtypeId);
			Subtypable item = (Subtypable)subtype.getMainItemType().getById(dc, itemId);
			item.setItemSubtype(subtype);
			dc.commit();
			dc = TestUtil.getDbControl();
			item = (Subtypable)item.getType().getById(dc, itemId);
			if (!TestUtil.getSilent())
			{
				write("--" + item + "; subtype=" + item.getItemSubtype());
			}
			write("--Set item subtype OK");
		}
		catch (Throwable ex)
		{
			write("--Set item subtype FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
	}


}

