/* $Id: main.js 5947 2012-02-06 14:12:05Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	Contains a lot of JavaScript functions that should be available
	on every page.

	@author Nicklas
	@version 2.0
*/

var Main = new MainClass();
// Holder for all functions in this script
function MainClass()
{
	this.onLoadFunctions = [];
	this.hasAddedOnLoadEventListener = false;
	this.hasCalledEventListeners = false;
	
	/**
		Add a function that is going to be executed as soon as the page
		has been loaded. This is equivalent to setting window.onload but
		support calling multiple functions. The functations are called 
		in the reverse order as they were added to this method.
	*/
	this.onLoad = function(f)
	{
		if (!f && !f.call) return;
		this.onLoadFunctions[this.onLoadFunctions.length] = f;
		if (!this.hasAddedOnLoadEventListener) 
		{
			this.hasAddedOnLoadEventListener = true;
			if (document.addEventListener)
			{
				// Firefox and compatible
				document.addEventListener('DOMContentLoaded', this.onLoadEventListener, false);
				// Just in case the 'DOMContentLoaded' event is not implemented
				window.addEventListener('load', this.onLoadEventListener, false);
			}
			else if (window.attachEvent)
			{
				// IE
				window.attachEvent('onload', this.onLoadEventListener);
			}
			else
			{
				window.onload = this.onLoadEventListener;
			}
		}
	}
	
	/**
		This event lister is attached to the window.onload event and
		executes all functions registered with Main.onLoad() when the
		event is received. This method will call the registered functions
		at most once.
	*/
	this.onLoadEventListener = function()
	{
		if (Main.hasCalledEventListeners) return;
		Main.hasCalledEventListeners = true;
		//for (var i = 0; i < Main.onLoadFunctions.length; i++)
		for (var i = Main.onLoadFunctions.length - 1; i >= 0; i--)
		{
			try
			{
				Main.onLoadFunctions[i].call();
			}
			catch (err)
			{}  // Ignore
		}
	}
	
	/*
		Opens a popup window with the specified name and size and loads the given url
	
		@param url The url to load in the window
		@param name The name of the window
		@param width The width in pixels of the window
		@param height The height in pixels of the window
		@param options Specifies options for the decoration of the window as defined
			by the JavaScript window.open() method
		@param parentWin The window object to use when calling open(). If not given
			the current window is used
		@see http://devedge.netscape.com/library/manuals/2000/javascript/1.3/reference/window.html#1202731
	*/
	this.openPopup = function(url, name, width, height, options, parentWin)
	{
		if (!parentWin) parentWin = window;
		if (window.top != window && window.top.Main) 
		{
			window.top.Main.openPopup(url, name, width, height, options, parentWin);
			return;
		}
		
		// Rescale the width and height
		var scale = getScale();
		width = width * scale;
		height = height * scale;
		/*
			We will try to position the popup window in the center of
			the parent window, or at the center of the screen. Different browsers
			have different properties for the width and height.
		*/
		var wTop = window.top.screenY;
		if (wTop == undefined) wTop = window.top.screenTop;
		if (wTop == undefined) wTop = 0;
	
		var wLeft = window.top.screenX;
		if (wLeft == undefined) wLeft = window.top.screenLeft;
		if (wLeft == undefined) wLeft = 0;
	
		var wHeight = Main.getWindowHeight(window.top);
		var wWidth = Main.getWindowWidth(window.top);
		
		var top = Math.ceil(wTop+(wHeight-height) / 2);
		var left = Math.ceil(wLeft+(wWidth-width) / 2);
		if (top < 0 && wTop >= 0) top = 0;
		if (left < 0 && wLeft >= 0) left = 0;
		if (options == null) options = "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=yes,resizable=yes";
		options = "top="+top+",left="+left+",width="+width+",height="+height+"," + options;

		var newWin = Main.getPageValue('open.windows', name);
		if (newWin && !newWin.closed)
		{
			var msg = 'This window has already been opened. ';
			msg += 'Opening it again causes unsaved changes to be lost. Continue?';
			if (!confirm(msg))
			{
				return;
			}
			newWin.close();
			url += '&warnIfOpen=0';
		}
		if (Main.isTooLongURL(url))
		{
			newWin = window.open(getRoot()+'blank.jsp', name, options);
			Main.safeSetLocation(url, newWin, true);
		}
		else
		{
			newWin = parentWin.open(url, name, options);
		}
		Main.setPageValue('open.windows', name, newWin);
		setTimeout("Main.focusWindow('" + name + "')", 100);
	}

	/**
		Brings a named window to focus.
	*/
	this.focusWindow = function(windowName)
	{
		var win = this.getPageValue('open.windows', windowName);
		if (win) win.focus();
	}
	
	/**
		Get a reference to a named window. If the window
		doesn't exists or if it is closed, a reference to
		the current window is returned.
	 */
	this.getWindow = function(windowName)
	{
		var win = this.getPageValue('open.windows', windowName);
		if (!win || win.closed) win = window;
		return win;
	}
	
	/*
		Opens a popupwindow with a larger textarea field. The text is taken from
		the specified textarea of the specified form and is saved back to
		when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the textarea is located
		@param textarea The name of the textarea to zoom
	*/
	this.zoom = function(title, form, textarea, disabled, callback)
	{
		var url = getRoot()+'common/zoom.jsp?title='+title+'&form='+form+'&textarea='+textarea;
		url += '&disabled='+(disabled ? '1' : '0');
		url += '&ID='+this.getIdFromLocation();
		if (callback) url += '&callback='+callback;
		Main.openPopup(url, title.replace(/[^\w]/g, ''), 640, 600);
	}
	
	this.expressionBuilder = function(ID, title, form, textarea, formulaType, rawDataType, channels, restrictions, bioAssaySetId, callback)
	{
		var url = getRoot()+'common/expression_builder.jsp?ID='+ID+'&title='+title;
		if (form) url += '&form='+form;
		if (textarea) url += '&textarea='+textarea;
		if (callback) url += '&callback='+callback;
		url += '&formulatype='+formulaType;
		url += '&rawdatatype='+rawDataType;
		url += '&channels='+channels;
		url += '&restrictions=' + (restrictions ? 1 : 0);
		if (bioAssaySetId) url += '&bioassayset_id='+bioAssaySetId;
		Main.openPopup(url, title.replace(/[^\w]/g, ''), 800, 460);
	}
	
	/*
		Opens a popup window allowing a user to select a color from a color chart. The 
		current color is taken from the specified input field of the specified form and 
		is saved back to it when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the input field is located
		@param input The name of the input field that holds the color value
		@param callback If specified, the color is returned by using this javascript call 
			instead of by setting the form field
	*/
	this.selectColor = function(title, form, input, callback)
	{
		var url = getRoot()+'common/select_color.jsp?title='+escape(title)+'&form='+form+'&input='+input;
		url += '&ID='+Main.getIdFromLocation();
		if (callback) url += '&callback='+callback;
		Main.openPopup(url, title.replace(/[^\w]/, ''), 400, 320);
	}

	
	/*
	  	Opens a popupwindow with helptext
	  	
	  	@param session The ID of the session.
	  	@param helpid The id of the helptext to be displayed.
	*/
	this.openHelp = function(ID, helpid)	
	{
		var url = getRoot()+'common/help/view_help.jsp?ID='+ID+'&external_id='+helpid;
		Main.openPopup(url, 'Help', 640, 550);
	}

	/*
	  	Opens a popupwindow with helptext getting the help id from
	  	a tabcontrol.
	  	
	  	@param session The ID of the session
	  	@param tabcontrol The ID of the tabcontrol
	  	@param helpid The id of the helptext to be displayed, only
	  		used if the current tab doesn't specify a help id
	*/
	this.openTabControlHelp = function(ID, tabcontrol, helpId)	
	{
		var useHelpId = TabControl.getActiveTabHelpId(tabcontrol);
		if (!useHelpId) useHelpId = helpId;
		var url = getRoot()+'common/help/view_help.jsp?ID='+ID+'&external_id='+useHelpId;
		Main.openPopup(url, 'Help', 640, 550);
	}

	
	/**
		Cut a string if it is longer than a specified length. If cut,
		three dots are appended to the string. The returned string is never
		longer than the specified max length.

		@param value The string to cut
		@param maxLength The maximum length
	*/
	this.cutString = function(value, maxLength)
	{
		if (value.length > maxLength) value = value.substr(0, maxLength-3)+'...';
		return value;
	}
	
	/**
		Trim leading and trailing whitespace from a string.
		@param value The string to trim
		@return The trimmed string
	*/
	this.trimString = function(value)
	{
		value = value.replace(/^\s+/g, '');
		value = value.replace(/\s+$/g, '');
		return value;
	}
	
	// Show or hide an element with the given ID attribute as a block element
	this.showHide = function(id, show)
	{
		var element = document.getElementById(id);
		if (element != undefined) 
		{
			if (show == undefined) show = element.style.display == 'none';
			if (show)
			{
				this.show(id);
			}
			else
			{
				this.hide(id);
			}
		}
	}
	
	// Shows an element with the given ID attribute as a block element
	this.show = function(id)
	{
		var element = document.getElementById(id);
		if (element != undefined) 
		{
			var display = 'block';
			if (element.nodeName && !Browser.isIE)
			{
				if (element.nodeName == 'TR')
				{
					display='table-row';
				}
				else if (element.nodeName == 'TBODY')
				{
					display='table-row-group';
				}
				else if (element.nodeName == 'TD')
				{
					display = 'table-cell';
				}
			}
			element.style.display = display;
		}
	}
	// Shows an element with the given ID attribute as an inline element
	this.showInline = function(id)
	{
		var element = document.getElementById(id);
		if (element != undefined) element.style.display='inline';
	}
	// Hides an element with the given ID attribute
	this.hide = function(id)
	{
		var element = document.getElementById(id);
		if (element != undefined) element.style.display='none';
	}
	
	this.setContent = function(id, content)
	{
		var element = document.getElementById(id);
		if (element != undefined) element.innerHTML = content;
	}

	this.addClass = function(obj, className)
	{
		if (obj.className.indexOf(className) != -1) return;
		obj.className = obj.className +' '+className;
	}
	
	this.removeClass = function(obj, className)
	{
		obj.className = obj.className.replace(className, '');
	}
	
	this.addOrRemoveClass = function(obj, className, addIfTrue)
	{
		if (addIfTrue)
		{
			this.addClass(obj, className);
		}
		else
		{
			this.removeClass(obj, className);
		}
	}
	
	this.toggleClass = function(obj, className)
	{
		if (obj.className.indexOf(className) != -1)
		{
			this.removeClass(obj, className);
		}
		else
		{
			obj.className = obj.className +' '+className;
		}
	}
	
	// Get the character code of an event that invloves a key, returns 0 if no code can be found
	// Use this method for onKeyPress events only. See also: getEventKeyCode()
	this.getEventCharCode = function(event)
	{
		var charCode = event.charCode;
		if (event.ctrlKey) 
		{
			// CTRL + some key usually doesn't result in a printable character
			charCode = 0;
		}
		if (charCode == undefined)
		{
			// IE8 
			charCode = event.keyCode;
		}
		if (charCode == 0 && event.keyCode < 32)
		{
			// In FF, if event.charCode == 0 a special key was pressed
			// It is safe to return it's keyCode if < 32
			// Also note that IE doesn't send events for all special keys.
			charCode = event.keyCode;
		}
		return charCode;
	}
	
	// Get the key code of an event that invloves a key, returns 0 if no code can be found
	// Use this method for onKeyDown and onKeyUp events only. See also: getEventCharCode();
	this.getEventKeyCode = function(event)
	{
		return event.keyCode;
	}
	
	// Get the target element of an event
	this.getEventTarget = function(event)
	{
		var target = event.srcElement;
		if (!target) target = event.target;
		return target;
	}
	
	// Try to get the window height
	this.getWindowHeight = function(theWin)
	{
		if (!theWin) theWin = window;
		var height;
		if (theWin.innerHeight)
		{
			height = theWin.innerHeight;
		}
		else if (theWin.document.documentElement && theWin.document.documentElement.clientHeight)
		{
			height = theWin.document.documentElement.clientHeight;
		}
		else if (theWin.document.body && theWin.document.body.clientHeight) 
		{
			height = theWin.document.body.clientHeight;
		}
		return height;
	}

	// Get the position and size of an element; 
	// returns an object with 'left', 'top', 'width' and 'height' properties
	// If 'rightAndBottom' is true, then 'right' and 'bottom' properties are also calculated
	this.getElementPosition = function(theElement, rightAndBottom)
	{
		var nextNode = theElement;
		var offsetTrail = theElement;
		var offsetTop = 0;
		var offsetLeft = 0;
		var offsetWidth = theElement.offsetWidth;
		var offsetHeight = theElement.offsetHeight;
		while (nextNode)
		{
			//if (offsetTrail.scrollTop) alert(offsetTrail.scrollTop);
			if (nextNode == offsetTrail)
			{
				if (offsetTrail.offsetTop) offsetTop += offsetTrail.offsetTop;
				if (offsetTrail.offsetLeft) offsetLeft += offsetTrail.offsetLeft;
				offsetTrail = offsetTrail.offsetParent;
			}
			if (nextNode != theElement && nextNode.scrollTop) offsetTop -= nextNode.scrollTop;
			nextNode = nextNode.parentNode;
		}
		//alert(offsetTop + ',' + offsetLeft);
		var pos = {left: offsetLeft, top:offsetTop, width:offsetWidth, height:offsetHeight};
		if (rightAndBottom)
		{
			pos.right = pos.left + pos.width;
			pos.bottom = pos.top + pos.height;
		}
		return pos;
	}
	
	// Try to get the window width
	this.getWindowWidth = function(theWin)
	{
		if (!theWin) theWin = window;
		var width;
		if (theWin.innerWidth)
		{
			width = theWin.innerWidth;
		}
		else if (theWin.document.documentElement && theWin.document.documentElement.clientWidth)
		{
			width = theWin.document.documentElement.clientWidth;
		}
		else if (theWin.document.body && theWin.document.body.clientWidth) 
		{
			width = theWin.document.body.clientWidth;
		}
		return width;
	}
	
	this.encodeURI = function(value)
	{
		var encoded = encodeURI(value);
		encoded = encoded.replace(/\+/g, '%2B');
		encoded = encoded.replace(/\&/g, '%26');
		encoded = encoded.replace(/\#/g, '%23');
		return encoded;
	}

	this.encodeTags = function(value)
	{
		var encoded = value
		encoded = encoded.replace(/\&/g, '&amp;');
		encoded = encoded.replace(/\</g, '&lt;');
		encoded = encoded.replace(/\>/g, '&gt;');
		return encoded;
	}
	
	this.itemOnClick = function(evt, ID, itemType, itemId, enableEdit, extraUrl)
	{
		if (enableEdit && evt && (evt.altKey || evt.ctrlKey || evt.shiftKey))
		{
			Main.viewOrEditItem(ID, itemType, itemId, true, extraUrl);
		}
		else
		{
			Main.viewOrEditItem(ID, itemType, itemId, false, extraUrl);
		}
	}
	
	this.viewFile = function(ID, fileId)
	{
		var controller = this.getController('FILE');
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd=ViewFile&item_id='+fileId;
		window.open(url, '_new');
		//Main.openPopup(url, 'ViewFile', 800, 600, 'x');
	}

	this.downloadFile = function(ID, fileId)
	{
		var controller = this.getController('FILE');
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd=DownloadFile&item_id='+fileId;
		Main.openPopup(url, 'DownloadFile', 400, 300);
	}
	
	this.initControllers = function()
	{
		this.controllers = new Array();
		this.controllers['EXPERIMENT'] = { url:'views/experiments/index.jsp', width:800, height:500 };
		this.controllers['BIOASSAY'] = { url:'views/experiments/bioassays/index.jsp', width:800, height:500 };
		this.controllers['BIOASSAYSET'] = { url:'views/experiments/bioassaysets/index.jsp', width:800, height:500 };
		this.controllers['TRANSFORMATION'] = { url:'views/experiments/transformations/index.jsp', width:500, height:300 };
		this.controllers['JOB'] = { url:'views/jobs/index.jsp', width:640, height:480, popup:true, edit:false };
		this.controllers['FORMULA'] = { url:'views/formulas/index.jsp', width:850, height:600 };
		this.controllers['PHYSICALBIOASSAY'] = { url:'views/physicalbioassays/index.jsp', width:800, height:500 };
		this.controllers['DERIVEDBIOASSAY'] = { url:'views/derivedbioassays/index.jsp', width:800, height:500 };
		this.controllers['RAWBIOASSAY'] = { url:'views/rawbioassays/index.jsp', width:800, height:500 };
		this.controllers['RAWDATA'] = { url:'views/rawbioassays/rawdata/index.jsp', width:700, height:500, popup: true, edit:false };
		this.controllers['REPORTERLIST'] = { url:'views/reporterlists/index.jsp', width:600, height:400 };
		this.controllers['REPORTER'] = { url:'views/reporters/index.jsp', width:540, height:460 };
		this.controllers['REPORTERSCORE'] = this.controllers['REPORTER'];
		this.controllers['SCAN'] = { url:'views/scans/index.jsp', width:800, height:500 };
		this.controllers['IMAGE'] = { url:'views/scans/images/index.jsp', width:500, height:340 };
		this.controllers['SESSION'] = { url:'views/sessions/index.jsp', width:400, height:300, popup:true, edit:false };
		this.controllers['TAG'] = { url:'biomaterials/tags/index.jsp', width:450, height:280 };
		this.controllers['EXTRACT'] = { url:'biomaterials/extracts/index.jsp', width:800, height:500 };
		this.controllers['SAMPLE'] = { url:'biomaterials/samples/index.jsp', width:800, height:500 };
		this.controllers['BIOMATERIALEVENT'] = { url:'biomaterials/events/index.jsp', width:600, height:380 };
		this.controllers['BIOSOURCE'] = { url:'biomaterials/biosources/index.jsp', width:800, height:500 };
		this.controllers['BIOMATERIALLIST'] = { url:'biomaterials/lists/index.jsp', width:800, height:500 };
		this.controllers['BIOPLATE'] = { url:'biomaterials/bioplates/index.jsp', width:800, height:500 };
		this.controllers['BIOPLATETYPE'] = { url:'biomaterials/bioplatetypes/index.jsp', width:600, height:400 };
		this.controllers['BIOPLATEEVENT'] = { url:'biomaterials/bioplates/events/index.jsp', width:600, height:460 };
		this.controllers['BIOPLATEEVENTTYPE'] = { url:'biomaterials/bioplateeventtypes/index.jsp', width:600, height:400 };
		this.controllers['BIOPLATEEVENTDEFINITION'] = { url:'biomaterials/eventdef/index.jsp', width:600, height:500 };
		this.controllers['BIOWELL'] = { url:'biomaterials/bioplates/wells/index.jsp', width:480, height:320 };
		this.controllers['MESSAGE'] = { url:'my_base/messages/index.jsp', width:600, height:420, popup:true, edit:false };
		this.controllers['PROJECT'] = { url:'my_base/projects/index.jsp', width:640, height:480 };
		this.controllers['PERMISSIONTEMPLATE'] = { url:'views/permissiontemplates/index.jsp', width:450, height:280 };
		this.controllers['FILE'] = { url:'filemanager/index.jsp', width:560, height:420 };
		this.controllers['FILESERVER'] = { url:'filemanager/fileservers/index.jsp', width:600, height:460 };
		this.controllers['DIRECTORY'] = { url:'filemanager/directories/index.jsp', width:600, height:400, xpopup:true };
		this.controllers['ARRAYBATCH'] = { url:'lims/arraybatches/index.jsp', width:800, height:500 };
		this.controllers['ARRAYDESIGN'] = { url:'lims/arraydesigns/index.jsp', width:800, height:500 };
		this.controllers['FEATURE'] = { url:'lims/arraydesigns/features/index.jsp', width:700, height:540, popup:true, edit:false };
		this.controllers['ARRAYSLIDE'] = { url:'lims/arrayslides/index.jsp', width:800, height:500 };
		this.controllers['PLATEGEOMETRY'] = { url:'lims/geometries/index.jsp', width:500, height:340 };
		this.controllers['PLATEMAPPING'] = { url:'lims/platemappings/index.jsp', width:600, height:400 };
		this.controllers['PLATE'] = { url:'lims/plates/index.jsp', width:800, height:500 };
		this.controllers['WELL'] = { url:'lims/plates/wells/index.jsp', width:800, height:500 };
		this.controllers['PLATEEVENT'] = { url:'lims/plates/events/index.jsp', width:600, height:400 };
		this.controllers['PLATETYPE'] = { url:'lims/platetypes/index.jsp', width:500, height:340 };
		this.controllers['PLATEEVENTTYPE'] = { url:'lims/platetypes/eventtypes/index.jsp', width:500, height:340 };
		this.controllers['ANNOTATIONTYPE'] = { url:'admin/annotationtypes/index.jsp', width:540, height:420 };
		this.controllers['ANNOTATIONTYPECATEGORY'] = { url:'admin/annotationtypecategories/index.jsp', width:540, height:400 };
		this.controllers['CLIENT'] = { url:'admin/clients/index.jsp', width:500, height:340 };
		this.controllers['HELP'] = { url:'admin/clients/help/index.jsp', width:600, height:500 };
		this.controllers['FILETYPE'] = { url:'admin/filetypes/index.jsp', width:440, height:280 };
		this.controllers['GROUP'] = { url:'admin/groups/index.jsp', width:540, height:400 };
		this.controllers['HARDWARE'] = { url:'admin/hardware/index.jsp', width:500, height:340 };
		this.controllers['HARDWARETYPE'] = { url:'admin/hardwaretypes/index.jsp', width:500, height:340 };
		this.controllers['MIMETYPE'] = { url:'admin/mimetypes/index.jsp', width:540, height:400 };
		this.controllers['NEWS'] = { url:'admin/news/index.jsp', width:540, height:400 };
		this.controllers['PLUGINCONFIGURATION'] = { url:'admin/pluginconfigurations/index.jsp', width:800, height:500 };
		this.controllers['PLUGINDEFINITION'] = { url:'admin/plugindefinitions/index.jsp', width:800, height:500 };
		this.controllers['PLUGINTYPE'] = { url:'admin/plugintypes/index.jsp', width:700, height:400 };
		this.controllers['JOBAGENT'] = { url:'admin/jobagents/index.jsp', width:700, height:400 };
		this.controllers['PROTOCOL'] = { url:'admin/protocols/index.jsp', width:800, height:500 };
		this.controllers['PROTOCOLTYPE'] = { url:'admin/protocoltypes/index.jsp', width:500, height:340 };
		this.controllers['QUOTA'] = { url:'admin/quota/index.jsp', width:600, height:440 };
		this.controllers['QUOTATYPE'] = { url:'admin/quotatypes/index.jsp', width:500, height:340 };
		this.controllers['REPORTERTYPE'] = { url:'admin/reportertypes/index.jsp', width:500, height:340 };
		this.controllers['ROLE'] = { url:'admin/roles/index.jsp', width:600, height:440 };
		this.controllers['SOFTWARE'] = { url:'admin/software/index.jsp', width:500, height:340 };
		this.controllers['SOFTWARETYPE'] = { url:'admin/softwaretypes/index.jsp', width:500, height:340 };
		this.controllers['USER'] = { url:'admin/users/index.jsp', width:600, height:440 };
		this.controllers['ANYTOANY'] = { url:'common/anytoany/index.jsp', width:500, height:340, popup:true };
		this.controllers['EXTRAVALUETYPE'] = { url:'admin/extravaluetypes/index.jsp', width:500, height:400 };
		this.controllers['EXTRAVALUE'] = { url:'views/experiments/extravalues/index.jsp', width:500, height:340, edit:false };
		this.controllers['PLATFORM'] = { url:'admin/platforms/index.jsp', width:540, height:400 };
		this.controllers['PLATFORMVARIANT'] = { url:'admin/platforms/variants/index.jsp', width:540, height:400 };
		this.controllers['DATAFILETYPE'] = { url:'admin/datafiletypes/index.jsp', width:600, height:480 };
		this.controllers['QUANTITY'] = { url:'admin/quantities/index.jsp', width:600, height:480 };
		this.controllers['UNIT'] = { url:'admin/quantities/units/index.jsp', width:600, height:480 };
		this.controllers['CHANGEHISTORY'] = { url:'common/history/index.jsp', width:600, height:480, edit:false, popup:true };
		this.controllers['ANNOTATION'] = { url:'common/annotations/index.jsp', width:700, height:480, edit:true, popup:true };
		this.controllers['ITEMSUBTYPE'] = { url:'admin/itemsubtypes/index.jsp', width:640, height:440 };
	}
	
	this.getController = function(itemType)
	{
		if (!this.controllers) this.initControllers();
		var controller = this.controllers[itemType];
		
		if (!controller)
		{
			var msg = 'Unhandled item: ' + itemType + '\n';
			msg += 'Please report this as a bug to the development team';
			alert(msg);
			return null;
		}
		return controller;
	}
	
	this.listItems = function(ID, itemType, extraUrl)
	{
		var controller = this.getController(itemType);
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd=List';
		if (extraUrl) url += extraUrl;
		location.replace(url);
	}
	
	this.viewOrEditItem = function(ID, itemType, itemId, edit, extraUrl, popupOptions)
	{
		var controller = this.getController(itemType);

		if (controller.edit != undefined) edit = controller.edit;
		var popup = controller.popup == undefined ? edit : controller.popup;
		var width = controller.width ? controller.width : 600;
		var height = controller.height ? controller.height : 400;
		
		var cmd = itemId == 0 ? 'NewItem' : (edit ? 'EditItem' : 'ViewItem');
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd='+cmd;
		if (itemId) url += '&item_id='+itemId;
		if (extraUrl) url += extraUrl;
		if (popup)
		{
			Main.openPopup(url, itemType, width, height, popupOptions);
		}
		else
		{
			var theWindow = window;
			var i = 0;
			while (theWindow.name != 'main' && i < 10)
			{
				theWindow = theWindow.parent;
				i++;
			}
			Main.safeSetLocation(url, theWindow);
		}	
	}
	
	this.copyItem = function(ID, itemType, itemId, extraUrl, popupOptions)
	{
		var controller = this.getController(itemType);	
		
		var popup = true;
		var width = controller.width ? controller.width : 600;
		var height = controller.height ? controller.height : 400;
		
		var cmd = itemId != 0 ? 'CopyItem' : 'NewItem';
		var url = getRoot();
		
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd='+cmd+'&item_id='+itemId;
		if (extraUrl) url += extraUrl;
		if (popup)
		{
			Main.openPopup(url, cmd+itemType, width, height, popupOptions);
		}
		else
		{
			var theWindow = window;
			var i = 0;
			while (theWindow.name != 'main' && i < 10)
			{
				theWindow = theWindow.parent;
				i++;
			}
			Main.safeSetLocation(url, theWindow);
		}
	}
	
	this.selectItem = function(ID, itemType, mode, callback, extraUrl, popupOptions)
	{
		var controller = this.getController(itemType);
		if (!controller) return;
		
		var width = 1000;
		var height = 700;
		var cmd = 'UpdateContext';
		var url = getRoot();
		var url = getRoot() + controller.url + '?ID='+ID+'&cmd='+cmd;
		if (mode) url += '&mode='+mode;
		if (callback) url += '&callback='+callback;
		if (extraUrl) url += extraUrl;
		Main.openPopup(url, 'Select'+itemType, width, height, popupOptions);
	}
	
	this.deleteItemPermanently = function(ID, ask, itemType, itemId, extraUrl, popupOptions)
	{
		if (ask)
		{
			if (!confirm('Delete this item permanently?')) return;
		}
		var url = getRoot();
		url += 'views/trashcan/index.jsp?ID='+ID+'&cmd=DeleteItem&popup=1';
		url += '&item_type='+itemType+'&item_id=' + itemId;
		if (extraUrl) url += extraUrl;
		var width = 300;
		var height = 200;
		Main.openPopup(url, 'Delete'+itemType, width, height, popupOptions);
	}

	this.showUsingItems = function(ID, itemType, itemId, extraUrl)
	{
		var url = getRoot();
		url += 'views/trashcan/index.jsp?ID='+ID+'&cmd=ViewUsingItems';
		url += '&item_type='+itemType+'&item_id='+itemId;
		if (extraUrl) url += extraUrl;
		Main.safeSetLocation(url);
	}
	
	this.linkItem = function(ID, itemType, itemId, editable, name)
	{
		var html = '<span class="link"';
		html += ' onclick="Main.itemOnClick(event,\'' + ID + '\'';
		html += ',\'' + itemType + '\',' + itemId + ',' + editable + ')"';
		html += ' title="View this item';
		if (editable) html += " (use CTRL, ALT or SHIFT to edit)";
		html += '">' + name + '</span>'
		return html;
	}
	
	this.getIdFromLocation = function(url)
	{
		var findId = /ID=([0-9a-f]*)/;
		if (!url) url = location.search;
		var matches = url.match(findId);
		return matches && matches.length > 1 ? matches[1] : null;
	}
	
	/**
		Create a new ajax request.
		@return A request object or null if the browser doesn't support ajax
	*/
	this.getAjaxRequest = function()
	{
		var xmlHttp = null;
		if (window.XMLHttpRequest)
		{
			// Non-IE browswers
			xmlHttp = new XMLHttpRequest();
		}
		else if (window.ActiveXObject)
		{
			// IE browsers
			try 
			{
				// IE #1
		  		xmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
			}
			catch (e) 
			{
				try 
				{
					// IE #2
					xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
				} 
				catch (e2) 
				{}
			}
		}
		return xmlHttp;
	}
	
	/*
		Safely set the location of a document using as if
		win.document.location.href = url or win.document.replace(url)
		had been used. If the url is longer
		than specified by getMaxUrlLength() function the url will
		be re-written to a <form> using hidden fields to submit
		query values.
		@param url The url to set
		@param win The window to set the location on, if not given,
			the current window is assumed
		@param replace If given, use location.replace() instead of location.href
			(only if url is not too long)
	*/
	this.safeSetLocation = function(url, win, replace)
	{
		if (!url) return;
		if (!win) win = window;
		var postform;
		if (this.isTooLongURL(url))
		{
			postform = this.convertToPostForm(url, win);
		}
		if (postform)
		{
			document.body.appendChild(postform);
			postform.submit();
		}
		else if (replace)
		{
			win.document.location.replace(url);
		}
		else
		{
			win.document.location.href = url;
		}
	}
	
	/*
		Check if the URL is longer than the configured max URL length.
	*/
	this.isTooLongURL = function(url)
	{
		return window.getMaxUrlLength && url.length > window.getMaxUrlLength();
	}
	
	/*
		Convert an URL to a HTML form using POST and write it to
		a document. This method	extract all query parameters from the 
		URL (except ID) and	generate a <form> tag with hidden input fields. 
		The method returns an unattached Form object. To post it, 
		do: document.body.appendChild(form); form.submit();
		@param url The url to convert
		@param win The window the form should post to, if no window is given
			or if it doesn't have a name the form is posted to the current
			window.
	*/
	this.convertToPostForm = function(url, win)
	{
		var i = url.indexOf('?');
		if (i < 0) return null;
		
		var query = url.substring(i+1);
		if (!query) return null;
		
		url = url.substring(0, i);
		var ID = Main.getIdFromLocation(query);
		if (ID) url += '?ID=' + ID;

		var frm = document.createElement('form');
		frm.setAttribute('action', url);
		frm.setAttribute('method', 'post');
		if (win && win.name) frm.setAttribute('target', win.name);
		
		var options = query.split('&');
		for (var i = 0; i < options.length; i++)
		{
			var pair = options[i].split('=');
			var name = pair[0];
			var value = decodeURIComponent(pair[1]);
			if (name && name != 'ID')
			{
				Forms.createHidden(frm, name, value, document);
			}
		}
		return frm;
	}
	
	/**
		Save a key-value pair for a page.
	*/
	this.setPageValue = function(page, key, value)
	{
		if (!window.top.pageValues) window.top.pageValues = new Array();
		if (!window.top.pageValues[page]) window.top.pageValues[page] = new Array();
		window.top.pageValues[page][key] = value;
	}

	/**
		Get the saved value for the given key on a page
	*/
	this.getPageValue = function(page, key)
	{
		var value = null;
		try
		{
			if (window.top.pageValues && window.top.pageValues[page])
			{
				value = window.top.pageValues[page][key];
			}
		}
		catch (e)
		{
			value = null;
		}
		return value;
	}

	/**
		Get all elements with a given <tag> and class="..."
	*/
	this.getElementsByTagAndClassName = function(tagName, className)
	{
		var tags = new Array();
		var temp = document.getElementsByTagName(tagName);
		if (className)
		{
			for (var i = 0; i < temp.length; i++)
			{
				var tag = temp[i];
				if (tag.className.indexOf(className) >= 0)
				{
					tags[tags.length] = tag;
				}
			}
		}
		return tags;
	}
	
	this.viewSource = function(tag)
	{
		if (!tag) tag = document.body.parentNode;
		Main.openPopup('', 'ViewSource', 800, 800);
		var html = tag.innerHTML;
		html = html.replace(/\</g, '&lt;');
		html = html.replace(/\>/g, '&gt;');
		var newWin = this.getPageValue('open.windows', 'ViewSource');
		newWin.document.write('<textarea cols=100 rows=50 style="border: 1px solid #000000;" wrap=off>'+html+'</textarea>');
	}
	
	this.debug = function(message, clearOutput)
	{
		if (!this.debugDiv)
		{
			this.debugDiv = document.createElement('div');
			this.debugDiv.className = 'debug';
			this.debugDiv.title = 'Double click to hide';
			this.debugDiv.ondblclick = Main.hideDebug;
			document.body.appendChild(this.debugDiv);
		}
		this.debugDiv.style.display = 'block';
		if (clearOutput) this.debugDiv.innerHTML = '';
		
		var newDiv = document.createElement('div');
		newDiv.innerHTML = '<table><tr valign="top"><td>&gt;</td><td>' + message + '</td></tr></table>';
		this.debugDiv.appendChild(newDiv);
		var scrollTop = newDiv.offsetTop+newDiv.offsetHeight-this.debugDiv.offsetHeight;
		if (scrollTop > 0)this.debugDiv.scrollTop=scrollTop;
	}
	
	this.hideDebug = function()
	{
		if (!Main.debugDiv) return;
		Main.debugDiv.style.display = 'none';
	}
	
}

var Dates = new DateClass();
// Functions related to handling date values
// Implementation based on code provided by:
// ===================================================================
// Author: Matt Kruse <matt@mattkruse.com>
// WWW: http://www.mattkruse.com/
//
// NOTICE: You may use this code for any purpose, commercial or
// private, without any further permission from the author. You may
// remove this notice from your final code if you wish, however it is
// appreciated by the author if at least my web site address is kept.
//
// You may *NOT* re-distribute this code in any way except through its
// use. That means, you can include it in your product, or your web
// site, or any other form where the code is actually being used. You
// may not put the plain javascript up on your site for download or
// include it in your javascript libraries for download. 
// If you wish to share this code with others, please just point them
// to the URL instead.
// Please DO NOT link directly to my .js files from your site. Copy
// the files to your server and use them there. Thank you.
// ===================================================================
// HISTORY
// ------------------------------------------------------------------
// May 17, 2003: Fixed bug in parseDate() for dates <1970
// March 11, 2003: Added parseDate() function
// March 11, 2003: Added "NNN" formatting option. Doesn't match up
//                 perfectly with SimpleDateFormat formats, but 
//                 backwards-compatability was required.

// ------------------------------------------------------------------
// These functions use the same 'format' strings as the 
// java.text.SimpleDateFormat class, with minor exceptions.
// The format string consists of the following abbreviations:
// 
// Field        | Full Form          | Short Form
// -------------+--------------------+-----------------------
// Year         | yyyy (4 digits)    | yy (2 digits), y (2 or 4 digits)
// Month        | MMMM(name or abbr.)| MM (2 digits), M (1 or 2 digits)
//              | NNN (abbr.)        |
// Day of Month | dd (2 digits)      | d (1 or 2 digits)
// Day of Week  | EE (name)          | E (abbr)
// Hour (1-12)  | hh (2 digits)      | h (1 or 2 digits)
// Hour (0-23)  | HH (2 digits)      | H (1 or 2 digits)
// Hour (0-11)  | KK (2 digits)      | K (1 or 2 digits)
// Hour (1-24)  | kk (2 digits)      | k (1 or 2 digits)
// Minute       | mm (2 digits)      | m (1 or 2 digits)
// Second       | ss (2 digits)      | s (1 or 2 digits)
// AM/PM        | a                  |
//
// NOTE THE DIFFERENCE BETWEEN MM and mm! Month=MM, not mm!
// Examples:
//  "MMM d, y" matches: January 01, 2000
//                      Dec 1, 1900
//                      Nov 20, 00
//  "M/d/yy"   matches: 01/20/00
//                      9/2/00
//  "MMM dd, yyyy hh:mm:ssa" matches: "January 01, 2000 12:30:45AM"
// ------------------------------------------------------------------
function DateClass()
{
	this.MONTH_NAMES = new Array('January','February','March','April','May','June','July','August','September','October','November','December','Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
	this.DAY_NAMES = new Array('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday','Sun','Mon','Tue','Wed','Thu','Fri','Sat');

	// convert a date to a string:
	// default format is yyyy-MM-dd
	this.formatDate = function(date, format)
	{
		if (!format) format = "yyyy-MM-dd";
		var result="";
		var i_format=0;
		var c="";
		var token="";
		var y=date.getYear()+"";
		var M=date.getMonth()+1;
		var d=date.getDate();
		var E=date.getDay();
		var H=date.getHours();
		var m=date.getMinutes();
		var s=date.getSeconds();
		var yyyy,yy,MMM,MM,dd,hh,h,mm,ss,ampm,HH,H,KK,K,kk,k;
		
		// Convert real date parts into formatted versions
		var value=new Object();
		if (y.length < 4) 
		{
			y=""+(y-0+1900);
		}
		value["y"]=""+y;
		value["yyyy"]=y;
		value["yy"]=y.substring(2,4);
		value["M"]=M;
		value["MM"]=LZ(M);
		value["MMM"]=this.MONTH_NAMES[M+11];
		value["MMMM"]=this.MONTH_NAMES[M-1];
		value["NNN"]=this.MONTH_NAMES[M+11];
		value["d"]=d;
		value["dd"]=LZ(d);
		value["E"]=this.DAY_NAMES[E+7];
		value["EE"]=this.DAY_NAMES[E];
		value["H"]=H;
		value["HH"]=LZ(H);
		if (H==0) 
		{
			value["h"]=12;
		}
		else if (H>12)
		{
			value["h"]=H-12;
		}
		else 
		{
			value["h"]=H;
		}
		value["hh"]=LZ(value["h"]);
		if (H>11)
		{
			value["K"]=H-12;
		}
		else 
		{
			value["K"]=H;
		}
		value["k"]=H+1;
		value["KK"]=LZ(value["K"]);
		value["kk"]=LZ(value["k"]);
		if (H > 11) 
		{
			value["a"]="PM"; 
		}
		else 
		{
			value["a"]="AM"; 
		}
		value["m"]=m;
		value["mm"]=LZ(m);
		value["s"]=s;
		value["ss"]=LZ(s);
		while (i_format < format.length) 
		{
			c=format.charAt(i_format);
			token="";
			while ((format.charAt(i_format)==c) && (i_format < format.length)) 
			{
				token += format.charAt(i_format++);
			}
			if (value[token] != null) 
			{
				result=result + value[token]; 
			}
			else 
			{
				result=result + token; 
			}
		}
		return result;
	}

	/*
		Convert a year value to a string with four digits, ie. pad with zeroes
		as needed
	*/
	this.getFourDigitYear = function(year)
	{
		if (year < 10) 
		{
			year = '000'+year;
		}
		else if (year < 100) 
		{
			year = '00'+year;
		}
		else if (year < 1000) 
		{
			year = '0'+year;
		}
		return year;
	}
	
	/*
		Create a new date object with the specified year, month and day
	*/
	this.newDate = function(year, month, day)
	{
		var date = new Date();
		date.setFullYear(year, month, day);
		return date;
	}
	
	/*
		Attach this function to the onkeypress attribute of a field to
		prevent other characters than numbers and hyphens. This function will not validate
		the entered value.
		Example: <input type="text" name="date" onkeypress="return Dates.dateOnly(event)">
		@param event The event object
		@param submitOnEnter If true, the form is submitted when the enter key is pressed
		@param validate A function reference that validates the form,
	*/
	this.dateOnly = function(event, submitOnEnter, validate)
	{
		var charCode = Main.getEventCharCode(event);
		if (submitOnEnter && (charCode == 3 || charCode == 13))
		{
			var target = Main.getEventTarget(event);
			var frm = target.form;
			var value = target.value;
			if (!validate || (validate && validate(frm, value)))
			{
				frm.submit();
			}
		}
		return charCode <= 31 || (charCode >= 48 && charCode <= 57) || charCode == 45;
	}
	
	// ------------------------------------------------------------------
	// isDate ( date_string, format_string )
	// Returns true if date string matches format of format string and
	// is a valid date. Else returns false.
	// It is recommended that you trim whitespace around the value before
	// passing it to this function, as whitespace is NOT ignored!
	// ------------------------------------------------------------------
	this.isDate = function(val, format) 
	{
		return this.parseString(val, format) != null;
	}
	
	// ------------------------------------------------------------------
	// parseString( date_string , format_string )
	//
	// This function takes a date string and a format string. It matches
	// If the date string matches the format string, it returns the 
	// the date object. If it does not match, it returns null.
	// ------------------------------------------------------------------
	this.parseString = function(val, format)
	{
		if (!format) format = 'yyyy-MM-dd';
		var i_val=0;
		var i_format=0;
		var c="";
		var token="";
		var token2="";
		var x,y;
		var now=new Date();
		var year=now.getYear();
		var month=now.getMonth()+1;
		var date=1;
		var hh=now.getHours();
		var mm=now.getMinutes();
		var ss=now.getSeconds();
		var ampm="";
		
		while (i_format < format.length) 
		{
			// Get next token from format string
			c=format.charAt(i_format);
			token="";
			while ((format.charAt(i_format)==c) && (i_format < format.length)) 
			{
				token += format.charAt(i_format++);
			}
			// Extract contents of value based on format token
			if (token=="yyyy" || token=="yy" || token=="y") 
			{
				if (token=="yyyy") { x=4;y=4; }
				if (token=="yy")   { x=2;y=2; }
				if (token=="y")    { x=2;y=4; }
				year=_getInt(val,i_val,x,y);
				if (year==null) { return null; }
				i_val += year.length;
				if (year.length==2) 
				{
					if (year > 70) 
					{ 
						year=1900+(year-0); 
					}
					else 
					{ 
						year=2000+(year-0); 
					}
				}
			}
			else if (token=='MMMM' || token=="MMM" || token=="NNN")
			{
				month=0;
				for (var i=0; i<this.MONTH_NAMES.length; i++) 
				{
					var month_name=this.MONTH_NAMES[i];
					if (val.substring(i_val,i_val+month_name.length).toLowerCase()==month_name.toLowerCase()) 
					{
						month=i+1;
						if (month>12) { month -= 12; }
						i_val += month_name.length;
						break;
					}
				}
				if ((month < 1)||(month>12)) { return null; }
			}
			else if (token=="EE"||token=="E")
			{
				for (var i=0; i<this.DAY_NAMES.length; i++) 
				{
					var day_name=this.DAY_NAMES[i];
					if (val.substring(i_val,i_val+day_name.length).toLowerCase()==day_name.toLowerCase()) 
					{
						i_val += day_name.length;
						break;
					}
				}
			}
			else if (token=="MM"||token=="M") 
			{
				month=_getInt(val,i_val,token.length,2);
				if(month==null||(month<1)||(month>12)){return null;}
				i_val+=month.length;
			}
			else if (token=="dd"||token=="d") 
			{
				date=_getInt(val,i_val,token.length,2);
				if(date==null||(date<1)||(date>31)){return null;}
				i_val+=date.length;
			}
			else if (token=="hh"||token=="h") 
			{
				hh=_getInt(val,i_val,token.length,2);
				if(hh==null||(hh<1)||(hh>12)){return null;}
				i_val+=hh.length;
			}
			else if (token=="HH"||token=="H") 
			{
				hh=_getInt(val,i_val,token.length,2);
				if(hh==null||(hh<0)||(hh>23)){return null;}
				i_val+=hh.length;
			}
			else if (token=="KK"||token=="K") 
			{
				hh=_getInt(val,i_val,token.length,2);
				if(hh==null||(hh<0)||(hh>11)){return null;}
				i_val+=hh.length;
			}
			else if (token=="kk"||token=="k") 
			{
				hh=_getInt(val,i_val,token.length,2);
				if(hh==null||(hh<1)||(hh>24)){return null;}
				i_val+=hh.length;hh--;
			}
			else if (token=="mm"||token=="m") 
			{
				mm=_getInt(val,i_val,token.length,2);
				if(mm==null||(mm<0)||(mm>59)){return null;}
				i_val+=mm.length;
			}
			else if (token=="ss"||token=="s") 
			{
				ss=_getInt(val,i_val,token.length,2);
				if(ss==null||(ss<0)||(ss>59)){return null;}
				i_val+=ss.length;
			}
			else if (token=="a") 
			{
				if (val.substring(i_val,i_val+2).toLowerCase()=="am") {ampm="AM";}
				else if (val.substring(i_val,i_val+2).toLowerCase()=="pm") {ampm="PM";}
				else {return null;}
				i_val+=2;
			}
			else 
			{
				if (val.substring(i_val,i_val+token.length)!=token) {return null;}
				else {i_val+=token.length;}
			}
		}
		
		// If there are any trailing characters left in the value, it doesn't match
		if (i_val != val.length) { return null; }
		
		// Is date valid for month?
		if (month==2) 
		{
			// Check for leap year
			if ( ( (year%4==0)&&(year%100 != 0) ) || (year%400==0) ) 
			{ // leap year
				if (date > 29){ return null; }
			}
			else 
			{ 
				if (date > 28) { return null; } 
			}
		}
		if ((month==4)||(month==6)||(month==9)||(month==11)) 
		{
			if (date > 30) { return null; }
		}
		
		// Correct hours value
		if (hh<12 && ampm=="PM") 
		{ 
			hh=hh-0+12; 
		}
		else if (hh>11 && ampm=="AM") 
		{ 
			hh-=12; 
		}
		return new Date(year,month-1,date,hh,mm,ss);
	}

	function _isInteger(val) {
		var digits="1234567890";
		for (var i=0; i < val.length; i++) {
			if (digits.indexOf(val.charAt(i))==-1) { return false; }
			}
		return true;
		}
	function _getInt(str,i,minlength,maxlength) {
		for (var x=maxlength; x>=minlength; x--) {
			var token=str.substring(i,i+x);
			if (token.length < minlength) { return null; }
			if (_isInteger(token)) { return token; }
			}
		return null;
		}
	function LZ(x) {return(x<0||x>9?"":"0")+x}

	/*
		Calculate the number of days in the specified month
		Months are numbered from 1 to 12. Returns 0 if the 
		specified month is invalid.
	*/
	this.daysInMonth = function(year, month)
	{
		switch (month)
		{
			// Months with 31 days
			case 1:
			case 3:
			case 5:
			case 7:
			case 8:
			case 10:
			case 12:
				return 31;
				break;
			// Months with 30 days
			case 4:
			case 6:
			case 9:
			case 11:
				return 30;
				break;
			
			// February, taking leap years into account
			case 2:
				if (((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0))
				{
					return 29;
				}
				else
				{
					return 28;
				}
				break;

			default:
				return 0;
		}
		return 0;
	}
	
	/*
		Opens a popup window allowing a user to select a date graphically. The 
		current date is taken from the specified input field of the specified form and 
		is saved back to it when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the input field is located
		@param input The name of the input field that holds the date value
		@param callback If specified, the date is returned by using this javascript call instead
			of by setting the form field
	*/
	this.selectDate = function(title, form, input, callback, format)
	{
		var url = getRoot()+'common/calendar.jsp?title='+escape(title)+'&form='+form+'&input='+input+'&format='+escape(format);
		url += '&ID='+getSessionId();
		if (callback) url += '&callback='+callback;
		Main.openPopup(url, title.replace(/[^\w]/, ''), 440, 340);
	}

	/*
		Opens a popup window allowing a user to select a date and time graphically. The 
		current value is taken from the specified input field of the specified form and 
		is saved back to it when the window is closed.
	
		@param title The title of the window
		@param form The name of the form where the input field is located
		@param input The name of the input field that holds the date value
		@param callback If specified, the date is returned by using this javascript call instead
			of by setting the form field
	*/
	this.selectDateTime = function(title, form, input, callback, format)
	{
		var url = getRoot()+'common/calendar.jsp?datetime=1&title='+escape(title)+'&form='+form+'&input='+input+'&format='+escape(format);
		url += '&ID='+getSessionId();
		if (callback) url += '&callback='+callback;
		Main.openPopup(url, title.replace(/[^\w]/, ''), 440, 340);
	}


}

var Numbers = new NumberClass();
// Functions related to handling numeric values
function NumberClass()
{
	/*
		Attach this function to the onkeypress attribute of a field to
		prevent other characters than numbers.
		Example: <input type="text" name="date" onkeypress="return Numbers.integerOnly(event)">
		@param event The event object
		@param submitOnEnter If true, the form is submitted when the enter key is pressed
		@param validate A function reference that validates the form,
	*/
	this.integerOnly = function(event, submitOnEnter, validate)
	{
		var charCode = Main.getEventCharCode(event);
		if (submitOnEnter && (charCode == 3 || charCode == 13))
		{
			var target = Main.getEventTarget(event);
			var frm = target.form;
			var value = target.value;
			if (!validate || (validate && validate(frm, value)))
			{
				frm.submit();
			}
		}
		return charCode <= 31 || (charCode >= 48 && charCode <= 57) || charCode == 45;
	}
	
	/*
		Checks if the supplied value is an integer.
	*/
	this.isInteger = function(value)
	{
		return (value.search(/^\-?\d+$/) != -1);
	}

	/*
		Attach this function to the onkeypress attribute of a field to
		prevent other characters than numbers, a period, a minus sign and 'E'.
		Example: <input type="text" name="date" onkeypress="return Numbers.numbersOnly(event)">
		@param event The event object
		@param submitOnEnter If true, the form is submitted when the enter key is pressed
		@param validate A function reference that validates the form,
	*/
	this.numberOnly = function(event, submitOnEnter, validate)
	{
		var charCode = Main.getEventCharCode(event);
		if (submitOnEnter && (charCode == 3 || charCode == 13))
		{
			var target = Main.getEventTarget(event);
			var frm = target.form;
			var value = target.value;
			if (!validate || (validate && validate(frm, value)))
			{
				frm.submit();
			}
		}
		return charCode <= 31 || (charCode >= 48 && charCode <= 57) || charCode == 45 || charCode == 46 || charCode == 69 || charCode == 101;
	}
	
	/*
		Format a number with the specified number of decimals
	*/
	this.formatNumber = function(value, numDecimals, unit)
	{
		if (!value) return '';
		var result = '';
		if (value < 0)
		{
			result += "-";
			value = -value;
		}
		if (numDecimals < 0)
		{
			result += value;
		}
		else if (numDecimals == 0)
		{
			result += Math.round(value);
		}
		else
		{
			var intPart = Math.floor(value);
			var decimals = Math.round((value - intPart) * Math.pow(10, numDecimals));
			result += intPart + '.' + decimals;
		}
		if (unit) result += unit;
		return result;
	}
	
	this.GB = 1073741824;
	this.MB = 1048576;
	this.kB = 1024;
	
	this.formatBytes = function(value)
	{
		if (!value) return '';
		if (value > this.GB)
		{
			return this.formatNumber(value / this.GB, 1, ' GB');
		}
		else if (value > this.MB)
		{
			return this.formatNumber(value / this.MB, 1, ' MB');
		}
		else if (value > this.kB)
		{
			return this.formatNumber(value / this.kB, 1, ' kB');
		}
		else
		{
			return value + ' bytes';
		}
	}
	
	this.formatSeconds = function(seconds)
	{
		if (!seconds) return '';
		var result = '';
		var mustAppend = false;
		if (seconds >= 86400)
		{
			var days = Math.floor(seconds / 86400);
			seconds = seconds - 86400*days;
			result += days;
			result += days == 1 ? " day " : " days ";
			mustAppend = true;
		}
		if (seconds >= 3600 || mustAppend)
		{
			var hours = Math.floor(seconds / 3600);
			seconds = seconds - 3600*hours;
			result += hours;
			result += hours == 1 ? " hour " : " hours ";
			mustAppend = true;
		}
		if (seconds >= 60 || mustAppend)
		{
			var minutes = Math.floor(seconds / 60);
			seconds = seconds - 60*minutes;
			result += minutes;
			result += minutes == 1 ? " minute " : " minutes ";
			mustAppend = true;
		}
		result += seconds;
		result += seconds == 1 ? " second" : " seconds";
		return result;
	}
	
}

var Strings = new StringClass();
// Functions related to string handling
function StringClass()
{
	/*
		Trims a string to a maximum length. If the original
		string is too long it is cut off and three dots (...) are
		appended.
	*/
	this.trimString = function(value, maxLength)
	{
		if (value && value.length > maxLength)
		{
			value = value.substr(0, maxLength)+'...';
		}
		return value;
	}

	/**
		Compare two strings ignoreing case.
		returns -1 if a<b, +1 if a>b, 0 if a==b
	*/
	this.compareIgnoreCase = function(a, b)
	{
		var lcA = a.toLowerCase();
		var lcB = b.toLowerCase();
		if (lcA < lcB) return -1;
		if (lcA > lcB) return +1;
		return 0;
	}
}

var Forms = new FormClass();
// Functions related to form handling
function FormClass()
{

	/*
		Attach this function to the onkeypress attribute of a field to
		automatically submit a form when the enter key is pressed.
		Example: <input type="text" name="title" onkeypress="return Forms.submitOnEnter(event)">
		@param event The event object
		@param validate A function reference that validates the form
	*/
	this.submitOnEnter = function(event, validate)
	{
		var charCode = Main.getEventCharCode(event);
		if (charCode == 3 || charCode == 13)
		{
			var target = Main.getEventTarget(event);
			var frm = target.form;
			var value = target.value;
			if (!validate || (validate && validate(frm, value)))
			{
				frm.submit();
			}
		}
		return true;
	}

	/*
		Submit the form that received the event.
		@param event The event object
		@param validate A function reference that validates the form
	*/
	this.submit = function(event, validate)
	{
		var target = Main.getEventTarget(event);
		var frm = target.form;
		var value = target.value;
		if (!validate || (validate && validate(frm, value)))
		{
			frm.submit();
		}
	}
	
	/*
		Check or unchecks all enabled checkboxes in a form. If the first checkbox that
		is found is checked they are unchecked, otherwise they are checked.
		The method only considers checkboxes having a name consisting only
		of digits.
		@param frm The form object
		@param nameRegexp A regular expression that must match the name of the checkbox,
			default matches names with digits
	*/
	this.checkUncheck = function(frm, nameRegexp)
	{
		var first = null;
		if (!nameRegexp) nameRegexp = /\d+/;
		for (var i=0; i < frm.elements.length; i++) // >
		{
			var element = frm.elements[i];
			if (element.type == 'checkbox' && !element.disabled && element.name.match(nameRegexp))
			{
				if (first == null)
				{
					first = element;
					element.checked = !element.checked;
				}
				else
				{
					element.checked = first.checked;
				}
			}
		}
	}
	
	/**
		Calculate the number of checked checkboxes in a form. The method only
		considers checkboxes having a name consisting only
		of digits. As a special case, if no checkboxes are found, then
		it checks if a radio button group with the name item_id exists and
		if one of those is checked.
		@param frm The form object
		@param nameRegexp A regular expression that must match the name of the checkbox,
			default matches names with digits
	*/
	this.numChecked = function(frm, nameRegexp)
	{
		var checked = 0;
		if (!nameRegexp) nameRegexp = /\d+/;
		for (var i=0; i < frm.elements.length; i++) // >
		{
			var element = frm.elements[i];
			if (element.type == 'checkbox' && element.name.match(nameRegexp) && element.checked)
			{
				checked++;
			}
		}
		if (checked == 0 && frm.item_id)
		{
			checked += Forms.getCheckedRadio(frm.item_id) ? 1 : 0;
		}
		return checked;
	}
	
	// If a form is hidden to begin with the text inside textarea tags
	// are lost if using the Konquerer browser.
	this.fixTextareaBug = function(id)
	{
		var frm = document.forms[id];
		for (i=0; i < frm.elements.length; i++)
		{
			if (frm.elements[i].type == 'textarea')
			{
				frm.elements[i].value = frm.elements[i].innerHTML;
			}
		}
	}
	
	/**
		Toggle the status of a checkbox.
		@param checkbox The checkbox object
	*/
	this.toggleCheckbox = function(checkbox)
	{
		checkbox.checked = !checkbox.checked;
	}
	
	/**
		Replaces an option in a selection list. If the specified index is after
		the last existing index, the option will be inserted last in the list.
		@param list The list as an Select object
		@param index The index of the new option
		@param option The Option object
	*/
	this.replaceListOption = function(list, index, option)
	{
		list[index] = option;
	}

	/**
		Adds a list option to a selection list. The option will be placed 
		before the option specified by the index value. Use -1 to insert the 
		option at the end.
		@param list The list as an Select object
		@param index The index of the new option
		@param option The Option object
	*/
	this.addListOption = function(list, index, option)
	{
		var before = (Browser.isIE) ? index : list[index];
		if (list.style.width) this.checkOptionText(list.style.width, option);
		list.add(option, before);
	}
	
	/**
		Check the length text attribute of a selection list option against
		the width of the selection list. If the text is longer than what
		can be displayed in the list, the text is automatically shortened
		and ... is appended. The full text is made available as a popup
		tooltip. If the text ends with a part inside brackets, [],
		that part is preserved, chopping is done on the text preceeding it.
		@param listWidth Width og the list including unit (em or px is supported)
		@param option The option that is beeing added to the list
	*/
	this.checkOptionText = function(listWidth, option)
	{
		var width = listWidth.match(/(\d+)(em|px)/);
		if (width == null) return;
		var maxLength;
		if (width[2] == 'em')
		{
			maxLength = 1.2 * width[1] + 0.01 * width[1] * width[1];
		}
		else
		{
			maxLength = 0.1 * width[1];
		}
		var hellip = String.fromCharCode(8230);
		if (option.text && option.text.length > maxLength)
		{
			var text = option.text;
			option.title = text;
			var parts = text.match(/(.*)\[(.*)\]/);
			if (parts != null)
			{
				maxLength = maxLength - parts[2].length - 3;
				if (maxLength < 4) maxLength = 4;
				text = parts[1].substring(0, maxLength) + hellip + ' [' + parts[2] + ']';
			}
			else
			{
				text = text.substring(0, maxLength) + hellip;
			}
			option.text = text;
		}
	}
	
	/**
		Move all selected options in the list up or down. Returns the number
		of options that were selected (and moved).
		@param list The list as an Select object
		@param up A boolean value 
	*/
	this.moveListOptions = function(list, up)
	{
		var moved = 0;
		if (list.options.length > 0)
		{
			var index = 1;
			var end_condition = list.options.length;
			var delta = -1;
			if (up)
			{
				index = list.options.length - 2;
				end_condition = -1;
				delta = 1;
			}
			while (index != end_condition)
			{
				if (!list.options[index+delta].selected && list.options[index].selected)
				{
					moved++;
					this.switchListOptions(list, index, index+delta);
				}
				index = index-delta;
			}
		}
		return moved;
	}

	/**
		Switch place of two options in a list
		@param list The list object
		@param index1 The index of the first option
		@param index2 The index of the second option
	*/
	this.switchListOptions = function(list, index1, index2)
	{
		var opt1 = list.options[index1];
		var opt2 = list.options[index2];
		list.options[index1] = new Option();
		list.options[index2] = new Option();
		list.options[index1] = opt2;
		list.options[index2] = opt1;
	}
	
	/**
		Select the option in a list with the specified value. If
		the value is not found among the options the selection is left
		unchanged.
		@param list The list object
		@param value The value to look for
	*/
	this.selectListOption = function(list, value)
	{
		for (var i = 0; i < list.length; i++)
		{
			if (list[i].value == value)
			{
				list.selectedIndex = i;
				return;
			}
		}
	}

	/**
		Select all options in a list which has a value among one of the specified 
		values. Values not found are deselected.
		@param list The list object
		@param values An array with the values
		@return The number of selected values in the list
	*/
	this.selectListOptions = function(list, values)
	{
		var numSelected = 0;
		for (var i = 0; i < list.length; i++)
		{
			var selected = false;
			for (var j = 0; j < values.length; j++)
			{
				if (list[i].value == values[j]) 
				{
					selected = true;
					j = values.length;
					numSelected++;
				}
			}
			list[i].selected = selected;
		}
		return numSelected;
	}
	
	/**
		Remove all options that are selected in a list
		@param list The list object
	*/
	this.removeSelected = function(list)
	{
		for (var i = 0; i < list.length; i++)
		{
			if (list[i].selected)
			{
				list[i--] = null;
			}
		}
	}
	
	
	/**
		Get the checked radio button element in a group of radio buttons. Returns
		null if no radio button is selected.
		@param radio The radio buttons group object
	*/
	this.getCheckedRadio = function(radio)
	{
		if (radio.checked)
		{
			// If there is only one radio button
			return radio;
		}
		for (var i = 0;  i < radio.length; i++)
		{
			if (radio[i].checked) return radio[i];
		}
		return null;
	}
	
	/**
		Check the radio button with the specified value. If no
		radio button has a matching value all buttons are unchecked. Returns
		the index of the checked radio button. Radio buttons that have
		been disabled are never checked.
		@param radio The radio buttons group object
		@param value The value of the radio button that should be checked
	*/
	this.checkRadio = function(radio, value)
	{
		var returnValue = -1;
		if (radio.length)
		{
			// More than one item in the list
			for (var i = 0; i < radio.length; i++)
			{
				radio[i].checked = (radio[i].value == value) && !radio[i].disabled;
				returnValue = radio[i].checked ? i : returnValue;
			}
		}
		else
		{
			// Only a single item in the list
			radio.checked = radio.value == value && !radio.disabled;
			returnValue = 0;
		}
		return returnValue;
	}
	
	/**
		Check all checkboxes having a value matching one of the
		specified values. Buttons that don't matches any value
		are unchecked.
		@param checkBoxes The check box group
		@param value An array of values
		@return The number of checked boxes
	*/
	this.checkCheckBoxes = function(checkBoxes, values)
	{
		var numChecked = 0;
		for (var i = 0; i < checkBoxes.length; i++)
		{
			checkBoxes[i].checked = false;
			for (var j = 0; j < values.length; j++)
			{
				if (checkBoxes[i].value == values[j])
				{
					checkBoxes[i].checked = true;
					numChecked++;
					j = values.length;
				}
			}
		}
		return numChecked;
	}
	
	/**
		Create a hidden input element in a form.
		@param frm The form object
		@param name The name of the hidden element
		@param value The value of the hidden element
		@param doc The document to create the element in, if not given
			uses the global 'document' object
	*/
	this.createHidden = function(frm, name, value, doc)
	{
		if (!doc) doc = document;
		var hidden = doc.createElement('input');
		hidden.setAttribute('type', 'hidden');
		hidden.setAttribute('name', name);
		hidden.setAttribute('value', value);
		frm.appendChild(hidden);
		if (!Browser.autoAddFormElements)
		{
			if (!frm.elements[name])
			{
				frm.elements[name] = hidden;
			}
			else 
			{
				// alert('[createHidden] WARNING: Form element \'' + name + '\' already exists in form \'' + frm.name + '\'; skipping');
			}
		}
		//alert("createHidden: name="+name+"; value="+value);
	}
	
}


var Browser = new BrowserClass();
// Functions related to browser incompatibilities

function BrowserClass()
{
	this.isIE = navigator.appName == 'Microsoft Internet Explorer';
	this.autoAddFormElements = !this.isIE;
}

// Handle sections that can be shown/hidden and remembered by context settings
var HideableSection = new HideableSectionClass();
function HideableSectionClass()
{
	// Show/hide a section 
	this.toggle = function(sectionId)
	{
		var main = document.getElementById(sectionId);
		var icon = document.getElementById(sectionId+'.icon');
		var content = document.getElementById(sectionId+'.content');
		var visible = true;
		var showClass = main.attributes['showclass'];
		var hideClass = main.attributes['hideclass'];
		
		if (content.style.display == 'none')
		{
			Main.show(content.id);
			icon.src = getRoot() + '/images/hide_section.gif';
			visible = true;
			if (showClass) Main.addClass(main, showClass.value);
			if (hideClass) Main.removeClass(main, hideClass.value);
		}
		else
		{
			Main.hide(content.id);
			icon.src = getRoot() + '/images/show_section.gif';
			visible = false;
			if (showClass) Main.removeClass(main, showClass.value);
			if (hideClass) Main.addClass(main, hideClass.value);
		}
		return visible;
	}
	
	/**
		Show/hide a section and send an Ajax request to the server with
		the new status. If the browser doesn't support Ajax, this method is
		identical to toggle()
	*/
	this.toggleAndSendWithAjax = function(sectionId, itemType, subcontext)
	{
		var visible = this.toggle(sectionId);
		var request = Main.getAjaxRequest();
		if (request != null)
		{
			var url = getRoot()+'/common/store_show_hide_section.jsp?ID='+Main.getIdFromLocation();
			url += '&sectionId='+sectionId;
			url += '&itemType='+itemType;
			url += '&subcontext='+subcontext;
			url += '&visible='+(visible ? '1' : '0');
			request.open("GET", url, true);
			request.send(null);
		}
	}
}

var SmartEnum = new SmartEnumClass();
/*
	Handles "smart" input fields which can have an attached list of
	enumeration options. As the user types a popup-list is displayed
	with the matching options.
*/
function SmartEnumClass()
{
	// Configuration parameters
	this.initialDelay = 300;
	this.repeatDelay = 50;
	this.leftOffset = Browser.isIE ? -2 : -1;
	this.topOffset = Browser.isIE ? 1 : 2;
	this.widthOffset = 15;

	this.activeField = null;
	this.selectedIndex = -1;
	this.lastSelectedEnum = null;
	this.lastMatchedText = null;

	/*
		Initialise the smart enum system by creating the <div> tag that
		is used to hold the options.
	*/
	this.init = function()
	{
		if (this.div) return;
		this.div = document.createElement('div');
		var div = this.div;
		div.className = 'smartenum';
		div.style.display = 'none';
		div.onmouseenter = function() {SmartEnum.disableHide = true; };
		div.onmouseleave = function() {SmartEnum.disableHide = false; };
		document.body.appendChild(div);
	}

	/*
		Enable the input field for smart enum. Ignored if
		the form or field doesn't exists, the field is not an
		input field or if there are no options. This method
		changes event handlers for the input field.
		@param frmName The name of the form 
		@param fieldName The name of the field
		@param An array object with the options
	*/
	this.enableForField = function(frmName, fieldName, options)
	{
		if (!options) return;
		var frm = document.forms[frmName];
		if (!frm) return;
		var field = frm[fieldName];
		if (!field) return;

		// Sort the options in alphabetical order, ignoring case
		field.smartOptions = options.sort(Strings.compareIgnoreCase);
		field.setAttribute('autocomplete', 'off');
		Main.addClass(field, 'smartinput');
		field.onfocus = this.onfocus;
		field.onkeyup = this.onkeyup;
		field.onkeydown = this.onkeydown;
		field.onblur = this.onblur;
	}

	/*
		Activate and display the enum options for a field.
	*/
	this.activateAndDisplay = function(formName, fieldName)
	{
		var frm = document.forms[formName];
		if (!frm) return;
		var field = frm[fieldName];
		if (!field) return;
		
		// We need to disable the onfocus event to make sure that
		// methods are called in the correct order. In IE. the
		// onfocus() event is called last, but in Firefox it is called
		// immediately after the call to focus().
		this.ignoreNextOnFocus = true;
		field.focus();
		this.hideOptions();
		this.setActive(field);
		this.filterOptions(true);
		this.showOptions();
	}

	/*
		Set a field as the active field. All other operations will
		now use this field. Set to null to disable all smart enum.
		Ignored if the field doesn't have any options. This method
		also positions the <div> with the options.
	*/
	this.setActive = function(field)
	{
		if (field == this.activeField) return;
		this.activeField = field;
		this.lastMatchedText = null;
		if (!field || !field.smartOptions) return;
		var pos = Main.getElementPosition(this.activeField);
		if (!this.div) this.init();
		var div = this.div;
		div.innerHTML = '';
		div.style.top = (pos.top + this.activeField.offsetHeight + this.topOffset) + 'px';
		div.style.left = (pos.left + this.leftOffset) + 'px';
		div.style.width = (this.activeField.offsetWidth + this.widthOffset) + 'px';
	}

	/*
		Hide the enum options.
	*/
	this.hideOptions = function()
	{
		if (!this.div) return;
		this.div.style.display = 'none';
		this.lastSelectedEnum = null;
	}
	
	/*
		Hide the enum options after a short delay. The delay is needed
		in so a possible onClick event can get sent to the currently
		selected option.
	*/
	this.hideOptionsDelayed = function()
	{
		if (this.disableHide) return;
		this.hideTimerId = setTimeout("SmartEnum.hideOptions();", 100);
	}

	/*
		Show the enum options setting the correct position.
	*/
	this.showOptions = function()
	{
		this.cancelHideOptions();
		if (!this.div) this.init();
		if (this.div.style.display == 'block') return;
		this.div.style.display = 'block';
		this.div.scrollTop = 0;
	}

	/*
		Filter the enum options by the text in the active field.
		The options that passed the filter are added as <div> tags
		to the enum <div>.
		@return TRUE if the filter generated any options, FALSE otherwise
	*/
	this.filterOptions = function(allowEmpty)
	{
		if (!this.activeField) return false;
		var text = this.activeField.value.replace(/^[=!<>]*/, '');
		if (text == this.lastMatchedText) return true;
		if (text == '' && !allowEmpty) return false;
		
		var options = this.activeField.smartOptions;
		this.lastMatchedText = text;
		this.selectedIndex = -1;
		this.lastSelectedEnum = null;
		var matching = '';
		var index = 0;
		for (var i = 0; i < options.length; i++)
		{
			if (text == '' || options[i].indexOf(text) >= 0)
			{
				matching += '<div class="smartoption" id="smartoption.' + index + '"';
				matching += ' onmouseover="SmartEnum.setSelectedEnum(' + index + ')"'; 
				matching += ' onclick="SmartEnum.submitSelected()">' + options[i] + '</div>';
				index++;
			}
		}
		if (!this.div) this.init();
		this.div.innerHTML = matching;
		return true;
	}

	/*
		Mark a specific enum option as selected. Also makes sure
		that the selected option is within the viewable area.
		@param index The index of the option
	*/
	this.setSelectedEnum = function(index)
	{
		if (this.lastSelectedEnum)
		{
			Main.removeClass(this.lastSelectedEnum, 'selected');
			this.lastSelectedEnum = null;
		}
		var option = document.getElementById('smartoption.' + index);
		if (option)
		{
			Main.addClass(option, 'selected');
			this.lastSelectedEnum = option;
			this.selectedIndex = index;
			var optionTop = option.offsetTop;
			var optionBottom = optionTop + option.offsetHeight;
			var parent = option.offsetParent;
			var parentTop = parent.scrollTop;
			var parentBottom = parentTop + parent.offsetHeight;
			if (optionTop < parentTop)
			{
				parent.scrollTop = optionTop;
			}
			else if (optionBottom > parentBottom)
			{
				parent.scrollTop = optionBottom - parent.offsetHeight + 2;
			}
		}
	}

	/*
		Moves the selection up or down, optionally
		repeating the operation at specified time intervals.
		@param delta The number of options to shift the selection
		@param delayRepeat Number of milliseconds to wait until
			the move is repeated for the first time
		@param repeat Number of milliseconds to wait between
			subsequent repeats
	*/
	this.moveSelection = function(delta, delayRepeat, repeat)
	{
		var option = document.getElementById('smartoption.' + (this.selectedIndex + delta));
		if (option)
		{
			this.setSelectedEnum(this.selectedIndex + delta);
		}
		if (delayRepeat && repeat)
		{
			var call = 'SmartEnum.moveSelection(' + delta + ',' + repeat + ',' + repeat + ')';
			if (this.timerId) clearTimeout(this.timerId);
			this.timerId = setTimeout(call, delayRepeat);
		}
	}

	/*
		Copies the selected option to the active field and submits
		the form. If no option is selected, just submit the form.
	*/
	this.submitSelected = function(event)
	{
		if (!this.activeField) return;
		if (this.lastSelectedEnum)
		{
			var text = this.activeField.value;
			var selectedText = this.lastSelectedEnum.innerHTML;
			this.activeField.value = text.replace(/^([=!<>]*).*/, '$1' + selectedText);
		}
		this.hideOptions();
		this.activeField.onkeyup = null;
		this.activeField.form.submit();
	}

	/*
		Cancels a move selection repeat set in the 'moveSelection' method.
	*/
	this.cancelKeyRepeat = function()
	{
		if (this.timerId)
		{
			clearTimeout(this.timerId);
			this.timerId = null;
		}
	}
	
	/*
		Cancels a scheduled hide of the dropdown enum options.
	*/
	this.cancelHideOptions = function()
	{
		if (this.hideTimerId) 
		{
			clearTimeout(this.hideTimerId);
			this.hideTimerId = null;
		}
	}

	/*
		Called when a smart-enum enabled field receivs
		the focus. This should set the current field to
		the active field.
	*/
	this.onfocus = function(event)
	{
		if (SmartEnum.ignoreNextOnFocus)
		{
			SmartEnum.ignoreNextOnFocus = false;
			return;
		}
		if (!event) event = window.event;
		var field = Main.getEventTarget(event);
		SmartEnum.hideOptions();
		SmartEnum.setActive(field);
	}
	
	/*
		Called when a smart-enum enabled field loses
		the focus. This should hide the options and
		set the current field to null.
	*/
	this.onblur = function(event)
	{
		if (!event) event = window.event;
		SmartEnum.hideOptionsDelayed();
	}
	
	/*
		Called when a key is pressed in a smart-enum enabled field.
		This function detects:
		* up/down arrow: moves the selected option
		* enter: submits the form
	*/
	this.onkeydown = function(event)
	{
		if (!event) event = window.event;
		var keyCode = Main.getEventKeyCode(event);
		if (keyCode == 38)
		{
			// Up arrow
			SmartEnum.moveSelection(-1, SmartEnum.initialDelay, SmartEnum.repeatDelay);
		}
		else if (keyCode == 40)
		{
			// Down arrow
			SmartEnum.moveSelection(1, SmartEnum.initialDelay, SmartEnum.repeatDelay);
		}
		else if (keyCode == 13)
		{
			// Enter
			SmartEnum.submitSelected(event);
		}
	}
	
	/*
		Called when a key is released in a smart-enum enabled field. 
		This function detects changes to the text and displays matching
		options.
	*/
	this.onkeyup = function(event)
	{
		if (!event) event = window.event;
		SmartEnum.cancelKeyRepeat();
		var keyCode = Main.getEventKeyCode(event);
		if (SmartEnum.filterOptions(keyCode == 40 || keyCode == 38))
		{
			SmartEnum.showOptions();
		}
		else
		{
			SmartEnum.hideOptions();
		}
	}
}

var MultiOptions = new MultiOptionsClass();
function MultiOptionsClass()
{
	// Configuration parameters
	this.initialDelay = 300;
	this.repeatDelay = 50;
	this.leftOffset = Browser.isIE ? -2 : -1;
	this.topOffset = Browser.isIE ? 1 : 2;
	this.widthOffset = 15;

	this.activeField = null;
	this.selectedIndex = -1;

	/*
		Initialise the multi-options system by creating the <div> tag that
		is used to hold the options.
	*/
	this.init = function()
	{
		if (this.div) return;
		this.div = document.createElement('div');
		var div = this.div;
		div.className = 'multioptions';
		div.style.display = 'none';
		if (div.addEventListener)
		{
			// Mozilla
			div.addEventListener('mouseover', function() {MultiOptions.disableHide = true;}, false);
			div.addEventListener('mouseout', function() {MultiOptions.disableHide = false;}, false);
			div.addEventListener('click', function() {MultiOptions.activeField.focus();}, false);
		}
		else
		{
			// IE.
			div.onmouseover = function() {MultiOptions.disableHide = true;};
			div.onmouseout = function() {MultiOptions.disableHide = false;};
			div.onclick = function() {MultiOptions.activeField.focus();};
		}
		document.body.appendChild(div);
	}

	/*
		Enable the input field for multi-options. Ignored if
		the form or field doesn't exists, the field is not an
		input field or if there are no options. This method
		changes event handlers for the input field.
		@param frmName The name of the form 
		@param fieldName The name of the field
		@param hiddenName The name of a hidden field which will hold the key values
		@param multiple If it is allowed to select multiple options or only a single
			one
		@param An array of object with three properties:
			- key: The value to use in the query
			- value: The value to display for the user
			- selected: If the option is selected or not
	*/
	this.enableForField = function(frmName, fieldName, hiddenName, multiple, options)
	{
		if (!options) return;
		var frm = document.forms[frmName];
		if (!frm) return;
		var field = frm[fieldName];
		if (!field) return;
		var hidden = frm[hiddenName];
		if (!hidden) return;

		field.multiOptions = options;
		field.allowMultiple = multiple;
		field.hiddenField = hidden;
		field.neq = hidden.value.indexOf('<>') == 0;
		field.setAttribute('autocomplete', 'off');
		field.onfocus = this.onfocus;
		field.onkeypress = this.onkeypress;
		field.onkeyup = this.onkeyup;
		field.onkeydown = this.onkeydown;
		field.onblur = this.onblur;
		
		this.generateFilter(field);
	}
	
	/*
		Activate and display the mulit-options for a field.
	*/
	this.activateAndDisplay = function(formName, fieldName)
	{
		var frm = document.forms[formName];
		if (!frm) return;
		var field = frm[fieldName];
		if (!field) return;
		
		if (this.activeField != field)
		{
			this.hideOptions();
			this.setActive(field);
			this.showOptions();
			field.focus();
		}
		else
		{
			this.hideOptions();
			this.activeField = null;
		}
	}
	
	/*
		Create the filter string and store it in the hidden field.
	*/
	this.generateFilter = function(field)
	{
		var keys = new Array();
		var values = new Array();
		var numSelected = 0;
		for (var i = 0; i < field.multiOptions.length; i++)
		{
			var option = field.multiOptions[i];
			if (option.selected)
			{
				keys[keys.length] = option.key;
				values[values.length] = option.value;
				numSelected++;
			}
		}
		var prefix = field.neq && values.length > 0 ? '<>' : '';
		field.value = prefix + values.join('|');
		field.hiddenField.value = prefix + keys.join('|');
		if (values.length > 0 && field.hiddenField.value == '')
		{
			field.hiddenField.value = '=';
		}
		field.numSelected = numSelected;
		var info = document.getElementById('multioptions.info');
		if (info) info.innerHTML = field.numSelected + ' selected';
	} 
	
	/*
		Hide the options.
	*/
	this.hideOptions = function()
	{
		if (!this.div) return;
		this.div.style.display = 'none';
	}
	
	/*
		Hide the multi-options after a short delay. The delay is needed
		so a possible onClick event can get sent to the currently
		selected option.
	*/
	this.hideOptionsDelayed = function()
	{
		if (this.disableHide) return;
		this.hideTimerId = setTimeout("MultiOptions.hideOptions();", 100);
	}
	
	/*
		Cancels a scheduled hide of the dropdown enum options.
	*/
	this.cancelHideOptions = function()
	{
		if (this.hideTimerId) 
		{
			clearTimeout(this.hideTimerId);
			this.hideTimerId = null;
		}
	}
	
	/*
		Cancels a move selection repeat set in the 'moveSelection' method.
	*/
	this.cancelKeyRepeat = function()
	{
		if (this.timerId)
		{
			clearTimeout(this.timerId);
			this.timerId = null;
		}
	}

	
	/*
		Show the multi-options.
	*/
	this.showOptions = function()
	{
		this.cancelHideOptions();
		if (!this.div) this.init();
		if (this.div.style.display == 'block') return;
		this.div.style.display = 'block';
		this.div.scrollTop = 0;
	}

	/*
		Set a field as the active field. All other operations will
		now use this field. Set to null to disable all multi-options.
		Ignored if the field doesn't have any options. This method
		also positions the <div> with the options.
	*/
	this.setActive = function(field)
	{
		if (field == this.activeField) return;
		this.activeField = field;
		if (!field || !field.multiOptions) return;
		var pos = Main.getElementPosition(this.activeField);
		if (!this.div) this.init();
		var div = this.div;
		div.style.top = (pos.top + this.activeField.offsetHeight + this.topOffset) + 'px';
		div.style.left = (pos.left + this.leftOffset) + 'px';
		div.style.width = (this.activeField.offsetWidth + this.widthOffset) + 'px';
		
		var options = this.activeField.multiOptions;
		this.activeField.currentlySelected = -1;
		var multiple = this.activeField.allowMultiple;
		var imgRoot = getRoot() + 'images/';
		var text = '<div class="multioptioncontrol">';
		text += '<table width="100%" border="0" cellspacing="0" cellpadding="0"><tr>';
		if (multiple)
		{
			text += '<td><img class="link" src="' + imgRoot + 'check_uncheck.gif"';
			text += ' title="Check/uncheck all" onclick="MultiOptions.checkUncheck()"></td>';
			text += '<td id="multioptions.info">' + field.numSelected + ' selected</td>';
		}
		else
		{
			text += '<td><input type="checkbox" name="neq"';
			if (this.activeField.neq) text += ' checked';
			text += ' onclick="MultiOptions.notEqualOnClick(this)">&lt;&gt;</td>';
		}
		text += '<td class="link" style="text-align: right;"';
		text += ' title="Perform search" onclick="MultiOptions.activeField.form.submit();">';
		text += '<img src="' + imgRoot + 'ok.gif"></td>';
		text += '</tr></table>';
		text += '</div>';
		text += '<div class="multioptionoptions">';
		for (var i = 0; i < options.length; i++)
		{
			var icon = multiple ? 'option_unselected.gif' : 'option_single_unselected.gif';
			if (options[i].selected)
			{
				if (multiple)
				{
					icon = 'option_selected.gif';
				}
				else
				{
					if (this.activeField.currentlySelected >= 0)
					{
						options[i].selected = false;
					}
					else
					{
						icon = 'option_single_selected.gif';
						this.activeField.currentlySelected = i;
					}
				}
			}
			text += '<div class="multioption" id="multioption.' + i + '"';
			text += ' onmouseover="MultiOptions.highlightOption(' + i + ')"'; 
			text += ' onclick="MultiOptions.toggleSelected(' + i + ', false)">';
			text += '<img id="multioption.' + i + '.img" src="' + imgRoot + icon + '">';
			text += options[i].value + '</div>';
		}
		text += '</div>';
		this.div.innerHTML = text;
		this.selectedIndex = -1;
	}

	/*
		Highlight a specific option. Also makes sure
		that the highlighted option is within the viewable area.
		@param index The index of the option to highlight
	*/
	this.highlightOption = function(index)
	{
		if (this.lastSelectedOption)
		{
			Main.removeClass(this.lastSelectedOption, 'selected');
			this.lastSelectedOption = null;
		}
		var option = document.getElementById('multioption.' + index);
		if (option)
		{
			Main.addClass(option, 'selected');
			this.lastSelectedOption = option;
			this.selectedIndex = index;
			var optionTop = option.offsetTop;
			var optionBottom = optionTop + option.offsetHeight;
			var parent = option.offsetParent;
			var parentTop = parent.scrollTop;
			var parentBottom = parentTop + parent.offsetHeight;
			if (optionTop < parentTop)
			{
				parent.scrollTop = optionTop;
			}
			else if (optionBottom > parentBottom)
			{
				parent.scrollTop = optionBottom - parent.offsetHeight + 2;
			}
		}
	}
	
	/*
		Moves the selection up or down, optionally
		repeating the operation at specified time intervals.
		@param delta The number of options to shift the selection
		@param delayRepeat Number of milliseconds to wait until
			the move is repeated for the first time
		@param repeat Number of milliseconds to wait between
			subsequent repeats
	*/
	this.moveSelection = function(delta, delayRepeat, repeat)
	{
		var option = document.getElementById('multioption.' + (this.selectedIndex + delta));
		if (option)
		{
			this.highlightOption(this.selectedIndex + delta);
		}
		if (delayRepeat && repeat)
		{
			var call = 'MultiOptions.moveSelection(' + delta + ',' + repeat + ',' + repeat + ')';
			if (this.timerId) clearTimeout(this.timerId);
			this.timerId = setTimeout(call, delayRepeat);
		}
	}

	/*
		Check or uncheck all options. The status of the first
		option is reversed and all other options are set to the
		same value.
	*/
	this.checkUncheck = function()
	{
		var options = this.activeField.multiOptions;
		var check = !options[0].selected;
		var icon = check ? 'option_selected.gif' : 'option_unselected.gif';
		for (var i = 0; i < options.length; i++)
		{
			options[i].selected = check;
			var img = document.getElementById('multioption.' + i + '.img');
			if (img) img.src = getRoot() + '/images/' + icon;
		}
		this.generateFilter(this.activeField);
	}

	this.notEqualOnClick = function(checkbox)
	{
		if (!this.activeField) return;
		this.activeField.neq = checkbox.checked;
		this.generateFilter(this.activeField);
	}

	/*
		Toggle the selected status of a given option
		@param index The index of the option to toggle
		@param submitForm If true, the form is submitted
	*/
	this.toggleSelected = function(index, submitForm)
	{
		var option = document.getElementById('multioption.' + index);
		if (option)
		{
			var options = this.activeField.multiOptions;
			var multiple = this.activeField.allowMultiple;
			if (!multiple)
			{
				// Deselect the currently selected item
				var selected = this.activeField.currentlySelected;
				if (selected >= 0 && selected != index)
				{
					options[selected].selected = false;
					var img = document.getElementById('multioption.' + selected + '.img');
					if (img) img.src = getRoot() + '/images/option_single_unselected.gif';
				}
				this.activeField.currentlySelected = selected != index ? index : -1;
			}
			options[index].selected = !options[index].selected;
			if (options[index].selected)
			{
				icon = multiple ? 'option_selected.gif' : 'option_single_selected.gif';
			}
			else
			{
				icon = multiple ? 'option_unselected.gif' : 'option_single_unselected.gif';
			}
			var img = document.getElementById('multioption.' + index + '.img');
			if (img) img.src = getRoot() + '/images/' + icon;
			this.generateFilter(this.activeField);
			if (submitForm) 
			{
				this.activeField.form.submit();
			}
		}
	}
	
	/*
		Called when a multi-option enabled field receivs
		the focus. This should set the current field to
		the active field.
	*/
	this.onfocus = function(event)
	{
		if (MultiOptions.ignoreNextOnFocus)
		{
			MultiOptions.ignoreNextOnFocus = false;
			return;
		}
		if (!event) event = window.event;
		var field = Main.getEventTarget(event);
		MultiOptions.hideOptions();
		MultiOptions.setActive(field);
		MultiOptions.showOptions();
	}
	
	/*
		Called when a multi-options enabled field loses
		the focus. This should hide the options and
		set the current field to null.
	*/
	this.onblur = function(event)
	{
		if (!event) event = window.event;
		MultiOptions.hideOptionsDelayed();
	}
	
	/*
		Called when a key is pressed in a multi-options enabled field.
		This function detects:
		* up/down arrow: moves the selected option
		* space: toggles the selected status of the current option
		* enter: toggles the current option and submits the form
	*/
	this.onkeydown = function(event)
	{
		if (!event) event = window.event;
		var keyCode = Main.getEventKeyCode(event);
		if (keyCode == 38)
		{
			// Up arrow
			MultiOptions.moveSelection(-1, MultiOptions.initialDelay, MultiOptions.repeatDelay);
		}
		else if (keyCode == 40)
		{
			// Down arrow
			MultiOptions.moveSelection(1, MultiOptions.initialDelay, MultiOptions.repeatDelay);
		}
		else if (keyCode ==32)
		{
			// Space
			MultiOptions.toggleSelected(MultiOptions.selectedIndex, false);
		}
		else if (keyCode == 13)
		{
			// Enter
			MultiOptions.toggleSelected(MultiOptions.selectedIndex, true);
		}
	}
	/*
		Called when a key is released in a multi-options enabled field. 
		This function detects:
		* up/down arrow: show the options
	*/
	this.onkeyup = function(event)
	{
		if (!event) event = window.event;
		MultiOptions.cancelKeyRepeat();
		var keyCode = Main.getEventKeyCode(event);
		if (keyCode == 40 || keyCode == 38)
		{
			MultiOptions.showOptions();
		}
	}
	
	/*
		Called when a key is pressed in a multi-options enabled field. 
		This function disabled all keys except the tab key.
	*/
	this.onkeypress = function(event)
	{
		if (!event) event = window.event;
		var charCode = Main.getEventCharCode(event);
		// Ignore all but tabs
		return charCode == 9;
	}
}

function TextOverflowChecker(tagName, className)
{
	this.nextTag = 0;
	
	this.checkAll = function()
	{
		this.tags = Main.getElementsByTagAndClassName(tagName, className);
		for (var i = 0; i < this.tags.length; i++)
		{
			this.checkTag(this.tags[i]);
		}
		for (var i = 0; i < this.tags.length; i++)
		{
			this.fixTag(this.tags[i]);
		}
	}
	
	this.checkTag = function(tag)
	{
		var isWidthOverflowed = tag.clientWidth < tag.scrollWidth;
		var isHeightOverflowed = tag.clientHeight +5 < tag.scrollHeight;
		//alert("checkTag: " + tag + ':' + ':' + tag.clientWidth + ':' + tag.scrollWidth + ':' + tag.clientHeight + ':' + tag.scrollHeight);
		tag.isWidthOverflowed = isWidthOverflowed;
		tag.isHeightOverflowed = isHeightOverflowed;
		tag.isOverflowed = isWidthOverflowed || isHeightOverflowed;
	}
	
	this.fixTag = function(tag)
	{
		var className = 'constrained';
		if (tag.isOverflowed)
		{
			className += ' overflowed';
			var self = this;
			var onClickListener = function() { self.showOverflowedText(tag); };
			var useClickEvent = false;
			if (tag.addEventListener)
			{
				// Firefox and compatible
				tag.addEventListener('click', onClickListener, false);
				useClickEvent = true;
			}
			else if (tag.attachEvent)
			{
				// IE
				tag.attachEvent('onclick', onClickListener);
				useClickEvent = true;
			}
			if (useClickEvent)
			{
				className += ' manualshow';
				tag.title = 'Click to show/hide the complete text!';
			}
			else
			{
				className += ' autoshow';
			}
		}
		tag.className = className;
	}

	this.showOverflowedText = function(tag)
	{
		if (tag.isWidthOverflowed)
		{
			Main.toggleClass(tag, 'constrained');
		}
		else
		{
			Main.toggleClass(tag, 'shown');
		}
	}
}


